


#include "HIVE_CppPlugin.h"


NS_HIVE_BEGIN


static Auth::onAuthInitialize g_onAuthInitialize = NULL;

static Auth::onAuthLogin g_onAuthLogin = NULL;

static Auth::onAuthLogin g_onAuthShowLoginSelection = NULL;

static Auth::onAuthLogin g_onAuthBindLogin = NULL;

static Auth::onAuthLogout g_onAuthLogout = NULL;

static Auth::onAuthShowTerms g_onAuthShowTerms = NULL;

static Auth::onAuthMaintenance g_onAuthMaintenance = NULL;

static Auth::onAuthAdultConfirm g_onAuthAdultConfirm = NULL;

static Auth::onAuthRequestPermissionViewData g_onAuthRequestPermissionViewData = NULL;


static std::string LoginTypeToString(LoginType loginType)
{
	if(loginType == LoginType::ACCOUNT)
		return "ACCOUNT";
	else if(loginType == LoginType::SELECT)
		return "SELECT";
	else if(loginType == LoginType::AUTO)
		return "AUTO";
	else
		return "GUEST";
}


static LoginType StringToLoginType(std::string loginTypeString)
{
    if(loginTypeString.compare("ACCOUNT") == 0)
    	return LoginType::ACCOUNT;
    else if(loginTypeString.compare("SELECT") == 0)
    	return LoginType::SELECT;
    else if(loginTypeString.compare("AUTO") == 0)
    	return LoginType::AUTO;
    else
    	return LoginType::GUEST;
}


void Auth::initialize(onAuthInitialize listener)
{
	g_onAuthInitialize = listener;


	picojson::object jsonParam = HiveCppPlugin::createParam("Auth", "initialize");

	HiveCppPlugin::callNative(jsonParam);
}


LoginType Auth::getLoginType()
{
	picojson::object jsonParam = HiveCppPlugin::createParam("Auth", "getLoginType");

	picojson::value resJson = HiveCppPlugin::callNative(jsonParam);
    
    std::string loginTypeString;
    picojson::value jsonValue = resJson.get("getLoginType");
    if(jsonValue.is<picojson::null>() == false)
        loginTypeString = jsonValue.to_str();

    return StringToLoginType(loginTypeString);
}


void Auth::login(LoginType loginType, onAuthLogin listener)
{
	g_onAuthLogin = listener;


	picojson::object jsonParam = HiveCppPlugin::createParam("Auth", "login");

	std::string loginTypeString = LoginTypeToString(loginType);
	jsonParam["loginType"] = picojson::value(loginTypeString);

	HiveCppPlugin::callNative(jsonParam);
}


void Auth::showLoginSelection(picojson::object currentData, picojson::object selectData, onAuthLogin listener)
{
	g_onAuthShowLoginSelection = listener;


	picojson::object jsonParam = HiveCppPlugin::createParam("Auth", "showLoginSelection");
	jsonParam["currentData"] = picojson::value(currentData);
	jsonParam["selectData"] = picojson::value(selectData);

	HiveCppPlugin::callNative(jsonParam);
}


void Auth::bindLogin(std::string selectedVid, onAuthLogin listener)
{
    g_onAuthBindLogin = listener;
    
    
    picojson::object jsonParam = HiveCppPlugin::createParam("Auth", "bindLogin");
    jsonParam["selectedVid"] = picojson::value(selectedVid);
    
    HiveCppPlugin::callNative(jsonParam);
}


void Auth::logout(onAuthLogout listener)
{
	g_onAuthLogout = listener;


	picojson::object jsonParam = HiveCppPlugin::createParam("Auth", "logout");

	HiveCppPlugin::callNative(jsonParam);
}


Account Auth::getAccount()
{
	picojson::object jsonParam = HiveCppPlugin::createParam("Auth", "getAccount");

	picojson::value resJson = HiveCppPlugin::callNative(jsonParam);

	Account account(resJson.get("getAccount"));
	return account;
}

void Auth::showTerms(onAuthShowTerms listener)
{
	g_onAuthShowTerms = listener;


	picojson::object jsonParam = HiveCppPlugin::createParam("Auth", "showTerms");

	HiveCppPlugin::callNative(jsonParam);
}


void Auth::checkMaintenance(bool isShow, onAuthMaintenance listener)
{
    g_onAuthMaintenance = listener;
    
    picojson::object jsonParam = HiveCppPlugin::createParam("Auth", "checkMaintenance");
    
    jsonParam["isShow"] = picojson::value(isShow);
    
    HiveCppPlugin::callNative(jsonParam);
}


void Auth::showAdultConfirm(onAuthAdultConfirm listener)
{
    g_onAuthAdultConfirm = listener;
    
    picojson::object jsonParam = HiveCppPlugin::createParam("Auth", "showAdultConfirm");
    
    HiveCppPlugin::callNative(jsonParam);
}



void Auth::reset()
{
	picojson::object jsonParam = HiveCppPlugin::createParam("Auth", "reset");

	HiveCppPlugin::callNative(jsonParam);
}

void Auth::requestPermissionViewData(onAuthRequestPermissionViewData listener) {
    g_onAuthRequestPermissionViewData = listener;
    picojson::object jsonParam = HiveCppPlugin::createParam("Auth", "requestPermissionViewData");
    HiveCppPlugin::callNative(jsonParam);
}



bool Auth::setEmergencyMode()
{
    picojson::object jsonParam = HiveCppPlugin::createParam("Auth", "setEmergencyMode");
    
    picojson::value resJson = HiveCppPlugin::callNative(jsonParam);
    
    bool bIsEmergencyMode = false;
    picojson::value jsonValue = resJson.get("isEmergencyMode");
    if(jsonValue.is<picojson::null>() == false)
        bIsEmergencyMode = jsonValue.evaluate_as_boolean();
    
    return bIsEmergencyMode;
}
    
    
void Auth::executeEngine(picojson::value jsonParam)
{
	ResultAPI result(jsonParam.get("resultAPI"));


	picojson::value jsonMethodValue;
	jsonMethodValue = jsonParam.get("method");
	if(jsonMethodValue.is<picojson::null>()) {
		return;
	}

	std::string methodName = jsonMethodValue.to_str();
	if(methodName.compare("initialize") == 0)
	{
		if(g_onAuthInitialize != NULL)
		{
            AuthInitResult authInitResult(jsonParam.get("authInitResult"));
            g_onAuthInitialize(result, authInitResult);
		}
	}
	else if(methodName.compare("login") == 0)
	{
		if(g_onAuthLogin != NULL)
		{
            picojson::value jsonValue;

            std::string strValue;
            jsonValue = jsonParam.get("loginType");
            if(jsonValue.is<picojson::null>() == false)
                strValue = jsonValue.to_str();
			LoginType loginType = StringToLoginType(strValue);

            Account currentAccount(jsonParam.get("currentAccount"));
            Account usedAccount(jsonParam.get("usedAccount"));
            
            //try
            //{
                g_onAuthLogin(result, loginType, currentAccount, usedAccount);
            //}
            //catch(const std::bad_function_call& e) {
            //    std::cout << e.what() << '\n';
            //}
            
		}
	}
	else if(methodName.compare("showLoginSelection") == 0)
	{
		if(g_onAuthShowLoginSelection != NULL)
		{
			picojson::value jsonValue;

            std::string strValue;
            jsonValue = jsonParam.get("loginType");
            if(jsonValue.is<picojson::null>() == false)
                strValue = jsonValue.to_str();
			LoginType loginType = StringToLoginType(strValue);

            Account currentAccount(jsonParam.get("currentAccount"));
            Account usedAccount(jsonParam.get("usedAccount"));

            g_onAuthShowLoginSelection(result, loginType, currentAccount, usedAccount);
		}
	}
    else if(methodName.compare("bindLogin") == 0)
    {
        if(g_onAuthBindLogin != NULL)
        {
            picojson::value jsonValue;
            
            std::string strValue;
            jsonValue = jsonParam.get("loginType");
            if(jsonValue.is<picojson::null>() == false)
                strValue = jsonValue.to_str();
            LoginType loginType = StringToLoginType(strValue);
            
            Account currentAccount(jsonParam.get("currentAccount"));
            Account usedAccount(jsonParam.get("usedAccount"));
            
            g_onAuthBindLogin(result, loginType, currentAccount, usedAccount);            
        }
    }
	else if(methodName.compare("logout") == 0)
	{
		if(g_onAuthLogout != NULL)
		{
            g_onAuthLogout(result);
		}
	}
	else if(methodName.compare("showTerms") == 0)
	{
		if(g_onAuthShowTerms != NULL)
		{
            g_onAuthShowTerms(result);
		}
	}
    else if(methodName.compare("checkMaintenance") == 0)
    {
        if(g_onAuthMaintenance != NULL)
        {
            AuthMaintenanceInfo authMaintenanceInfo(jsonParam.get("authMaintenanceInfo"));
            g_onAuthMaintenance(result, authMaintenanceInfo);
        }
    }
    else if(methodName.compare("showAdultConfirm") == 0)
    {
        if(g_onAuthAdultConfirm != NULL)
        {
            g_onAuthAdultConfirm(result);
        }
    }
    else if(methodName.compare("requestPermissionViewData") == 0)
    {
        if(g_onAuthRequestPermissionViewData != NULL)
        {
            g_onAuthRequestPermissionViewData(result,PermissionViewData(jsonParam.get("data")));
        }
    }
}




// Account class Implements
Account::Account()
{
}


Account::Account(picojson::value jsonParam)
{
	if(jsonParam.is<picojson::null>())
	        return;

    picojson::value jsonValue;
    
    jsonValue = jsonParam.get("vid");
    if(jsonValue.is<picojson::null>() == false)
        this->vid = jsonValue.to_str();
    
    jsonValue = jsonParam.get("uid");
    if(jsonValue.is<picojson::null>() == false)
        this->uid = jsonValue.to_str();
    
    jsonValue = jsonParam.get("did");
    if(jsonValue.is<picojson::null>() == false)
        this->did = jsonValue.to_str();
    
    jsonValue = jsonParam.get("accessToken");
    if(jsonValue.is<picojson::null>() == false)
        this->accessToken = jsonValue.to_str();
}


picojson::object Account::toJson() const
{
	picojson::object resJson;
    
    resJson["vid"] = picojson::value(this->vid);
	resJson["uid"] = picojson::value(this->uid);
	resJson["did"] = picojson::value(this->did);
	resJson["accessToken"] = picojson::value(this->accessToken);
	return resJson;
}


std::string Account::toString() const
{
	picojson::object resJson = this->toJson();
	std::string resJsonString = picojson::value(resJson).serialize();
	return resJsonString;
}


// AuthInitResult class Implements
AuthInitResult::AuthInitResult()
{
	this->isAuthorized = false;
	this->loginType = LoginType::GUEST;
	this->isPGSLogin = false;
}


AuthInitResult::AuthInitResult(picojson::value jsonParam)
{
    if(jsonParam.is<picojson::null>())
        return;
    
    picojson::value jsonValue = jsonParam.get("isAuthorized");
    if(jsonValue.is<picojson::null>() == false)
        this->isAuthorized = jsonValue.get<bool>();

    std::string loginTypeString;
    jsonValue = jsonParam.get("loginType");
    if(jsonValue.is<picojson::null>() == false)
        loginTypeString = jsonValue.to_str();
	this->loginType = StringToLoginType(loginTypeString);

    jsonValue = jsonParam.get("did");
    if(jsonValue.is<picojson::null>() == false)
        this->did = jsonValue.to_str();

    jsonValue = jsonParam.get("isPGSLogin");
    if(jsonValue.is<picojson::null>() == false)
    	this->isPGSLogin = jsonValue.get<bool>();
    
    jsonValue = jsonParam.get("playerName");
    if(jsonValue.is<picojson::null>() == false)
        this->playerName = jsonValue.to_str();
    
    jsonValue = jsonParam.get("playerId");
    if(jsonValue.is<picojson::null>() == false)
        this->playerId = jsonValue.to_str();
}


picojson::object AuthInitResult::toJson() const
{
	picojson::object resJson;
	resJson["isAuthorized"] = picojson::value(this->isAuthorized);

	resJson["loginType"] = picojson::value(LoginTypeToString(this->loginType));
	resJson["did"] = picojson::value(this->did);
	resJson["isPGSLogin"] = picojson::value(this->isPGSLogin);
	resJson["playerName"] = picojson::value(this->playerName);
	resJson["playerId"] = picojson::value(this->playerId);

	return resJson;
}


std::string AuthInitResult::toString() const
{
	picojson::object resJson = this->toJson();
	std::string resJsonString = picojson::value(resJson).serialize();
	return resJsonString;
}




// AuthMaintenanceInfo class Implements
AuthMaintenanceInfo::AuthMaintenanceInfo()
{
}


AuthMaintenanceInfo::AuthMaintenanceInfo(picojson::value jsonParam)
{
    if(jsonParam.is<picojson::null>())
        return;
    
    picojson::value jsonValue;
    
    jsonValue = jsonParam.get("title");
    if(jsonValue.is<picojson::null>() == false)
        this->title = jsonValue.to_str();
    
    jsonValue = jsonParam.get("message");
    if(jsonValue.is<picojson::null>() == false)
        this->message = jsonValue.to_str();
    
    jsonValue = jsonParam.get("button");
    if(jsonValue.is<picojson::null>() == false)
        this->button = jsonValue.to_str();
    
    jsonValue = jsonParam.get("url");
    if(jsonValue.is<picojson::null>() == false)
        this->url = jsonValue.to_str();
    
    std::string strValue;
    jsonValue = jsonParam.get("action");
    if(jsonValue.is<picojson::null>() == false)
        strValue = jsonValue.to_str();
    
    if(strValue.compare("OPEN_URL") == 0)
        this->actionType = AuthMaintenanceActionType::OPEN_URL;
    else if(strValue.compare("OPEN_URL") == 0)
        this->actionType = AuthMaintenanceActionType::EXIT;
    else
        this->actionType = AuthMaintenanceActionType::DONE;
    
    jsonValue = jsonParam.get("remainingTime");
    if(jsonValue.is<picojson::null>() == false)
        this->remainingTime = jsonValue.get<double>();
}


picojson::object AuthMaintenanceInfo::toJson() const
{
    picojson::object resJson;
    
    resJson["title"] = picojson::value(this->title);
    resJson["message"] = picojson::value(this->message);
    resJson["button"] = picojson::value(this->button);
    resJson["url"] = picojson::value(this->url);
    
    if(this->actionType == AuthMaintenanceActionType::OPEN_URL)
        resJson["action"] = picojson::value("OPEN_URL");
    else if(this->actionType == AuthMaintenanceActionType::EXIT)
        resJson["action"] = picojson::value("EXIT");
    else
        resJson["action"] = picojson::value("DONE");
    
    resJson["remainingTime"] = picojson::value((double)this->remainingTime);
    return resJson;
}


std::string AuthMaintenanceInfo::toString() const
{
    picojson::object resJson = this->toJson();
    std::string resJsonString = picojson::value(resJson).serialize();
    return resJsonString;
}


NS_HIVE_END		// namespace hive



