#include "HIVE_CppPlugin.h"

#include <sstream>
#include <assert.h>

NS_HIVE_BEGIN

static AuthV4::onSetup g_Authv4SetupListener = nullptr;
static AuthV4::onSignIn g_Authv4SignInListener = nullptr;
static AuthV4::onSignOut g_Authv4SignOutListener = nullptr;
static AuthV4::onConnect g_Authv4ConnectListener = nullptr;
static AuthV4::onDisconnect g_Authv4DisconnectListener = nullptr;
static AuthV4::onSignIn g_Authv4SelectSignInListener = nullptr;
static AuthV4::onCheckProvider g_Authv4CheckProviderListener = nullptr;
static AuthV4::onMaintenance g_Authv4CheckMaintenanceListener = nullptr;
static AuthV4::onMaintenance g_Authv4CheckBlackListMaintenanceListener = nullptr;
static AuthV4::onCheckProvider g_Authv4SetProviderChangedHandler = nullptr;
static AuthV4::onSignIn g_Authv4ShowSignInListener = nullptr;
static AuthV4::onSignIn g_Authv4ConflictSelectionSignInListener = nullptr;
static AuthV4::onShowTerms g_Authv4ShowTermsListener = nullptr;
static AuthV4::onAdultConfirm g_Authv4ShowAdultConfirmListener = nullptr;
static AuthV4::onGetProfile g_Authv4GetProfileListener = nullptr;
static AuthV4::onShowProfile g_Authv4ShowProfileListener = nullptr;
static AuthV4::onShowCafe g_AuthV4ShowCafeListener = nullptr;
static AuthV4::onShowInquiry g_AuthV4ShowInquiryListener = nullptr;
static AuthV4::onGetProviderFriendsList g_AuthV4GetProviderFriendsListListener = nullptr;
static AuthV4::onResolveConflict g_AuthV4ResolveConflict = nullptr;
static AuthV4::onAuthV4DialogDismiss g_AuthV4DialogDismiss = nullptr;
static AuthV4::onAuthV4RequestPermissionViewData g_onAuthV4RequestPermissionViewData = nullptr;

//AuthV4 Helper Callbacks
static AuthV4::Helper::onAuthV4Helper g_AuthV4HelperSyncAccount = nullptr;
static AuthV4::Helper::onAuthV4Helper g_AuthV4HelperSignIn = nullptr;
static AuthV4::Helper::onAuthV4Helper g_AuthV4HelperSignOut = nullptr;
static AuthV4::Helper::onAuthV4Helper g_AuthV4HelperConnect = nullptr;
static AuthV4::Helper::onAuthV4Helper g_AuthV4HelperDisconnect = nullptr;
static AuthV4::Helper::onAuthV4Helper g_AuthV4HelperShowAchievements = nullptr;
static AuthV4::Helper::onAuthV4Helper g_AuthV4HelperShowLeaderboard = nullptr;
static AuthV4::Helper::onAuthV4Helper g_AuthV4HelperAchievementsReveal = nullptr;
static AuthV4::Helper::onAuthV4Helper g_AuthV4HelperAchievementsUnlock = nullptr;
static AuthV4::Helper::onAuthV4Helper g_AuthV4HelperAchievementsIncrement = nullptr;
static AuthV4::Helper::onAuthV4Helper g_AuthV4HelperLeaderBoardsSubmitScore = nullptr;
static AuthV4::Helper::onAuthV4Helper g_AuthV4HelperShowConflict = nullptr;
static AuthV4::Helper::onAuthV4Helper g_AuthV4HelperSwitchAccount = nullptr;
static AuthV4::Helper::onAuthV4Helper g_AuthV4HelperResolveConflict = nullptr;
static AuthV4::onAuthV4DialogDismiss g_AuthV4HelperDialogDismiss = nullptr;

static std::map<std::string,AuthV4::Helper::onAuthV4Helper> g_AuthV4HelperCallbacks;

const char* AUTHV4_FUNC_NAME_SETUP = "setup";
const char* AUTHV4_FUNC_NAME_SIGNIN = "signIn";
const char* AUTHV4_FUNC_NAME_SIGNOUT = "signOut";
const char* AUTHV4_FUNC_NAME_CONNECT = "connect";
const char* AUTHV4_FUNC_NAME_DISCONNECT = "disconnect";
const char* AUTHV4_FUNC_NAME_SELECTCONFLICT = "selectConflict";
const char* AUTHV4_FUNC_NAME_CHECKPROVIDER = "checkProvider";
const char* AUTHV4_FUNC_NAME_CHECKMAINTENANCE = "checkMaintenance";
const char* AUTHV4_FUNC_NAME_CHECKBLACKLIST = "checkBlacklist";
const char* AUTHV4_FUNC_NAME_SETPROVIDERCHANGEDLISTENER = "setProviderChangedListener";
const char* AUTHV4_FUNC_NAME_SHOWSIGNIN = "showSignIn";
const char* AUTHV4_FUNC_NAME_SHOWCONFLICTSELECTION = "showConflictSelection";
const char* AUTHV4_FUNC_NAME_SHOWCAFE = "showCafe";
const char* AUTHV4_FUNC_NAME_SHOWINQUIRY = "showInquiry";
const char* AUTHV4_FUNC_NAME_SHOWTERMS = "showTerms";
const char* AUTHV4_FUNC_NAME_SHOWADULTCONFIRM = "showAdultConfirm";
const char* AUTHV4_FUNC_NAME_GETPROFILE = "getProfile";
const char* AUTHV4_FUNC_NAME_SHOWPROFILE = "showProfile";
const char* AUTHV4_FUNC_NAME_RESET = "reset";
const char* AUTHV4_FUNC_NAME_ISAUTOSIGNIN = "isAutoSignIn";
const char* AUTHV4_FUNC_NAME_GETPLAYERINFO = "getPlayerInfo";
const char* AUTHV4_FUNC_NAME_GETPROVIDERFRIENDSLIST = "getProviderFriendsList";
const char* AUTHV4_FUNC_NAME_RESOLVECONFLICT = "resolveConflict";
const char* AUTHV4_FUNC_NAME_SHOWGAMECENTERLOGINCANCELDIALOG = "showGameCenterLoginCancelDialog";
const char* AUTHV4_FUNC_NAME_GETAGEGATEU13 = "getAgeGateU13";
const char* AUTHV4_FUNC_NAME_REQUESTPERMISSIONVIEWDATA = "requestPermissionViewData";

const char* AUTHV4HELPER_FUNC_NAME_SYNCACCOUNT = "syncAccount";
const char* AUTHV4HELPER_FUNC_NAME_SIGNIN = "signIn";
const char* AUTHV4HELPER_FUNC_NAME_SIGNOUT = "signOut";
const char* AUTHV4HELPER_FUNC_NAME_CONNECT = "connect";
const char* AUTHV4HELPER_FUNC_NAME_DISCONNECT = "disconnect";
const char* AUTHV4HELPER_FUNC_NAME_SHOWACHIEVEMENTS = "showAchievements";
const char* AUTHV4HELPER_FUNC_NAME_SHOWLEADERBOARD = "showLeaderboard";
const char* AUTHV4HELPER_FUNC_NAME_ACHIEVEMENTSREVEAL = "achievementsReveal";
const char* AUTHV4HELPER_FUNC_NAME_ACHIEVEMENTSUNLOCK = "achievementsUnlock";
const char* AUTHV4HELPER_FUNC_NAME_ACHIEVEMENTSINCREMENT = "achievementsIncrement";
const char* AUTHV4HELPER_FUNC_NAME_LEADERBOARDS_SUBMITSCORE = "leaderboardsSubmitScore";
const char* AUTHV4HELPER_FUNC_NAME_SHOWCONFLICT = "showConflict";
const char* AUTHV4HELPER_FUNC_NAME_SWITCHACCOUNT = "switchAccount";
const char* AUTHV4HELPER_FUNC_NAME_RESOLVECONFLICT = "resolveConflict";
const char* AUTHV4HELPER_FUNC_NAME_SHOWGAMECENTERLOGINCANCELDIALOG = "showGameCenterLoginCancelDialog";
const char* AUTHV4HELPER_FUNC_NAME_GETIDPLIST = "getIDPList";


static picojson::value callNativeFunction(const char* className,const char* funcName,picojson::object params) {
    
    picojson::object jsonParam = HiveCppPlugin::createParam(className, funcName);
    
    if(params.size() > 0) {
        std::for_each(params.begin(), params.end(), [&jsonParam](picojson::object::value_type it){
            jsonParam[it.first] = it.second;
        });
    }
    return HiveCppPlugin::callNative(jsonParam);
}

static picojson::value callNativeFunction(const char* className,const char* funcName,std::string params) {
    
    picojson::object jsonParam = HiveCppPlugin::createParam(className, funcName);
    
    if(params.size()>0){
        std::string addingParam = "";
        if(jsonParam.size() > 0) {
            std::for_each(jsonParam.begin(), jsonParam.end(), [&addingParam](picojson::object::value_type it){
                addingParam += std::string(",")+"\""+it.first+"\":";
                if(it.second.is<std::string>()){
                    addingParam += "\""+it.second.get<std::string>()+"\"";
                }else if(it.second.is<bool>()){
                    addingParam += it.second.get<double>() ? "true":"false";
                }else if(it.second.is<double>()){
                    addingParam += to_string(it.second.get<double>());
                }
            });
        }
        
        size_t find = params.rfind("}");
        params.insert(find, addingParam);
        printf("****** conflict parms : %s",params.c_str());
        return HiveCppPlugin::callNative(params);
    }else {
        return HiveCppPlugin::callNative(jsonParam);
    }
}

static picojson::value callNativeAuthV4(const char* funcName,picojson::object params) {
    return callNativeFunction("AuthV4", funcName, params);
}
static picojson::value callNativeAuthV4(const char* funcName,std::string params) {
    return callNativeFunction("AuthV4", funcName, params);
}
static picojson::value callNativeAuthV4Helper(const char* funcName,picojson::object params) {
    return callNativeFunction("AuthV4Helper", funcName, params);
}
static picojson::value callNativeAuthV4Helper(const char* funcName,std::string params) {
    return callNativeFunction("AuthV4Helper", funcName, params);
}

static picojson::value callNativeAuthV4(const char* funcName) {
    return callNativeAuthV4(funcName, picojson::object());
}

static picojson::value callNativeAuthV4Helper(const char* funcName) {
    return callNativeAuthV4Helper(funcName, picojson::object());
}


void AuthV4::setup(onSetup setupListener){
    g_Authv4SetupListener = setupListener;
    callNativeAuthV4(AUTHV4_FUNC_NAME_SETUP);
}
void AuthV4::signIn(ProviderType providerType, onSignIn signInListener){
    g_Authv4SignInListener = signInListener;
    picojson::object jsonParam;
    jsonParam["providerType"] = picojson::value(ProviderInfo::stringProviderType(providerType));
    callNativeAuthV4(AUTHV4_FUNC_NAME_SIGNIN,jsonParam);
}
void AuthV4::signOut(onSignOut signOutListener){
    g_Authv4SignOutListener = signOutListener;
    callNativeAuthV4(AUTHV4_FUNC_NAME_SIGNOUT);
}
void AuthV4::connect(ProviderType providerType, onConnect connectListener){
    g_Authv4ConnectListener = connectListener;
    picojson::object jsonParam;
    jsonParam["providerType"] = picojson::value(ProviderInfo::stringProviderType(providerType));
    callNativeAuthV4(AUTHV4_FUNC_NAME_CONNECT,jsonParam);
}
void AuthV4::disconnect(ProviderType providerType, onDisconnect disconnectListener){
    g_Authv4DisconnectListener = disconnectListener;
    picojson::object jsonParam;
    jsonParam["providerType"] = picojson::value(ProviderInfo::stringProviderType(providerType));
    callNativeAuthV4(AUTHV4_FUNC_NAME_DISCONNECT,jsonParam);
}
void AuthV4::selectConflict(PlayerID selectedPlayerId, onSignIn signInListener){
    g_Authv4SelectSignInListener = signInListener;
    picojson::object jsonParam;
    jsonParam["selectedPlayerId"] = picojson::value(to_string(selectedPlayerId));
    callNativeAuthV4(AUTHV4_FUNC_NAME_SELECTCONFLICT,jsonParam);
}
void AuthV4::checkProvider(ProviderType providerType, onCheckProvider checkProviderListener){
    g_Authv4CheckProviderListener = checkProviderListener;
    picojson::object jsonParam;
    jsonParam["providerType"] = picojson::value(ProviderInfo::stringProviderType(providerType));
    callNativeAuthV4(AUTHV4_FUNC_NAME_CHECKPROVIDER,jsonParam);
}
void AuthV4::checkMaintenance(bool isShow, onMaintenance maintenanceListener){
    g_Authv4CheckMaintenanceListener = maintenanceListener;
    picojson::object jsonParam;
    jsonParam["isShow"] = picojson::value(isShow);
    callNativeAuthV4(AUTHV4_FUNC_NAME_CHECKMAINTENANCE,jsonParam);
}
void AuthV4::checkBlacklist(bool isShow, onMaintenance maintenanceListener){
    g_Authv4CheckBlackListMaintenanceListener = maintenanceListener;
    picojson::object jsonParam;
    jsonParam["isShow"] = picojson::value(isShow);
    callNativeAuthV4(AUTHV4_FUNC_NAME_CHECKBLACKLIST,jsonParam);
}
void AuthV4::setProviderChangedListener(onCheckProvider checkProviderLisener){
    g_Authv4SetProviderChangedHandler = checkProviderLisener;
    callNativeAuthV4(AUTHV4_FUNC_NAME_SETPROVIDERCHANGEDLISTENER);
}
void AuthV4::showSignIn(onSignIn signInListener){
    g_Authv4ShowSignInListener = signInListener;
    callNativeAuthV4(AUTHV4_FUNC_NAME_SHOWSIGNIN);
}

void AuthV4::showConflictSelection(IConflictViewInfo const & viewInfo,onSignIn signInListener){
    g_Authv4ConflictSelectionSignInListener = signInListener;
    
    std::string json = "{";
    if(viewInfo.toJsonCurrent() != "")
        json += "\"currentPlayerData\":"+viewInfo.toJsonCurrent()+",";
    if(viewInfo.toJsonConflict() != "")
        json += "\"conflictPlayerData\":"+viewInfo.toJsonConflict();
    json += "}";
    callNativeAuthV4(AUTHV4_FUNC_NAME_SHOWCONFLICTSELECTION,json);
}

void AuthV4::showCafe(onShowCafe showCafeListener) {
    g_AuthV4ShowCafeListener = showCafeListener;
    callNativeAuthV4(AUTHV4_FUNC_NAME_SHOWCAFE);
}

void AuthV4::showInquiry(onShowInquiry showInquiryListener){
    g_AuthV4ShowInquiryListener = showInquiryListener;
    callNativeAuthV4(AUTHV4_FUNC_NAME_SHOWINQUIRY);
}

void AuthV4::showTerms(onShowTerms showTermsListener){
    g_Authv4ShowTermsListener = showTermsListener;
    callNativeAuthV4(AUTHV4_FUNC_NAME_SHOWTERMS);
}
void AuthV4::showAdultConfirm(onAdultConfirm adultConfirmListener){
    g_Authv4ShowAdultConfirmListener = adultConfirmListener;
    callNativeAuthV4(AUTHV4_FUNC_NAME_SHOWADULTCONFIRM);
}
void AuthV4::getProfile(std::vector<PlayerID> playerIdList, onGetProfile getProfileListener){
    g_Authv4GetProfileListener = getProfileListener;
    picojson::object jsonParam;
    picojson::array jsonArray;
    for (std::vector<PlayerID>::iterator it = playerIdList.begin(); it != playerIdList.end(); ++it) {
        jsonArray.push_back(picojson::value((to_string(*it))));
    }
    jsonParam["playerIdList"] = picojson::value(jsonArray);
    
    callNativeAuthV4(AUTHV4_FUNC_NAME_GETPROFILE,jsonParam);
}
void AuthV4::showProfile(PlayerID playerId, onShowProfile showProfileListener){
    g_Authv4ShowProfileListener = showProfileListener;
    picojson::object jsonParam;
    jsonParam["playerId"] = picojson::value(to_string(playerId));
    callNativeAuthV4(AUTHV4_FUNC_NAME_SHOWPROFILE,jsonParam);
}
void AuthV4::reset(){
    callNativeAuthV4(AUTHV4_FUNC_NAME_RESET);
}
bool AuthV4::isAutoSignIn(){
    picojson::value resJson = callNativeAuthV4(AUTHV4_FUNC_NAME_ISAUTOSIGNIN);
    bool isReadyResult = false;
    picojson::value jsonValue = resJson.get("isAutoSignInResult");
    if(jsonValue.is<picojson::null>() == false)
        isReadyResult = jsonValue.evaluate_as_boolean();
    
    return isReadyResult;
}
PlayerInfo AuthV4::getPlayerInfo(){
    picojson::value resJson = callNativeAuthV4(AUTHV4_FUNC_NAME_GETPLAYERINFO);
    PlayerInfo playerInfo(resJson.get("getPlayerInfo"));
    return playerInfo;
}

void AuthV4::getProviderFriendsList(ProviderType providerType, onGetProviderFriendsList getProviderFriendsListListener){
    g_AuthV4GetProviderFriendsListListener = getProviderFriendsListListener;
    picojson::object jsonParam;
    jsonParam["providerType"] = picojson::value(ProviderInfo::stringProviderType(providerType));
    callNativeAuthV4(AUTHV4_FUNC_NAME_GETPROVIDERFRIENDSLIST,jsonParam);
}

void AuthV4::resolveConflict(onResolveConflict resolveConflictListener) {
    g_AuthV4ResolveConflict = resolveConflictListener;
    callNativeAuthV4(AUTHV4_FUNC_NAME_RESOLVECONFLICT);
}

//4.7.0_ADDED
void AuthV4::showGameCenterLoginCancelDialog(onAuthV4DialogDismiss listener){
    g_AuthV4DialogDismiss = listener;
    callNativeAuthV4(AUTHV4_FUNC_NAME_SHOWGAMECENTERLOGINCANCELDIALOG);
}

bool AuthV4::getAgeGateU13() {
    picojson::value resJson = callNativeAuthV4(AUTHV4_FUNC_NAME_GETAGEGATEU13);
    
    picojson::value jsonValue = resJson.get("ageGateU13");
    if(jsonValue.is<picojson::null>() == false) {
        return jsonValue.evaluate_as_boolean();
    } else {
        return false;
    }
}

void AuthV4::requestPermissionViewData(onAuthV4RequestPermissionViewData listener) {
    g_onAuthV4RequestPermissionViewData = listener;
    callNativeAuthV4(AUTHV4_FUNC_NAME_REQUESTPERMISSIONVIEWDATA);
}

void AuthV4::executeEngine(picojson::value jsonParam) {
    ResultAPI result(jsonParam.get("resultAPI"));

    picojson::value jsonMethodValue;
    jsonMethodValue = jsonParam.get("method");
    if(jsonMethodValue.is<picojson::null>()) {
        return;
    }
    const bool isResultSuccess = result.isSuccess();
    
    std::string methodName = jsonMethodValue.to_str();
    
    if(methodName.compare(AUTHV4_FUNC_NAME_SETUP) == 0){
        if(g_Authv4SetupListener != nullptr) {
            
            bool isAuthSignIn = false;
            std::string did = "";
            std::vector<ProviderType> providerTypeList;
            
            if(isResultSuccess) {
                isAuthSignIn = jsonParam.get("isAutoSignIn").get<bool>();
                did = jsonParam.get("did").to_str();
				
				if(jsonParam.contains("providerTypeList") && !jsonParam.get("providerTypeList").is<picojson::null>()){
					picojson::array providerJsonArray = jsonParam.get("providerTypeList").get<picojson::array>();
					for(auto it = providerJsonArray.begin();it != providerJsonArray.end();++it) {
						auto provider = (*it).to_str();
						providerTypeList.push_back(ProviderInfo::providerTypeFromString(provider));
					}
				}
            }
            
            g_Authv4SetupListener(result,isAuthSignIn,did,providerTypeList);
        }
    }
    else if(methodName.compare(AUTHV4_FUNC_NAME_SIGNIN) == 0) {
        if(g_Authv4SignInListener != nullptr){
            PlayerInfo info;
            if(isResultSuccess) {
                info =PlayerInfo(jsonParam.get("playerInfo"));
            }
            g_Authv4SignInListener(result,info);
            
        }
    }else if(methodName.compare(AUTHV4_FUNC_NAME_SIGNOUT) == 0) {
        if(g_Authv4SignOutListener != nullptr) {
            g_Authv4SignOutListener(result);
        }
    }else if(methodName.compare(AUTHV4_FUNC_NAME_CONNECT) == 0) {
        if(g_Authv4ConnectListener != nullptr) {
            PlayerInfo info;
            bool bConflict = false;
            bConflict = jsonParam.get("conflictPlayer").is<picojson::null>() == false;
            if(bConflict)
                info = PlayerInfo(jsonParam.get("conflictPlayer"));
            g_Authv4ConnectListener(result,info);
        }
    }else if(methodName.compare(AUTHV4_FUNC_NAME_DISCONNECT) == 0) {
        if(g_Authv4DisconnectListener != nullptr) {
            g_Authv4DisconnectListener(result);
        }
    }else if(methodName.compare(AUTHV4_FUNC_NAME_SELECTCONFLICT) == 0) {
        if(g_Authv4SelectSignInListener != nullptr) {
            PlayerInfo info;
            if(isResultSuccess)
                info = PlayerInfo(jsonParam.get("playerInfo"));
            g_Authv4SelectSignInListener(result, info);
        }
    }else if(methodName.compare(AUTHV4_FUNC_NAME_CHECKPROVIDER) == 0) {
        if(g_Authv4CheckProviderListener != nullptr) {
            ProviderInfo info;
            if(isResultSuccess)
                info = ProviderInfo(jsonParam.get("providerInfo"));
            g_Authv4CheckProviderListener(result,info);
        }
    }else if(methodName.compare(AUTHV4_FUNC_NAME_CHECKMAINTENANCE) == 0) {
        if(g_Authv4CheckMaintenanceListener != nullptr) {
            std::vector<AuthV4MaintenanceInfo> infos;
            if(isResultSuccess){
                picojson::array array = jsonParam.get("AuthV4MaintenanceInfoList").get<picojson::array>();
                for(auto it=array.begin();it!=array.end();++it) {
                    AuthV4MaintenanceInfo info((*it));
                    infos.push_back(info);
                }
            }
            
            g_Authv4CheckMaintenanceListener(result,infos);
        }
    }else if(methodName.compare(AUTHV4_FUNC_NAME_CHECKBLACKLIST) == 0) {
        if(g_Authv4CheckBlackListMaintenanceListener != nullptr) {
            std::vector<AuthV4MaintenanceInfo> infos;
            if(isResultSuccess) {
                picojson::array array = jsonParam.get("AuthV4MaintenanceInfoList").get<picojson::array>();
                for(auto it=array.begin();it!=array.end();++it) {
                    AuthV4MaintenanceInfo info((*it));
                    infos.push_back(info);
                }
            }
            g_Authv4CheckBlackListMaintenanceListener(result,infos);
        }
    }else if(methodName.compare(AUTHV4_FUNC_NAME_SETPROVIDERCHANGEDLISTENER) == 0) {
        if(g_Authv4SetProviderChangedHandler != nullptr) {
            ProviderInfo info;
            if(isResultSuccess)
                info = ProviderInfo(jsonParam.get("providerInfo"));
            g_Authv4SetProviderChangedHandler(result,info);
        }
    }else if(methodName.compare(AUTHV4_FUNC_NAME_SHOWSIGNIN) == 0) {
        if(g_Authv4ShowSignInListener != nullptr) {
            PlayerInfo info;
            if(isResultSuccess)
                info = PlayerInfo(jsonParam.get("playerInfo"));
            g_Authv4ShowSignInListener(result,info);
        }
    }else if(methodName.compare(AUTHV4_FUNC_NAME_SHOWCONFLICTSELECTION) == 0) {
        if(g_Authv4ConflictSelectionSignInListener != nullptr) {
            PlayerInfo info;
            if(isResultSuccess)
                info = PlayerInfo(jsonParam.get("playerInfo"));
            g_Authv4ConflictSelectionSignInListener(result,info);
        }
    }else if(methodName.compare(AUTHV4_FUNC_NAME_SHOWCAFE) == 0) {
        if(g_AuthV4ShowCafeListener != nullptr) {
            g_AuthV4ShowCafeListener(result);
        }
    }else if(methodName.compare(AUTHV4_FUNC_NAME_SHOWINQUIRY) == 0) {
        if(g_AuthV4ShowInquiryListener != nullptr) {
            g_AuthV4ShowInquiryListener(result);
        }
    }else if(methodName.compare(AUTHV4_FUNC_NAME_SHOWTERMS) == 0) {
        if(g_Authv4ShowTermsListener != nullptr) {
            g_Authv4ShowTermsListener(result);
        }
    }else if(methodName.compare(AUTHV4_FUNC_NAME_SHOWADULTCONFIRM) == 0) {
        if(g_Authv4ShowAdultConfirmListener != nullptr) {
            g_Authv4ShowAdultConfirmListener(result);
        }
    }else if(methodName.compare(AUTHV4_FUNC_NAME_GETPROFILE) == 0) {
        if(g_Authv4GetProfileListener != nullptr) {
            std::vector<ProfileInfo> infos;
            if(isResultSuccess){
                picojson::array array = jsonParam.get("profileInfoList").get<picojson::array>();
                for(auto it=array.begin();it!=array.end();++it) {
                    ProfileInfo info((*it));
                    infos.push_back(info);
                }
            }
            g_Authv4GetProfileListener(result,infos);
        }
    }else if(methodName.compare(AUTHV4_FUNC_NAME_SHOWPROFILE) == 0) {
        if(g_Authv4ShowProfileListener != nullptr) {
            g_Authv4ShowProfileListener(result);
        }
    }else if(methodName.compare(AUTHV4_FUNC_NAME_GETPROVIDERFRIENDSLIST) == 0) {
        if(g_AuthV4GetProviderFriendsListListener != nullptr) {
            std::map<std::string,PlayerID> providerUserIdList;
            ProviderType providerType;
            
            if(!jsonParam.get("providerType").is<picojson::null>()){
                providerType = ProviderInfo::providerTypeFromString(jsonParam.get("providerType").get<std::string>());
            }
            
            if(!jsonParam.get("providerUserIdList").is<picojson::null>()){
                auto providerUserListJson = jsonParam.get("providerUserIdList").get<picojson::object>();
                for(auto it = providerUserListJson.begin();it != providerUserListJson.end();++it){
                    providerUserIdList.insert(std::make_pair(it->first, static_cast<PlayerID>(it->second.get<double>())));
                }
            }
            g_AuthV4GetProviderFriendsListListener(result,providerType,providerUserIdList);
        }
    }else if(methodName.compare(AUTHV4_FUNC_NAME_RESOLVECONFLICT) == 0){
        if(g_AuthV4ResolveConflict != nullptr){
            g_AuthV4ResolveConflict(result);
        }
    }else if(methodName.compare(AUTHV4_FUNC_NAME_SHOWGAMECENTERLOGINCANCELDIALOG) == 0){
        if(g_AuthV4DialogDismiss != nullptr){
            bool isDismiss = jsonParam.get("isDismiss").get<bool>();
            g_AuthV4DialogDismiss(isDismiss);
        }
    }else if(methodName.compare(AUTHV4_FUNC_NAME_REQUESTPERMISSIONVIEWDATA) == 0){
        if(g_onAuthV4RequestPermissionViewData != NULL){
            g_onAuthV4RequestPermissionViewData(result,PermissionViewData(jsonParam.get("data")));
        }
    }
}

ConflictSingleViewInfo::ConflictSingleViewInfo(PlayerID playerId)
{
    std::string strPlayerId = to_string(playerId);
    jsonData = "{\"player_id\":";
    jsonData.append(strPlayerId);
    jsonData.append(",\"game_data\":{ ");
}

ConflictSingleViewInfo::~ConflictSingleViewInfo(){
    
}

void ConflictSingleViewInfo::SetValue(const std::string key,const long long playerData) {
    jsonData += "\""+key+"\":"+to_string(playerData)+",";
}
void ConflictSingleViewInfo::SetValue(const std::string key,const double playerData){
    jsonData += "\""+key+"\":"+to_string(playerData)+",";
}
void ConflictSingleViewInfo::SetValue(const std::string key,const std::string playerData) {
    jsonData += "\""+key+"\":\""+playerData+"\""+",";
}
std::string ConflictSingleViewInfo::toJsonConflict() const {
    std::string tempString = jsonData;
    if( tempString[tempString.length() -1 ] == ',' ) //
    {
        tempString.erase( tempString.length()-1, 1 );
    }
    tempString.append("}}");
    return tempString;
}
std::string ConflictSingleViewInfo::toJsonCurrent() const {
    return "";
}

ConflictViewInfo::ConflictViewInfo(PlayerID currentPlayerId,PlayerID conflictPlayerId)
{
    std::string strCurrentID = to_string(currentPlayerId);      // prevent ios 8 Crash on Unreal Engine 4.15. memory allocate fail crash.
    jsonCurrent = "{\"player_id\":";
    jsonCurrent.append(strCurrentID);
    jsonCurrent.append(",\"game_data\":{ ");

    std::string strConflictID = to_string(conflictPlayerId);
    jsonConflict = "{\"player_id\":";
    jsonConflict.append(strConflictID);
    jsonConflict.append(",\"game_data\":{ ");
}
ConflictViewInfo::~ConflictViewInfo() {
    
}
void ConflictViewInfo::SetValue(const std::string key,const long long currentPlayerData,const long long conflictPlayerData) {
    jsonCurrent += "\""+key+"\":"+to_string(currentPlayerData)+",";
    jsonConflict += "\""+key+"\":"+to_string(conflictPlayerData)+",";
}
void ConflictViewInfo::SetValue(const std::string key,const double currentPlayerData,const double conflictPlayerData) {
    jsonCurrent += "\""+key+"\":"+to_string(currentPlayerData)+",";
    jsonConflict += "\""+key+"\":"+to_string(conflictPlayerData)+",";
}
void ConflictViewInfo::SetValue(const std::string key,const std::string currentPlayerData,const std::string conflictPlayerData) {
    jsonCurrent += "\""+key+"\":\""+currentPlayerData+"\""+",";
    jsonConflict += "\""+key+"\":\""+conflictPlayerData+"\""+",";
}

std::string ConflictViewInfo::toJsonConflict() const {
    std::string tempString = jsonConflict;
    if( tempString[tempString.length() -1 ] == ',' ) //
    {
        tempString.erase( tempString.length()-1, 1 );
    }
    tempString.append("}}");
    return tempString;
}
std::string ConflictViewInfo::toJsonCurrent() const {
    std::string tempString = jsonCurrent;
    if( tempString[tempString.length() -1 ] == ',' ) //
    {
        tempString.erase( tempString.length()-1, 1 );
    }
    tempString.append("}}");
    return tempString;
}

ProviderInfo::ProviderInfo() {
    
}
ProviderInfo::ProviderInfo(picojson::value object)
{
    if(object.is<picojson::null>())
        return;
    
    picojson::value jsonValue;
    
    jsonValue = object.get("providerType");
    if(jsonValue.is<picojson::null>() == false)
        this->providerType = static_cast<ProviderType>(jsonValue.get<double>());
    
    jsonValue = object.get("providerUserId");
    if(jsonValue.is<picojson::null>() == false)
        this->providerUserId = jsonValue.to_str();
}
std::string ProviderInfo::stringProviderType(ProviderType providerType) {
    switch(providerType){
        case ProviderType::GUEST:           return "GUEST";
        case ProviderType::HIVE:            return "HIVE";
        case ProviderType::FACEBOOK:    	return "FACEBOOK";
        case ProviderType::GOOGLE:          return "GOOGLE";
        case ProviderType::QQ:              return "QQ";
        case ProviderType::WEIBO:           return "WEIBO";
        case ProviderType::VK:              return "VK";
        case ProviderType::WECHAT:      	return "WECHAT";
        case ProviderType::APPLE:           return "APPLE";
        case ProviderType::SIGNIN_APPLE:    return "SIGNIN_APPLE";
        case ProviderType::LINE:            return "LINE";
        case ProviderType::TWITTER:         return "TWITTER";
        case ProviderType::AUTO:            return "AUTO";
        default:
            return "";
    }
}

ProviderType ProviderInfo::providerTypeFromString(const std::string typeString) {
    static std::map<std::string,ProviderType> stot = {
        {"GUEST",           ProviderType::GUEST}
        ,{"HIVE",           ProviderType::HIVE  }
        ,{"FACEBOOK",       ProviderType::FACEBOOK}
        ,{"GOOGLE",         ProviderType::GOOGLE }
        ,{"QQ",             ProviderType::QQ  }
        ,{"WEIBO",          ProviderType::WEIBO  }
        ,{"VK",             ProviderType::VK   }
        ,{"WECHAT",         ProviderType::WECHAT }
        ,{"APPLE",          ProviderType::APPLE  }
        ,{"SIGNIN_APPLE",   ProviderType::SIGNIN_APPLE  }
        ,{"LINE",           ProviderType::LINE  }
        ,{"TWITTER",        ProviderType::TWITTER   }
        ,{"AUTO",           ProviderType::AUTO   }
    };
    return stot[typeString];
}
ProfileInfo::ProfileInfo() {
    
}

ProfileInfo::ProfileInfo(picojson::value object) {
    if(object.is<picojson::null>())
       return;
    
    picojson::value jsonValue;
    
    jsonValue = object.get("playerId");
    if(jsonValue.is<picojson::null>() == false)
        this->playerId = static_cast<PlayerID>(jsonValue.get<double>());
    
    jsonValue = object.get("playerName");
    if(jsonValue.is<picojson::null>() == false)
        this->playerName = jsonValue.to_str();
    
    jsonValue = object.get("playerImageUrl");
    if(jsonValue.is<picojson::null>() == false)
        this->playerImageUrl = jsonValue.to_str();
}

ProfileInfo::~ProfileInfo(){
    
}

PlayerInfo::PlayerInfo() {
    
}

PlayerInfo::PlayerInfo(picojson::value object) : ProfileInfo(object) {
    if(object.is<picojson::null>())
        return;
    
    picojson::value jsonValue;
    
    jsonValue = object.get("playerToken");
    if(jsonValue.is<picojson::null>() == false)
        this->playerToken = jsonValue.to_str();
    
    jsonValue = object.get("did");
    if(jsonValue.is<picojson::null>() == false)
        this->did = jsonValue.to_str();
    
    jsonValue = object.get("providerInfoData");
    if(jsonValue.is<picojson::null>() == false){
        auto map = jsonValue.get<picojson::object>();
        for(auto it = map.begin();it != map.end();++it){
            ProviderInfo data(it->second);
            this->providerInfoData.insert(std::make_pair(ProviderInfo::providerTypeFromString(it->first), data));
        }
    }
}

PlayerInfo::~PlayerInfo() {
    
}

AuthV4MaintenanceInfo::AuthV4MaintenanceInfo(picojson::value object) {
    if(object.is<picojson::null>())
        return;
    
    picojson::value jsonValue;
    title = object.get("title").to_str();
    message = object.get("message").to_str();
    button = object.get("button").to_str();
    action = static_cast<AuthV4MaintenanceActionType>(object.get("action").get<double>());
    url = object.get("url").to_str();
    startDate = object.get("startDate").to_str();
    endDate = object.get("endDate").to_str();
    remainingTime = object.get("remainingTime").get<double>();
}

void AuthV4::Helper::syncAccount(ProviderType providerType, onAuthV4Helper listener){
    g_AuthV4HelperCallbacks[AUTHV4HELPER_FUNC_NAME_SYNCACCOUNT] = listener;
    picojson::object jsonParam;
    jsonParam["providerType"] = picojson::value(ProviderInfo::stringProviderType(providerType));
    callNativeAuthV4Helper(AUTHV4HELPER_FUNC_NAME_SYNCACCOUNT,jsonParam);
}
void AuthV4::Helper::signIn(onAuthV4Helper listener){
    g_AuthV4HelperCallbacks[AUTHV4HELPER_FUNC_NAME_SIGNIN] = listener;
    callNativeAuthV4Helper(AUTHV4HELPER_FUNC_NAME_SIGNIN);
}
void AuthV4::Helper::signOut(onAuthV4Helper listener){
    g_AuthV4HelperCallbacks[AUTHV4HELPER_FUNC_NAME_SIGNOUT] = listener;
    callNativeAuthV4Helper(AUTHV4HELPER_FUNC_NAME_SIGNOUT);
}
void AuthV4::Helper::connect(ProviderType providerType, onAuthV4Helper listener){
    g_AuthV4HelperCallbacks[AUTHV4HELPER_FUNC_NAME_CONNECT] = listener;
    picojson::object jsonParam;
    jsonParam["providerType"] = picojson::value(ProviderInfo::stringProviderType(providerType));
    callNativeAuthV4Helper(AUTHV4HELPER_FUNC_NAME_CONNECT,jsonParam);
}
void AuthV4::Helper::disconnect(ProviderType providerType, onAuthV4Helper listener){
    g_AuthV4HelperCallbacks[AUTHV4HELPER_FUNC_NAME_DISCONNECT] = listener;
    picojson::object jsonParam;
    jsonParam["providerType"] = picojson::value(ProviderInfo::stringProviderType(providerType));
    callNativeAuthV4Helper(AUTHV4HELPER_FUNC_NAME_DISCONNECT,jsonParam);
}
void AuthV4::Helper::showAchievements(onAuthV4Helper listener){
    g_AuthV4HelperCallbacks[AUTHV4HELPER_FUNC_NAME_SHOWACHIEVEMENTS] = listener;
    callNativeAuthV4Helper(AUTHV4HELPER_FUNC_NAME_SHOWACHIEVEMENTS);
}
void AuthV4::Helper::showLeaderboard(onAuthV4Helper listener){
    g_AuthV4HelperCallbacks[AUTHV4HELPER_FUNC_NAME_SHOWLEADERBOARD] = listener;
    callNativeAuthV4Helper(AUTHV4HELPER_FUNC_NAME_SHOWLEADERBOARD);
}

void AuthV4::Helper::achievementsReveal(const std::string achievementId, onAuthV4Helper listener){
    g_AuthV4HelperCallbacks[AUTHV4HELPER_FUNC_NAME_ACHIEVEMENTSREVEAL] = listener;
    picojson::object jsonParam;
    jsonParam["achievementId"] = picojson::value(achievementId);
    jsonParam["handler"] = picojson::value("true");
    callNativeAuthV4Helper(AUTHV4HELPER_FUNC_NAME_ACHIEVEMENTSREVEAL, jsonParam);
}

void AuthV4::Helper::achievementsUnlock(const std::string achievementId, onAuthV4Helper listener){
    g_AuthV4HelperCallbacks[AUTHV4HELPER_FUNC_NAME_ACHIEVEMENTSUNLOCK] = listener;
    picojson::object jsonParam;
    jsonParam["achievementId"] = picojson::value(achievementId);
    jsonParam["handler"] = picojson::value("true");
    callNativeAuthV4Helper(AUTHV4HELPER_FUNC_NAME_ACHIEVEMENTSUNLOCK, jsonParam);
}

void AuthV4::Helper::achievementsIncrement(const std::string achievementId, const int value, onAuthV4Helper listener){
    g_AuthV4HelperCallbacks[AUTHV4HELPER_FUNC_NAME_ACHIEVEMENTSINCREMENT] = listener;
    picojson::object jsonParam;
    jsonParam["incrementalAchievementId"] = picojson::value(achievementId);
    jsonParam["value"] = picojson::value(to_string(value));
    jsonParam["handler"] = picojson::value("true");
    callNativeAuthV4Helper(AUTHV4HELPER_FUNC_NAME_ACHIEVEMENTSINCREMENT, jsonParam);
}

void AuthV4::Helper::leaderboardsSubmitScore(const std::string leaderboardId, const long long score, onAuthV4Helper listener){
    g_AuthV4HelperCallbacks[AUTHV4HELPER_FUNC_NAME_LEADERBOARDS_SUBMITSCORE] = listener;
    picojson::object jsonParam;
    jsonParam["leaderboardId"] = picojson::value(leaderboardId);
    jsonParam["score"] = picojson::value(to_string(score));
    jsonParam["handler"] = picojson::value("true");
    callNativeAuthV4Helper(AUTHV4HELPER_FUNC_NAME_LEADERBOARDS_SUBMITSCORE, jsonParam);
}

void AuthV4::Helper::showConflict(onAuthV4Helper listener){
    g_AuthV4HelperCallbacks[AUTHV4HELPER_FUNC_NAME_SHOWCONFLICT] = listener;
    callNativeAuthV4Helper(AUTHV4HELPER_FUNC_NAME_SHOWCONFLICT);
}

void AuthV4::Helper::showConflict(ConflictSingleViewInfo const & viewInfo,onAuthV4Helper listener){
    
    g_AuthV4HelperCallbacks[AUTHV4HELPER_FUNC_NAME_SHOWCONFLICT] = listener;
    std::string json = "{";
    if(viewInfo.toJsonConflict() != "")
        json += "\"playerData\":"+viewInfo.toJsonConflict();
    json += "}";
    callNativeAuthV4Helper(AUTHV4HELPER_FUNC_NAME_SHOWCONFLICT,json);
}

void AuthV4::Helper::switchAccount(onAuthV4Helper listener){
    g_AuthV4HelperCallbacks[AUTHV4HELPER_FUNC_NAME_SWITCHACCOUNT] = listener;
    callNativeAuthV4Helper(AUTHV4HELPER_FUNC_NAME_SWITCHACCOUNT);
}
void AuthV4::Helper::resolveConflict(onAuthV4Helper listener){
    g_AuthV4HelperCallbacks[AUTHV4HELPER_FUNC_NAME_RESOLVECONFLICT] = listener;
    callNativeAuthV4Helper(AUTHV4HELPER_FUNC_NAME_RESOLVECONFLICT);
}
std::vector<ProviderType> AuthV4::Helper::getIDPList(){
    picojson::value resJson = callNativeAuthV4Helper(AUTHV4HELPER_FUNC_NAME_GETIDPLIST);
    
    std::vector<ProviderType> types;
    picojson::array array = resJson.get("idpList").get<picojson::array>();
    for(auto it=array.begin();it!=array.end();++it) {
        ProviderType type = ProviderInfo::providerTypeFromString((*it).to_str());
        types.push_back(type);
    }
    
    return types;
}
void AuthV4::Helper::showGameCenterLoginCancelDialog(onAuthV4DialogDismiss listener){
    g_AuthV4HelperDialogDismiss = listener;
    callNativeAuthV4Helper(AUTHV4HELPER_FUNC_NAME_SHOWGAMECENTERLOGINCANCELDIALOG);
}

void AuthV4::Helper::executeEngine(picojson::value jsonParam) {
    ResultAPI result(jsonParam.get("resultAPI"));
    
    picojson::value jsonMethodValue;
    jsonMethodValue = jsonParam.get("method");
    if(jsonMethodValue.is<picojson::null>()) {
        return;
    }
    const bool isResultSuccess = result.isSuccess();
    
    std::string methodName = jsonMethodValue.to_str();
    
    if((methodName.compare(AUTHV4HELPER_FUNC_NAME_SYNCACCOUNT) == 0)
       || (methodName.compare(AUTHV4HELPER_FUNC_NAME_SIGNIN) == 0)
       || (methodName.compare(AUTHV4HELPER_FUNC_NAME_SIGNOUT) == 0)
       || (methodName.compare(AUTHV4HELPER_FUNC_NAME_CONNECT) == 0)
       || (methodName.compare(AUTHV4HELPER_FUNC_NAME_DISCONNECT) == 0)
       || (methodName.compare(AUTHV4HELPER_FUNC_NAME_SHOWACHIEVEMENTS) == 0)
       || (methodName.compare(AUTHV4HELPER_FUNC_NAME_SHOWLEADERBOARD) == 0)
       || (methodName.compare(AUTHV4HELPER_FUNC_NAME_ACHIEVEMENTSREVEAL) == 0)
       || (methodName.compare(AUTHV4HELPER_FUNC_NAME_ACHIEVEMENTSUNLOCK) == 0)
       || (methodName.compare(AUTHV4HELPER_FUNC_NAME_ACHIEVEMENTSINCREMENT) == 0)
       || (methodName.compare(AUTHV4HELPER_FUNC_NAME_LEADERBOARDS_SUBMITSCORE) == 0)
       || (methodName.compare(AUTHV4HELPER_FUNC_NAME_SHOWCONFLICT) == 0)
       || (methodName.compare(AUTHV4HELPER_FUNC_NAME_SWITCHACCOUNT) == 0)
       || (methodName.compare(AUTHV4HELPER_FUNC_NAME_RESOLVECONFLICT) == 0)) {
        AuthV4::Helper::onAuthV4Helper callback = g_AuthV4HelperCallbacks[methodName];
        if(jsonParam.contains("playerInfo") && !jsonParam.get("playerInfo").is<picojson::null>()){
            std::shared_ptr<PlayerInfo> info = nullptr;
            if(jsonParam.get("playerInfo").is<picojson::object>()) {
                auto v = jsonParam.get("playerInfo");
                const picojson::object o = v.get<picojson::object>();
                if(!o.empty()) {
                    info = std::make_shared<PlayerInfo>(PlayerInfo(jsonParam.get("playerInfo")));
                }
            }
            if(callback){
                callback(result,info);
            }
        } else {
            std::shared_ptr<PlayerInfo> info = nullptr;
            if(callback) {
                callback(result,info);
            }
        }
    }
    else if(methodName.compare(AUTHV4HELPER_FUNC_NAME_SHOWGAMECENTERLOGINCANCELDIALOG) == 0){
        if(g_AuthV4HelperDialogDismiss != nullptr){
            bool isDismiss = jsonParam.get("isDismiss").get<bool>();
            g_AuthV4HelperDialogDismiss(isDismiss);
        }
    }

}
NS_HIVE_END
