


#include "HIVE_CppPlugin.h"


NS_HIVE_BEGIN


static IAP::onIAPMarketInfo g_onIAPInitializeMarketInfo = NULL;

static IAP::onIAPShopInfo g_onIAPShopInfo = NULL;

static IAP::onIAPPurchase g_onIAPPurchase = NULL;

static IAP::onIAPMarketInfo g_onIAPShowMarketSelectionMarketInfo = NULL;

static IAP::onIAPBalanceInfo g_onIAPBalanceInfo = NULL;

static IAP::onIAPBalanceInfo g_onIAPChargeBalanceInfo = NULL;

//4.3.1
static IAP::onIAPPurchaseReceipt g_onIAPPurchaseReceipt = NULL;

static IAP::onIAPRestoreReceipt g_onIAPRestoreReceipt = NULL;

//4.4.1
static IAP::onCheckPromotePurchase g_onIAPCheckPromotePurchase = nullptr;

void IAP::initialize(onIAPMarketInfo listener)
{
	g_onIAPInitializeMarketInfo = listener;

	picojson::object jsonParam = HiveCppPlugin::createParam("IAP", "initialize");

	HiveCppPlugin::callNative(jsonParam);
}
    

void IAP::getShopInfo(std::string locationCode, onIAPShopInfo listener)
{
	g_onIAPShopInfo = listener;

	picojson::object jsonParam = HiveCppPlugin::createParam("IAP", "getShopInfo");
	jsonParam["locationCode"] = picojson::value(locationCode);

	HiveCppPlugin::callNative(jsonParam);
}


//4.3.1
void IAP::purchase(std::string pid, std::string additionalInfo, onIAPPurchaseReceipt listener)
{
    g_onIAPPurchaseReceipt = listener;
    
    picojson::object jsonParam = HiveCppPlugin::createParam("IAP", "purchaseReceipt");
    jsonParam["pid"] = picojson::value(pid);
    
    jsonParam["additionalInfo"] = picojson::value(additionalInfo);
    HiveCppPlugin::callNative(jsonParam);
}

//4.3.1
void IAP::restoreReceipt(onIAPRestoreReceipt listener)
{
    g_onIAPRestoreReceipt = listener;
    
    picojson::object jsonParam = HiveCppPlugin::createParam("IAP", "restoreReceipt");
    
    HiveCppPlugin::callNative(jsonParam);
}


void IAP::showMarketSelection(onIAPMarketInfo listener)
{
    g_onIAPShowMarketSelectionMarketInfo = listener;
    
    picojson::object jsonParam = HiveCppPlugin::createParam("IAP", "showMarketSelection");
    
    HiveCppPlugin::callNative(jsonParam);
}


void IAP::getBalanceInfo(onIAPBalanceInfo listener)
{
	g_onIAPBalanceInfo = listener;

	picojson::object jsonParam = HiveCppPlugin::createParam("IAP", "getBalanceInfo");

	HiveCppPlugin::callNative(jsonParam);
}
    

void IAP::showCharge(onIAPBalanceInfo listener)
{
	g_onIAPChargeBalanceInfo = listener;

	picojson::object jsonParam = HiveCppPlugin::createParam("IAP", "showCharge");

	HiveCppPlugin::callNative(jsonParam);
}

void IAP::checkPromotePurchase(onCheckPromotePurchase listener){
    g_onIAPCheckPromotePurchase = listener;
    picojson::object jsonParam = HiveCppPlugin::createParam("IAP", "checkPromotePurchase");
    HiveCppPlugin::callNative(jsonParam);
}


void IAP::executeEngine(picojson::value jsonParam)
{
	ResultAPI result(jsonParam.get("resultAPI"));

    const bool isResultSuccess = result.isSuccess();
    
	std::string methodName = jsonParam.get("method").to_str();
	if(methodName.compare("initialize") == 0)
	{
		if(g_onIAPInitializeMarketInfo != NULL)
		{
		    picojson::array jsonArray;

			picojson::value jsonValue;
		    jsonValue = jsonParam.get("iapTypeList");
		    if(jsonValue.is<picojson::null>() == false)
		    	jsonArray = jsonValue.get<picojson::array>();

			std::vector<IAPType> iapTypeList;
		    for (picojson::array::iterator iter = jsonArray.begin(); iter != jsonArray.end(); ++iter) {

		    	std::string iapTypeString = (*iter).to_str();
				if(iapTypeString.compare("APPLE_APPSTORE") == 0)
                    iapTypeList.push_back(IAPType::APPLE_APPSTORE);
				else if(iapTypeString.compare("HIVE_LEBI") == 0)
					iapTypeList.push_back(IAPType::HIVE_LEBI);
				else if(iapTypeString.compare("ONESTORE") == 0)
					iapTypeList.push_back(IAPType::ONESTORE);
				else
					iapTypeList.push_back(IAPType::GOOGLE_PLAYSTORE);
		    }

			g_onIAPInitializeMarketInfo(result, iapTypeList);
		}
	}
	else if(methodName.compare("getShopInfo") == 0)
	{
		if(g_onIAPShopInfo != NULL)
		{
			IAPShop iapShop;
			iapShop.parse(jsonParam.get("iapShop"));

            int balance = (int)jsonParam.get("balance").get<double>();
            g_onIAPShopInfo(result, iapShop, balance);
		}
	}
	//4.3.1
    else if(methodName.compare("purchaseReceipt") == 0)
    {
        if(g_onIAPPurchaseReceipt != NULL)
        {
            picojson::value jsonValue;
            
            IAPReceipt lReceipt;
            if(isResultSuccess)
            {
                lReceipt = IAPReceipt(jsonParam.get("iapReceipt"));
            }

            g_onIAPPurchaseReceipt(result, lReceipt);
        }
    }
    //4.3.1
    else if(methodName.compare("restoreReceipt") == 0)
    {
        if(g_onIAPRestoreReceipt != NULL)
        {
            picojson::value jsonValue;
            std::vector<IAPReceipt> receiptList;

            jsonValue = jsonParam.get("receiptList");
            if(jsonValue.is<picojson::null>() || jsonValue.is<picojson::array>() == false) {
                receiptList.push_back(IAPReceipt());
            }
            else
            {
                picojson::array receiptListJson = jsonValue.get<picojson::array>();
                for (picojson::array::iterator iter = receiptListJson.begin(); iter != receiptListJson.end(); ++iter) {
                    
                    IAPReceipt receipt((*iter));
                    receiptList.push_back(receipt);
                }
            }
            
            g_onIAPRestoreReceipt(result, receiptList);
        }
    }
    
    if(methodName.compare("showMarketSelection") == 0)
    {
        if(g_onIAPShowMarketSelectionMarketInfo != NULL)
        {
            picojson::array jsonArray;
            
            picojson::value jsonValue;
            jsonValue = jsonParam.get("iapTypeList");
            if(jsonValue.is<picojson::null>() == false)
                jsonArray = jsonValue.get<picojson::array>();
            
            std::vector<IAPType> iapTypeList;
            for (picojson::array::iterator iter = jsonArray.begin(); iter != jsonArray.end(); ++iter) {
                
                std::string iapTypeString = (*iter).to_str();
                if(iapTypeString.compare("APPLE_APPSTORE") == 0)
                    iapTypeList.push_back(IAPType::APPLE_APPSTORE);
                else if(iapTypeString.compare("HIVE_LEBI") == 0)
                    iapTypeList.push_back(IAPType::HIVE_LEBI);
                else if(iapTypeString.compare("ONESTORE") == 0)
                    iapTypeList.push_back(IAPType::ONESTORE);
                else
                    iapTypeList.push_back(IAPType::GOOGLE_PLAYSTORE);
            }
            
            g_onIAPShowMarketSelectionMarketInfo(result, iapTypeList);
        }
    }
    else if(methodName.compare("getBalanceInfo") == 0)
    {
        if(g_onIAPBalanceInfo != NULL)
        {
            int balance = (int)jsonParam.get("balance").get<double>();
            g_onIAPBalanceInfo(result, balance);
        }
    }
	else if(methodName.compare("showCharge") == 0)
	{
		if(g_onIAPChargeBalanceInfo != NULL)
		{
            int balance = (int)jsonParam.get("balance").get<double>();
            g_onIAPChargeBalanceInfo(result, balance);
		}
	}
    else if(methodName.compare("checkPromotePurchase") == 0)
    {
        if(g_onIAPCheckPromotePurchase != nullptr) {
            std::string marketPid = "";
            if(!jsonParam.get("gamePid").is<picojson::null>()){
                marketPid = jsonParam.get("gamePid").get<std::string>();
            }
            g_onIAPCheckPromotePurchase(result,marketPid);
        }
    }

}




// IAPShop class Implements
IAPShop::IAPShop()
{

}

IAPShop::IAPShop(picojson::value jsonParam)
{
    parse(jsonParam);
}

bool IAPShop::parse(picojson::value jsonParam)
{
    if(jsonParam.is<picojson::null>())
        return false;

    picojson::value jsonValue;
    
    jsonValue = jsonParam.get("locationCode");
    if(jsonValue.is<picojson::null>() == false)
        this->locationCode = jsonValue.to_str();
    
    jsonValue = jsonParam.get("shopId");
    if(jsonValue.is<picojson::null>() == false)
        this->shopId = (int)jsonValue.get<double>();
    
    jsonValue = jsonParam.get("shopType");
    if(jsonValue.is<picojson::null>() == false)
        this->shopType = jsonValue.to_str();
    
    jsonValue = jsonParam.get("originalJson");
    if(jsonValue.is<picojson::null>() == false)
        this->originalJson = jsonValue.to_str();
    

    this->productList.clear();
    picojson::array iapProductListJson;
    
    jsonValue = jsonParam.get("productList");
    if(jsonValue.is<picojson::null>() == false)
        iapProductListJson = jsonValue.get<picojson::array>();
    
    for (picojson::array::iterator iter = iapProductListJson.begin(); iter != iapProductListJson.end(); ++iter) {

    	IAPProduct iapProduct;
    	iapProduct.parse((*iter));
        this->productList.push_back(iapProduct);
    }

    return true;
}

picojson::object IAPShop::toJson() const
{
	picojson::object resJson;

	resJson["locationCode"] = picojson::value(this->locationCode);
	resJson["shopId"] = picojson::value((double)this->shopId);
	resJson["shopType"] = picojson::value(this->shopType);
	resJson["originalJson"] = picojson::value(this->originalJson);

    picojson::array iapProductListJson;
    for(int i = 0; i < this->productList.size(); i++)
    {
    	IAPProduct iapProductPid = this->productList[i];
    	iapProductListJson.push_back(picojson::value(iapProductPid.toJson()));
    }

    resJson["productList"] = picojson::value(iapProductListJson);
    return resJson;
}

std::string IAPShop::toString() const
{
    picojson::object resJson = this->toJson();
    std::string resJsonString = picojson::value(resJson).serialize();
    return resJsonString;
}



// IAPProduct class Implements
IAPProduct::IAPProduct()
{

}

IAPProduct::IAPProduct(picojson::value jsonParam)
{
    parse(jsonParam);
}

bool IAPProduct::parse(picojson::value jsonParam)
{
    if(jsonParam.is<picojson::null>())
        return false;
    
    picojson::value jsonValue;
    
    jsonValue = jsonParam.get("gamePid");
    if(jsonValue.is<picojson::null>() == false)
        this->gamePid = jsonValue.to_str();
    
    jsonValue = jsonParam.get("marketPid");
    if(jsonValue.is<picojson::null>() == false)
        this->marketPid = jsonValue.to_str();
    
    jsonValue = jsonParam.get("currency");
    if(jsonValue.is<picojson::null>() == false)
        this->currency = jsonValue.to_str();
    
    
    jsonValue = jsonParam.get("price");
    if(jsonValue.is<picojson::null>() == false)
        this->price = jsonValue.get<double>();
    
    
    jsonValue = jsonParam.get("displayPrice");
    if(jsonValue.is<picojson::null>() == false)
        this->displayPrice = jsonValue.to_str();
	
    jsonValue = jsonParam.get("originalJson");
    if(jsonValue.is<picojson::null>() == false)
        this->originalJson = jsonValue.to_str();
    
    return true;
}

picojson::object IAPProduct::toJson() const
{
	picojson::object resJson;

	resJson["gamePid"] = picojson::value(this->gamePid);
	resJson["marketPid"] = picojson::value(this->marketPid);
	resJson["currency"] = picojson::value(this->currency);

	resJson["price"] = picojson::value(this->price);

	resJson["displayPrice"] = picojson::value(this->displayPrice);

	resJson["originalJson"] = picojson::value(this->originalJson);

	return resJson;
}

std::string IAPProduct::toString() const
{
    picojson::object resJson = this->toJson();
    std::string resJsonString = picojson::value(resJson).serialize();
    return resJsonString;
}


//4.3.1
IAPReceipt::IAPReceipt()
{
    
}

IAPReceipt::IAPReceipt(picojson::value jsonParam)
{
    parse(jsonParam);
}

bool IAPReceipt::parse(picojson::value jsonParam)
{
    if(jsonParam.is<picojson::null>())
        return false;
    
    picojson::value jsonValue;
    
    jsonValue = jsonParam.get("product");
    if(jsonValue.is<picojson::null>() == false)
        this->product = IAPProduct(jsonValue);
    
    jsonValue = jsonParam.get("iapTransactionId");
    if(jsonValue.is<picojson::null>() == false)
        this->iapTransactionId = jsonValue.to_str();
    
    jsonValue = jsonParam.get("additionalInfo");
    if(jsonValue.is<picojson::null>() == false)
        this->additionalInfo = jsonValue.to_str();
    
    return true;
}

picojson::object IAPReceipt::toJson() const
{
    picojson::object resJson;
    
    resJson["product"] = picojson::value(this->product.toJson());
    resJson["iapTransactionId"] = picojson::value(this->iapTransactionId);
    resJson["additionalInfo"] = picojson::value(this->additionalInfo);
    
    return resJson;
}
std::string IAPReceipt::toString() const
{

    picojson::object resJson = this->toJson();
    std::string resJsonString = picojson::value(resJson).serialize();
    return resJsonString;
}

NS_HIVE_END		// namespace hive



