//
//  HIVE_IAPV4.cpp
//  HIVE_SDK+Plugin
//
//  Created by paikjongman on 2017. 8. 28..
//  Copyright © 2017년 GCP. All rights reserved.
//


#include "HIVE_CppPlugin.h"

NS_HIVE_BEGIN



static IAPV4Type StringToHIVEIAPV4Type(const std::string& str){
    if(str == "APPLE_APPSTORE"){
        return IAPV4Type::AppleAppStore;
    }else if(str == "GOOGLE_PLAYSTORE"){
        return IAPV4Type::GooglePlayStore;
    }else if(str == "HIVE_LEBI"){
        return IAPV4Type::HIVELebi;
    }else if(str == "ONESTORE"){
        return IAPV4Type::OneStore;
    }
    return IAPV4Type::AppleAppStore;
}


static IAPV4::onMarketConnect g_IAPV4MarketConnect;
static IAPV4::onGetProductInfo g_IAPV4GetProductInfo;
static IAPV4::onPurchase g_IAPV4Purchase;
static IAPV4::onRestore g_IAPV4Restore;
// Subscription start
static IAPV4::onGetProductInfo g_IAPV4GetSubscriptionProductInfo;
static IAPV4::onPurchase g_IAPV4PurchaseSubscriptionUpdate;
static IAPV4::onRestore g_IAPV4RestoreSubscription;
// Subscription End
static IAPV4::onTransactionFinish g_IAPV4TransactionFinish;
static IAPV4::onTransactionMultiFinish g_IAPV4TransactionMultiFinish;
static IAPV4::onMarketConnect g_IAPV4ShowMarketSelection;
static IAPV4::onBalance g_IAPV4GetBalanceInfo;
static IAPV4::onBalance g_IAPV4ShowCharge;
static IAPV4::onCheckPromotePurchase g_IAPV4CheckPromotePurchase;

void IAPV4::marketConnect(onMarketConnect listener){
    g_IAPV4MarketConnect = listener;
    picojson::object jsonParam = HiveCppPlugin::createParam("IAPV4", "marketConnect");
    HiveCppPlugin::callNative(jsonParam);
}

void IAPV4::getProductInfo(onGetProductInfo listener){
    g_IAPV4GetProductInfo = listener;
    picojson::object jsonParam = HiveCppPlugin::createParam("IAPV4", "getProductInfo");
    HiveCppPlugin::callNative(jsonParam);
}

void IAPV4::purchase(std::string marketPid,std::string additionalInfo,onPurchase listener){
    g_IAPV4Purchase = listener;
    picojson::object jsonParam = HiveCppPlugin::createParam("IAPV4", "purchase");
    jsonParam["marketPid"] = picojson::value(marketPid);
    jsonParam["additionalInfo"] = picojson::value(additionalInfo);
    HiveCppPlugin::callNative(jsonParam);
}

void IAPV4::restore(onRestore listener){
    g_IAPV4Restore = listener;
    picojson::object jsonParam = HiveCppPlugin::createParam("IAPV4", "restore");
    HiveCppPlugin::callNative(jsonParam);
}

// Subscription Start

void IAPV4::getSubscriptionProductInfo(onGetProductInfo listener){
    g_IAPV4GetProductInfo = listener;
    picojson::object jsonParam = HiveCppPlugin::createParam("IAPV4", "getSubscriptionProductInfo");
    HiveCppPlugin::callNative(jsonParam);
}

void IAPV4::purchaseSubscriptionUpdate(std::string marketPid,std::string oldMarketPid,std::string additionalInfo,onPurchase listener){
    g_IAPV4Purchase = listener;
    picojson::object jsonParam = HiveCppPlugin::createParam("IAPV4", "purchaseSubscriptionUpdate");
    jsonParam["marketPid"] = picojson::value(marketPid);
    jsonParam["oldMarketPid"] = picojson::value(oldMarketPid);
    jsonParam["additionalInfo"] = picojson::value(additionalInfo);
    HiveCppPlugin::callNative(jsonParam);
}

void IAPV4::restoreSubscription(onRestore listener){
    g_IAPV4Restore = listener;
    picojson::object jsonParam = HiveCppPlugin::createParam("IAPV4", "restoreSubscription");
    HiveCppPlugin::callNative(jsonParam);
}

// Subscription End

void IAPV4::transactionFinish(std::string marketPid,onTransactionFinish listener){
    g_IAPV4TransactionFinish = listener;
    picojson::object jsonParam = HiveCppPlugin::createParam("IAPV4", "transactionFinish");
    jsonParam["marketPid"] = picojson::value(marketPid);
    HiveCppPlugin::callNative(jsonParam);
}

void IAPV4::transactionMultiFinish(std::vector<std::string> const & marketPidList,onTransactionMultiFinish listener){
    g_IAPV4TransactionMultiFinish = listener;
    picojson::object jsonParam = HiveCppPlugin::createParam("IAPV4", "transactionMultiFinish");
    picojson::array pidList;
    for(auto it=marketPidList.cbegin();it!=marketPidList.cend();++it){
        pidList.push_back(picojson::value(*it));
    }
    jsonParam["marketPidList"] = picojson::value(pidList);
    HiveCppPlugin::callNative(jsonParam);
}
void IAPV4::showMarketSelection(onMarketConnect listener) {
    g_IAPV4ShowMarketSelection = listener;
    picojson::object jsonParam = HiveCppPlugin::createParam("IAPV4", "showMarketSelection");
    HiveCppPlugin::callNative(jsonParam);
}
void IAPV4::getBalanceInfo(onBalance listener) {
    g_IAPV4GetBalanceInfo = listener;
    picojson::object jsonParam = HiveCppPlugin::createParam("IAPV4", "getBalanceInfo");
    HiveCppPlugin::callNative(jsonParam);
}
void IAPV4::showCharge(onBalance listener) {
    g_IAPV4ShowCharge = listener;
    picojson::object jsonParam = HiveCppPlugin::createParam("IAPV4", "showCharge");
    HiveCppPlugin::callNative(jsonParam);
}
void IAPV4::checkPromotePurchase(onCheckPromotePurchase listener) {
    g_IAPV4CheckPromotePurchase = listener;
    picojson::object jsonParam = HiveCppPlugin::createParam("IAPV4", "checkPromotePurchase");
    HiveCppPlugin::callNative(jsonParam);
}

void IAPV4::executeEngine(picojson::value jsonParam){
    
    ResultAPI result;
    if(!jsonParam.get("resultAPI").is<picojson::null>()){
        result = ResultAPI(jsonParam.get("resultAPI"));
    }
    
    std::string methodName = jsonParam.get("method").to_str();
    
    if(methodName.compare("marketConnect") == 0){
        if(g_IAPV4MarketConnect != nullptr) {
            std::vector<IAPV4Type> iapV4TypeList;
            if(!jsonParam.get("iapV4TypeList").is<picojson::null>()){
                auto jsonList = jsonParam.get("iapV4TypeList").get<picojson::array>();
                for(auto it=jsonList.begin();it!=jsonList.end();++it){
                    iapV4TypeList.push_back(StringToHIVEIAPV4Type((*it).get<std::string>()));
                }
            }
            g_IAPV4MarketConnect(result,iapV4TypeList);
        }
    }
    else if(methodName.compare("getProductInfo") == 0){
        if(g_IAPV4GetProductInfo != nullptr){
            
            std::vector<IAPV4Product> productList;
            unsigned int balance = 0;
            
            if(!jsonParam.get("iapV4ProductList").is<picojson::null>()){
                auto jsonList = jsonParam.get("iapV4ProductList").get<picojson::array>();
                for(auto it=jsonList.begin();it!=jsonList.end();++it){
                    productList.push_back(IAPV4Product(*it));
                }
            }
            if(!jsonParam.get("balance").is<picojson::null>()){
                balance = static_cast<unsigned int>(jsonParam.get("balance").get<double>());
            }
            g_IAPV4GetProductInfo(result,productList,balance);
        }
    }
    else if(methodName.compare("purchase") == 0){
        
        if(g_IAPV4Purchase != nullptr){
            std::shared_ptr<IAPV4Receipt>receipt;
            if(!jsonParam.get("iapV4Receipt").is<picojson::null>()){
                receipt = IAPV4Receipt::create(jsonParam.get("iapV4Receipt"));
            }
            g_IAPV4Purchase(result,*receipt);
        }
    }
    else if(methodName.compare("restore") == 0){
        
        if(g_IAPV4Restore != nullptr){
            std::vector<std::shared_ptr<IAPV4Receipt>> receipts;
            std::vector<std::reference_wrapper<IAPV4Receipt>> receiptRefs;
            if(!jsonParam.get("iapv4ReceiptList").is<picojson::null>()){
                auto iapv4ReceiptList = jsonParam.get("iapv4ReceiptList").get<picojson::array>();
                for(auto it = iapv4ReceiptList.begin();it != iapv4ReceiptList.end();++it){
                    std::shared_ptr<IAPV4Receipt> ptr = IAPV4Receipt::create(*it);
                    receipts.push_back(ptr);
                    receiptRefs.push_back(*ptr);
                }
            }
            g_IAPV4Restore(result,receiptRefs);
        }
    }
    
    // subscription Start
    
    else if(methodName.compare("getSubscriptionProductInfo") == 0){
        if(g_IAPV4GetProductInfo != nullptr){
            
            std::vector<IAPV4Product> productList;
            unsigned int balance = 0;
            
            if(!jsonParam.get("iapV4ProductList").is<picojson::null>()){
                auto jsonList = jsonParam.get("iapV4ProductList").get<picojson::array>();
                for(auto it=jsonList.begin();it!=jsonList.end();++it){
                    productList.push_back(IAPV4Product(*it));
                }
            }
            if(!jsonParam.get("balance").is<picojson::null>()){
                balance = static_cast<unsigned int>(jsonParam.get("balance").get<double>());
            }
            g_IAPV4GetProductInfo(result,productList,balance);
        }
    }
    else if(methodName.compare("purchaseSubscriptionUpdate") == 0){
        
        if(g_IAPV4Purchase != nullptr){
            std::shared_ptr<IAPV4Receipt>receipt;
            if(!jsonParam.get("iapV4Receipt").is<picojson::null>()){
                receipt = IAPV4Receipt::create(jsonParam.get("iapV4Receipt"));
            }
            g_IAPV4Purchase(result,*receipt);
        }
    }
    else if(methodName.compare("restoreSubscription") == 0){
        
        if(g_IAPV4Restore != nullptr){
            std::vector<std::shared_ptr<IAPV4Receipt>> receipts;
            std::vector<std::reference_wrapper<IAPV4Receipt>> receiptRefs;
            if(!jsonParam.get("iapv4ReceiptList").is<picojson::null>()){
                auto iapv4ReceiptList = jsonParam.get("iapv4ReceiptList").get<picojson::array>();
                for(auto it = iapv4ReceiptList.begin();it != iapv4ReceiptList.end();++it){
                    std::shared_ptr<IAPV4Receipt> ptr = IAPV4Receipt::create(*it);
                    receipts.push_back(ptr);
                    receiptRefs.push_back(*ptr);
                }
            }
            g_IAPV4Restore(result,receiptRefs);
        }
    }
    // Subscription End
    else if(methodName.compare("transactionFinish") == 0){
        if(g_IAPV4TransactionFinish != nullptr){
            std::string marketPid = "";
            if(!jsonParam.get("marketPid").is<picojson::null>()){
                marketPid = jsonParam.get("marketPid").get<std::string>();
            }
            g_IAPV4TransactionFinish(result,marketPid);
        }
    }
    else if(methodName.compare("transactionMultiFinish") == 0){
        if(g_IAPV4TransactionMultiFinish != nullptr){
            std::vector<ResultAPI> resultList;
            std::vector<std::string> marketPidList;
            
            if(!jsonParam.get("resultList").is<picojson::null>()){
                auto jsonResultList = jsonParam.get("resultList").get<picojson::array>();
                for(auto it=jsonResultList.begin();it!=jsonResultList.end();++it){
                    resultList.push_back(ResultAPI(*it));
                }
            }
            if(!jsonParam.get("marketPidList").is<picojson::null>()){
                auto jsonMarketPidList = jsonParam.get("marketPidList").get<picojson::array>();
                for(auto it=jsonMarketPidList.begin();it!=jsonMarketPidList.end();++it){
                    marketPidList.push_back((*it).get<std::string>());
                }
            }
            g_IAPV4TransactionMultiFinish(resultList,marketPidList);
        }
    }
    else if(methodName.compare("showMarketSelection") == 0) {
        if(g_IAPV4ShowMarketSelection != nullptr) {
            std::vector<IAPV4Type> iapV4TypeList;
            if(!jsonParam.get("iapV4TypeList").is<picojson::null>()){
                auto jsonList = jsonParam.get("iapV4TypeList").get<picojson::array>();
                for(auto it=jsonList.begin();it!=jsonList.end();++it){
                    iapV4TypeList.push_back(StringToHIVEIAPV4Type((*it).get<std::string>()));
                }
            }
            g_IAPV4ShowMarketSelection(result,iapV4TypeList);
        }
    }
    else if(methodName.compare("getBalanceInfo") == 0) {
        if(g_IAPV4GetBalanceInfo != nullptr){
            unsigned int balance = 0;

            if(!jsonParam.get("balance").is<picojson::null>()){
                balance = static_cast<unsigned int>(jsonParam.get("balance").get<double>());
            }
            g_IAPV4GetBalanceInfo(result,balance);
        }
    }
    else if(methodName.compare("showCharge") == 0) {
        if(g_IAPV4ShowCharge != nullptr) {
            unsigned int balance = 0;
            
            if(!jsonParam.get("balance").is<picojson::null>()){
                balance = static_cast<unsigned int>(jsonParam.get("balance").get<double>());
            }
            g_IAPV4ShowCharge(result,balance);
        }
    }
    else if(methodName.compare("checkPromotePurchase") == 0) {
        if(g_IAPV4CheckPromotePurchase != nullptr){
            std::string marketPid = "";
            if(!jsonParam.get("marketPid").is<picojson::null>()){
                marketPid = jsonParam.get("marketPid").get<std::string>();
            }
            g_IAPV4CheckPromotePurchase(result,marketPid);
        }
    }
}


IAPV4Product::IAPV4Product()
:marketPid("")
,currency("")
,price(0)
,title("")
,displayPrice("")
,productDescription("")
,displayOriginalPrice("")
,originalPrice(0)
,iconURL(""){
}

IAPV4Product::IAPV4Product(picojson::value const& jsonParam){
    HIVE_SETVALUEFROMJSONSTRING(this->marketPid, jsonParam, "marketPid");
    HIVE_SETVALUEFROMJSONSTRING(this->currency, jsonParam, "currency");
    HIVE_SETVALUEFROMJSONNUMBER(this->price, double, jsonParam, "price");
    HIVE_SETVALUEFROMJSONSTRING(this->title, jsonParam, "title");
    HIVE_SETVALUEFROMJSONSTRING(this->displayPrice, jsonParam, "displayPrice");
    HIVE_SETVALUEFROMJSONSTRING(this->productDescription, jsonParam, "productDescription");
    HIVE_SETVALUEFROMJSONSTRING(this->displayOriginalPrice, jsonParam, "displayOriginalPrice");
    HIVE_SETVALUEFROMJSONNUMBER(this->originalPrice, double, jsonParam, "originalPrice");
    HIVE_SETVALUEFROMJSONSTRING(this->iconURL, jsonParam, "iconURL");
}

std::shared_ptr<IAPV4Receipt> IAPV4Receipt::create(picojson::value const& jsonParam){
    IAPV4Type _type = getIAPV4TypeFromJson(jsonParam);
    std::shared_ptr<IAPV4Receipt> receipt;
    
    switch (_type) {
        case IAPV4Type::AppleAppStore:
            receipt = std::make_shared<IAPV4ReceiptApple>(IAPV4ReceiptApple(jsonParam));
            break;
        case IAPV4Type::GooglePlayStore:
            receipt = std::make_shared<IAPV4ReceiptGoogle>(IAPV4ReceiptGoogle(jsonParam));
            break;
        case IAPV4Type::HIVELebi:
            receipt = std::make_shared<IAPV4ReceiptLebi>(IAPV4ReceiptLebi(jsonParam));
            break;
        case IAPV4Type::OneStore:
            receipt = std::make_shared<IAPV4ReceiptOneStoreV5>(IAPV4ReceiptOneStoreV5(jsonParam));
            break;
        default:
            break;
    }
    
    return receipt;
    
}

IAPV4Type IAPV4Receipt::getIAPV4TypeFromJson(picojson::value const& jsonParam){
    auto jsonValue = jsonParam.get("type");
    if(!jsonValue.is<picojson::null>())
        return StringToHIVEIAPV4Type(jsonValue.get<std::string>());
    
    return IAPV4Type::AppleAppStore;
}

IAPV4Receipt::IAPV4Receipt(){
    
}

IAPV4Receipt::IAPV4Receipt(picojson::value const& jsonParam){
    if(jsonParam.is<picojson::null>())
        return;
    
    HIVE_SETVALUEFROMJSONSTRING(this->additionalInfo, jsonParam, "additionalInfo");
    HIVE_SETVALUEFROMJSONSTRING(this->hiveiapReceipt, jsonParam, "hiveiapReceipt");
    HIVE_SETVALUEFROMJSONSTRING(this->bypassInfo, jsonParam, "bypassInfo");
    
    auto jsonValue = jsonParam.get("product");
    if(!jsonValue.is<picojson::null>())
        this->product = IAPV4Product(jsonValue);
    
    this->type = getIAPV4TypeFromJson(jsonParam);
}

IAPV4Receipt::~IAPV4Receipt(){
    
}


IAPV4ReceiptApple::IAPV4ReceiptApple(){
}

IAPV4ReceiptApple::IAPV4ReceiptApple(picojson::value const& jsonParam) : IAPV4Receipt(jsonParam){
}

IAPV4ReceiptApple::~IAPV4ReceiptApple(){
    
}

IAPV4ReceiptApple& IAPV4ReceiptApple::get() const{
    return *const_cast<IAPV4ReceiptApple*>(this);
}


IAPV4ReceiptGoogle::IAPV4ReceiptGoogle()
:purchaseData("")
,signature("")
,itemType("")
,orderId("")
,packageName("")
,sku("")
,developerPayload("")
,token("")
,vid("")
,gameCurrency("")
,gamePrice(0)
,serverId(""){
}

IAPV4ReceiptGoogle::IAPV4ReceiptGoogle(picojson::value const& jsonParam) : IAPV4Receipt(jsonParam){
    
    HIVE_SETVALUEFROMJSONSTRING(this->purchaseData,jsonParam,"purchaseData");
    HIVE_SETVALUEFROMJSONSTRING(this->signature,jsonParam,"signature");
    
    HIVE_SETVALUEFROMJSONSTRING(this->itemType,jsonParam,"itemType");
    HIVE_SETVALUEFROMJSONSTRING(this->orderId,jsonParam,"orderId");
    HIVE_SETVALUEFROMJSONSTRING(this->packageName,jsonParam,"packageName");
    HIVE_SETVALUEFROMJSONSTRING(this->sku,jsonParam,"sku");
    
    HIVE_SETVALUEFROMJSONSTRING(this->developerPayload,jsonParam,"developerPayload");
    HIVE_SETVALUEFROMJSONSTRING(this->token,jsonParam,"token");
    HIVE_SETVALUEFROMJSONSTRING(this->vid,jsonParam,"vid");
    HIVE_SETVALUEFROMJSONSTRING(this->gameCurrency,jsonParam,"gameCurrency");
    
    HIVE_SETVALUEFROMJSONNUMBER(this->gamePrice, double, jsonParam, "gamePrice");
    HIVE_SETVALUEFROMJSONSTRING(this->serverId,jsonParam,"serverId");
}

IAPV4ReceiptGoogle::~IAPV4ReceiptGoogle(){
    
}

IAPV4ReceiptGoogle& IAPV4ReceiptGoogle::get() const{
    return *const_cast<IAPV4ReceiptGoogle*>(this);
}

IAPV4ReceiptLebi::IAPV4ReceiptLebi()
:tradeno(0)
,uid(0)
,vid(0)
,billItemId("")
,gameName("")
,appId("")
,tradeMoney(0)
,tradeDate("")
,itemName("")
,originalJson(""){
}

IAPV4ReceiptLebi::IAPV4ReceiptLebi(picojson::value const& jsonParam) : IAPV4Receipt(jsonParam){
    HIVE_SETVALUEFROMJSONNUMBER(tradeno, long long, jsonParam, "tradeno");
    HIVE_SETVALUEFROMJSONNUMBER(uid, long long, jsonParam, "uid");
    HIVE_SETVALUEFROMJSONNUMBER(vid, long long, jsonParam, "vid");
    
    HIVE_SETVALUEFROMJSONSTRING(this->billItemId, jsonParam, "billItemId");
    HIVE_SETVALUEFROMJSONSTRING(this->gameName, jsonParam, "gameName");
    HIVE_SETVALUEFROMJSONSTRING(this->appId, jsonParam, "appId");
    
    HIVE_SETVALUEFROMJSONNUMBER(tradeMoney, int, jsonParam, "tradeMoney");
    
    HIVE_SETVALUEFROMJSONSTRING(this->tradeDate, jsonParam, "tradeDate");
    HIVE_SETVALUEFROMJSONSTRING(this->itemName, jsonParam, "itemName");
    HIVE_SETVALUEFROMJSONSTRING(this->originalJson, jsonParam, "originalJson");
}

IAPV4ReceiptLebi::~IAPV4ReceiptLebi(){
    
}

IAPV4ReceiptOneStore::IAPV4ReceiptOneStore()
:txid("")
,api_version("")
,identifier("")
,method("")
,code("")
,message("")
,receipt("")
,count(0)
,oneStoreProduct("")
,purchaseVid(""){
}

IAPV4ReceiptOneStore::IAPV4ReceiptOneStore(picojson::value const& jsonParam) : IAPV4Receipt(jsonParam){
    HIVE_SETVALUEFROMJSONSTRING(this->txid, jsonParam, "txid");
    HIVE_SETVALUEFROMJSONSTRING(this->api_version, jsonParam, "api_version");
    HIVE_SETVALUEFROMJSONSTRING(this->identifier, jsonParam, "identifier");
    HIVE_SETVALUEFROMJSONSTRING(this->method, jsonParam, "method");
    HIVE_SETVALUEFROMJSONSTRING(this->code, jsonParam, "code");
    HIVE_SETVALUEFROMJSONSTRING(this->message, jsonParam, "message");
    HIVE_SETVALUEFROMJSONSTRING(this->receipt, jsonParam, "receipt");
    
    HIVE_SETVALUEFROMJSONNUMBER(this->count, int, jsonParam, "count");
    
    HIVE_SETVALUEFROMJSONSTRING(this->oneStoreProduct, jsonParam, "oneStoreProduct");
    HIVE_SETVALUEFROMJSONSTRING(this->purchaseVid, jsonParam, "purchaseVid");
}

IAPV4ReceiptOneStore::~IAPV4ReceiptOneStore(){
    
}

IAPV4ReceiptOneStoreV5::IAPV4ReceiptOneStoreV5()
:orderId("")
,packageName("")
,productId("")
,purchaseTime(0)
,developerPayload("")
,purchaseId("")
,purchaseState(0)
,signature("")
,recurringState(0)
,originPurchaseData(""){
}

IAPV4ReceiptOneStoreV5::IAPV4ReceiptOneStoreV5(picojson::value const& jsonParam) : IAPV4Receipt(jsonParam){
    HIVE_SETVALUEFROMJSONSTRING(this->orderId, jsonParam, "orderId");
    HIVE_SETVALUEFROMJSONSTRING(this->packageName, jsonParam, "packageName");
    HIVE_SETVALUEFROMJSONSTRING(this->productId, jsonParam, "productId");
    HIVE_SETVALUEFROMJSONNUMBER(this->purchaseTime, long, jsonParam, "purchaseTime");
    HIVE_SETVALUEFROMJSONSTRING(this->developerPayload, jsonParam, "developerPayload");
    HIVE_SETVALUEFROMJSONSTRING(this->purchaseId, jsonParam, "purchaseId");
    HIVE_SETVALUEFROMJSONNUMBER(this->purchaseState, int, jsonParam, "purchaseState");
    HIVE_SETVALUEFROMJSONSTRING(this->signature, jsonParam, "signature");
    HIVE_SETVALUEFROMJSONNUMBER(this->recurringState, int, jsonParam, "recurringState");
    HIVE_SETVALUEFROMJSONSTRING(this->originPurchaseData, jsonParam, "originPurchaseData");
}

IAPV4ReceiptOneStoreV5::~IAPV4ReceiptOneStoreV5(){
    
}

NS_HIVE_END
