//
//  HIVE_PlatformHelper.cpp
//  HIVE_SDK_Plugin
//
//  Created by 김재원 on 2017. 3. 3..
//  Copyright © 2017년 GCP. All rights reserved.
//
#include "HIVE_CppPlugin.h"
#include "HIVE_PlatformHelper.h"

NS_HIVE_BEGIN

static PlatformHelper::onHIVEPlatformHelperShareHandler g_shareTextCB = NULL;

static PlatformHelper::onHIVEPlatformHelperShareHandler g_shareMediaCB = NULL;

static PlatformHelper::onRequestUserPermissionsHandler g_requestUserPermissionsCB = NULL;

void PlatformHelper::share(PlatformShare platformShare, onHIVEPlatformHelperShareHandler handler)
{
    picojson::object jsonParam = HiveCppPlugin::createParam("PlatformHelper", "share");
    
    int shareTypeInt = platformShare.getShareTypeInt();
    std::vector<std::string> paths = platformShare.getMedia();
    std::string subject = platformShare.getSubject();
    std::string text = platformShare.getText();
    
    picojson::array jsonArray;
    for (std::vector<std::string>::iterator it = paths.begin(); it != paths.end(); ++it) {
        jsonArray.push_back(picojson::value((*it)));
    }
    jsonParam["shareType"] = picojson::value((double)shareTypeInt);
    jsonParam["shareMedia"] = picojson::value(jsonArray);
    jsonParam["shareSubject"] = picojson::value(subject);
    jsonParam["shareText"] = picojson::value(text);
    
    g_shareMediaCB = handler;
    
    HiveCppPlugin::callNative(jsonParam);
}

void PlatformHelper::requestUserPermissions(std::vector<std::string> requests, onRequestUserPermissionsHandler handler)
{
    picojson::object jsonParam = HiveCppPlugin::createParam("PlatformHelper", "requestUserPermissions");

    picojson::array jsonArray;
    for (std::vector<std::string>::iterator it = requests.begin(); it != requests.end(); ++it) {
        jsonArray.push_back(picojson::value(*it));
    }
    jsonParam["requests"] = picojson::value(jsonArray);

    g_requestUserPermissionsCB = handler;

    HiveCppPlugin::callNative(jsonParam);
}

void PlatformHelper::executeEngine(picojson::value jsonParam)
{
    ResultAPI result(jsonParam.get("resultAPI"));
    
    std::string methodName = jsonParam.get("method").to_str();
    if(methodName.compare("shareText") == 0)
    {
        if(g_shareTextCB != NULL)
        {
            bool isSuccess = jsonParam.get("isSuccess").get<bool>();
            g_shareTextCB(isSuccess);
        }
    }
    else if(methodName.compare("shareMedia") == 0)
    {
        if( g_shareMediaCB != NULL)
        {
            bool isSuccess = jsonParam.get("isSuccess").get<bool>();
            g_shareMediaCB(isSuccess);
        }
        
    }
    else if(methodName.compare("requestUserPermissions") == 0)
    {
        if (g_requestUserPermissionsCB != NULL)
        {
            std::vector<std::string> grantedVector;
            picojson::value grantedJson = jsonParam.get("granted");

            if (!grantedJson.is<picojson::null>()) {
                picojson::array arrayGranted = grantedJson.get<picojson::array>();

                for (auto it=arrayGranted.begin(); it!=arrayGranted.end(); ++it) {
                    std::string str = (*it).to_str();
                    grantedVector.push_back(str);
                }
            }
            std::vector<std::string> deniedVector;
            picojson::value deniedJson = jsonParam.get("denied");

            if (!deniedJson.is<picojson::null>()) {
                picojson::array arrayDenied = jsonParam.get("denied").get<picojson::array>();

                for (auto it=arrayDenied.begin(); it!=arrayDenied.end(); ++it) {
                    std::string str = (*it).to_str();
                    deniedVector.push_back(str);
                }
            }
            g_requestUserPermissionsCB(result, grantedVector, deniedVector);
        }
    }
}

PlatformShare::PlatformShare()
{
    
}

PlatformShare::~PlatformShare()
{
    
}

PlatformShareType PlatformShare::getShareType()
{
    return this->shareType;
}

int PlatformShare::getShareTypeInt()
{
    int typeNum = 2;
    switch(this->shareType){
        case PlatformShareType::TEXT:
            typeNum = 1;
            break;
        case PlatformShareType::MEDIA:
            typeNum = 2;
            break;
        default:
            typeNum = 2;
            break;
    }
    return typeNum;
}

void PlatformShare::setShareType(PlatformShareType shareType)
{
    this->shareType = shareType;
}

std::string PlatformShare::getSubject()
{
    return this->subject;
}

void PlatformShare::setSubject(std::string subject)
{
    this->subject = subject;
}

std::string PlatformShare::getText()
{
    return this->text;
}

void PlatformShare::setText(std::string text)
{
    this->text = text;
}

std::vector<std::string> PlatformShare::getMedia()
{
    return this->paths;
}

void PlatformShare::setMedia(std::vector<std::string> paths)
{
    this->paths = paths;
}

NS_HIVE_END
