


#include "HIVE_CppPlugin.h"


NS_HIVE_BEGIN



static Push::onLocalPush g_onLocalPush = NULL;

static Push::onRemotePush g_onRemotePush = NULL;

static Push::onPushSetting g_onPushSetting = NULL;


void Push::registerLocalPush(LocalPush localPush, onLocalPush listener)
{
	g_onLocalPush = listener;


	picojson::object jsonParam = HiveCppPlugin::createParam("Push", "registerLocalPush");
	jsonParam["localPush"] = picojson::value(localPush.toJson());

	HiveCppPlugin::callNative(jsonParam);
}


void Push::unregisterLocalPush(int noticeId)
{
	picojson::object jsonParam = HiveCppPlugin::createParam("Push", "unregisterLocalPush");
	jsonParam["noticeID"] = picojson::value((double)noticeId);

	HiveCppPlugin::callNative(jsonParam);
}


void Push::getRemotePush(onRemotePush listener)
{
	g_onRemotePush = listener;


	picojson::object jsonParam = HiveCppPlugin::createParam("Push", "getRemotePush");

	HiveCppPlugin::callNative(jsonParam);
}


void Push::setRemotePush(RemotePush remotePush, onRemotePush listener)
{
	g_onRemotePush = listener;


	picojson::object jsonParam = HiveCppPlugin::createParam("Push", "setRemotePush");
	jsonParam["remotePush"] = picojson::value(remotePush.toJson());

	HiveCppPlugin::callNative(jsonParam);
}

// MARK: #12076: Push Setting
void Push::getForegroundPush(onPushSetting listener)
{
    g_onPushSetting = listener;
    
    picojson::object jsonParam = HiveCppPlugin::createParam("Push", "getForegroundPush");
    
    HiveCppPlugin::callNative(jsonParam);
}

void Push::setForegroundPush(PushSetting setting, onPushSetting listener)
{
    g_onPushSetting = listener;
    
    picojson::object jsonParam = HiveCppPlugin::createParam("Push", "setForegroundPush");
    jsonParam["pushSetting"] = picojson::value(setting.toJson());
    
    HiveCppPlugin::callNative(jsonParam);
}

// MARK: -
    
    
void Push::executeEngine(picojson::value jsonParam)
{
	ResultAPI result(jsonParam.get("resultAPI"));

	std::string methodName = jsonParam.get("method").to_str();
	if(methodName.compare("registerLocalPush") == 0)
	{
        if(g_onLocalPush != NULL)
        {
            LocalPush localPush(jsonParam.get("localPush"));
            g_onLocalPush(result, localPush);
        }
	}
	else if(methodName.compare("unregisterLocalPush") == 0)
	{

	}
	else if(methodName.compare("getRemotePush") == 0)
	{
        if(g_onRemotePush != NULL)
        {
        	RemotePush remotePush(jsonParam.get("remotePush"));
            g_onRemotePush(result, remotePush);
        }
	}
	else if(methodName.compare("setRemotePush") == 0)
	{
        if(g_onRemotePush != NULL)
        {
        	RemotePush remotePush(jsonParam.get("remotePush"));
            g_onRemotePush(result, remotePush);
        }
	}
    
    // MARK: #12076: Execute
    else if(methodName.compare("setForegroundPush") == 0 ||
            methodName.compare("getForegroundPush") == 0)
    {
        if(g_onPushSetting != NULL)
        {
            PushSetting pushSetting(jsonParam.get("pushSetting"));
            g_onPushSetting(result, pushSetting);
        }
    }
    // MARK: -
}




// MARK: RemotePush class Implements
RemotePush::RemotePush()
{
}


RemotePush::RemotePush(bool arg_isAgreeNotice, bool arg_isAgreeNight)
{
    this->isAgreeNotice = arg_isAgreeNotice;
    this->isAgreeNight = arg_isAgreeNight;
}


RemotePush::RemotePush(picojson::value jsonParam)
{
    if(jsonParam.is<picojson::null>())
        return;
    
    picojson::value jsonValue;
    
    jsonValue = jsonParam.get("isAgreeNotice");
    if(jsonValue.is<picojson::null>() == false)
        this->isAgreeNotice = jsonValue.get<bool>();
    
    jsonValue = jsonParam.get("isAgreeNight");
    if(jsonValue.is<picojson::null>() == false)
        this->isAgreeNight = jsonValue.get<bool>();
}


picojson::object RemotePush::toJson() const
{
    picojson::object resJson;
    resJson["isAgreeNotice"] = picojson::value(this->isAgreeNotice);
    resJson["isAgreeNight"] = picojson::value(this->isAgreeNight);
    return resJson;
}


std::string RemotePush::toString() const
{
    picojson::object resJson = this->toJson();
    std::string resJsonString = picojson::value(resJson).serialize();
    return resJsonString;
}

// MARK: -

// MARK: LocalPush class Implements
LocalPush::LocalPush()
{
    noticeId = -1;
}

LocalPush::LocalPush(int arg_noticeId, std::string arg_title, std::string arg_msg, long arg_after)
{
    this->noticeId = arg_noticeId;
    this->title = arg_title;
    this->msg = arg_msg;
    this->after = arg_after;
}

LocalPush::LocalPush(picojson::value jsonParam)
{
    if(jsonParam.is<picojson::null>())
        return;
    
    picojson::value jsonValue;
    
    jsonValue = jsonParam.get("noticeID");
    if(jsonValue.is<picojson::null>() == false)
        this->noticeId = static_cast<int>(jsonValue.get<double>());
    
    jsonValue = jsonParam.get("title");
    if(jsonValue.is<picojson::null>() == false)
        this->title = jsonValue.to_str();
    
    jsonValue = jsonParam.get("msg");
    if(jsonValue.is<picojson::null>() == false)
        this->msg = jsonValue.to_str();
    
    jsonValue = jsonParam.get("after");
    if(jsonValue.is<picojson::null>() == false)
        this->after = static_cast<long>(jsonValue.get<double>());
    
    
    // 이하 Android only
    jsonValue = jsonParam.get("bigmsg");
    if(jsonValue.is<picojson::null>() == false)
        this->bigmsg = jsonValue.to_str();
    
    jsonValue = jsonParam.get("ticker");
    if(jsonValue.is<picojson::null>() == false)
        this->ticker = jsonValue.to_str();
    
    jsonValue = jsonParam.get("type");
    if(jsonValue.is<picojson::null>() == false)
        this->type = jsonValue.to_str();
    
    jsonValue = jsonParam.get("icon");
    if(jsonValue.is<picojson::null>() == false)
        this->icon = jsonValue.to_str();
    
    jsonValue = jsonParam.get("sound");
    if(jsonValue.is<picojson::null>() == false)
        this->sound = jsonValue.to_str();
    
    jsonValue = jsonParam.get("active");
    if(jsonValue.is<picojson::null>() == false)
        this->active = jsonValue.to_str();
    
    jsonValue = jsonParam.get("broadcastAction");
    if(jsonValue.is<picojson::null>() == false)
        this->broadcastAction = jsonValue.to_str();
    
    jsonValue = jsonParam.get("buckettype");
    if(jsonValue.is<picojson::null>() == false)
        this->buckettype = static_cast<int>(jsonValue.get<double>());
    
    jsonValue = jsonParam.get("bucketsize");
    if(jsonValue.is<picojson::null>() == false)
        this->bucketsize = static_cast<int>(jsonValue.get<double>());
    
    jsonValue = jsonParam.get("bigpicture");
    if(jsonValue.is<picojson::null>() == false)
        this->bigpicture = jsonValue.to_str();
    
    jsonValue = jsonParam.get("icon_color");
    if(jsonValue.is<picojson::null>() == false)
        this->icon_color = jsonValue.to_str();
}


picojson::object LocalPush::toJson() const
{
    picojson::object resJson;
    resJson["noticeID"] = picojson::value((double)this->noticeId);
    resJson["title"] = picojson::value(this->title);
    resJson["msg"] = picojson::value(this->msg);
    resJson["after"] = picojson::value((double)this->after);
    
    // 이하 Android only
    resJson["bigmsg"] = picojson::value(this->bigmsg);
    resJson["ticker"] = picojson::value(this->ticker);
    resJson["type"] = picojson::value(this->type);
    resJson["icon"] = picojson::value(this->icon);
    resJson["sound"] = picojson::value(this->sound);
    resJson["active"] = picojson::value(this->active);
    resJson["broadcastAction"] = picojson::value(this->broadcastAction);
    resJson["buckettype"] = picojson::value((double)this->buckettype);
    resJson["bucketsize"] = picojson::value((double)this->bucketsize);
    resJson["bigpicture"] = picojson::value(this->bigpicture);
    resJson["icon_color"] = picojson::value(this->icon_color);
    return resJson;
}


std::string LocalPush::toString() const
{
    picojson::object resJson = this->toJson();
    std::string resJsonString = picojson::value(resJson).serialize();
    return resJsonString;
}
// MARK: -

// MARK: #12076: PushSetting class Implements
PushSetting::PushSetting()
{
    this->useForegroundLocalPush = true;
    this->useForegroundRemotePush = true;
}


PushSetting::PushSetting(bool useForegroundRemotePush, bool useForegroundLocalPush)
{
    this->useForegroundLocalPush = useForegroundLocalPush;
    this->useForegroundRemotePush = useForegroundRemotePush;
}

PushSetting::PushSetting(picojson::value jsonParam)
{
    if(jsonParam.is<picojson::null>())
        return;
    
    picojson::value jsonValue;
    
    jsonValue = jsonParam.get("useForegroundRemotePush");
    if(jsonValue.is<picojson::null>() == false)
        this->useForegroundRemotePush = jsonValue.get<bool>();
    
    jsonValue = jsonParam.get("useForegroundLocalPush");
    if(jsonValue.is<picojson::null>() == false)
        this->useForegroundLocalPush = jsonValue.get<bool>();
}


picojson::object PushSetting::toJson() const
{
    picojson::object resJson;
    
    resJson["useForegroundRemotePush"] = picojson::value(this->useForegroundRemotePush);
    resJson["useForegroundLocalPush"] = picojson::value(this->useForegroundLocalPush);
    
    return resJson;
}


std::string PushSetting::toString() const
{
    picojson::object resJson = this->toJson();
    std::string resJsonString = picojson::value(resJson).serialize();
    
    return resJsonString;
}
// MARK: -



NS_HIVE_END		// namespace hive



