package com.hive.sample;


import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import org.json.JSONException;
import org.json.JSONObject;

import com.com2us.hivesdk.normal.freefull.google.global.android.common.R;
import com.hive.Auth;
import com.hive.Auth.Account;
import com.hive.Auth.AuthInitResult;
import com.hive.Auth.AuthMaintenanceInfo;
import com.hive.PermissionView;
import com.hive.PlatformHelper;
import com.hive.ResultAPI;
import com.hive.IAPV4;
import com.hive.Promotion;
import com.hive.Promotion.*;
import com.hive.Auth.LoginType;
import com.hive.Configuration;
import com.hive.base.Android;
import com.hive.impl.userengagement.UserEngagementCoupon;
import com.hive.sample.game.GamePostboxV4Dialog;
import com.hive.sample.game.GameUser;
import com.hive.sample.game.Post;
import com.hive.sample.uihelper.ViewPagerBase;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.Switch;
import android.widget.Toast;




/**
 * HIVE SDK 의 Auth (인증) 기능 테스트 뷰
 * 
 * @author hife
 */
public class AuthTestView extends ViewPagerBase implements View.OnClickListener {
	
	
	private String recentCurrentAccountVid;
	private String recentUsedAccountVid;
	

	public AuthTestView(Activity activity, View view, String title) {
		super(activity, view, title);


		initUi();
	}

	
	@SuppressWarnings({ "unchecked", "rawtypes" })
	private void initUi() {

		createLogView(R.id.logview, R.id.btnLogDialog, R.id.btnLogClear);
		
		
		ArrayAdapter spAdapter;

		this.view.findViewById(R.id.btnAuthRequestPermissionViewData).setOnClickListener(this);
		this.view.findViewById(R.id.btnAuthInitialize).setOnClickListener(this);
		this.view.findViewById(R.id.btnAuthGetLoginType).setOnClickListener(this);
		
		spAdapter = new ArrayAdapter(activity, android.R.layout.simple_spinner_item);
		spAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
		spAdapter.add("Guest");
		spAdapter.add("Account");
		spAdapter.add("Select");
		spAdapter.add("Auto");

		((Spinner)this.view.findViewById(R.id.spLoginType)).setAdapter(spAdapter);
		
		this.view.findViewById(R.id.btnAuthLogin).setOnClickListener(this);
		this.view.findViewById(R.id.btnAuthSelectLogin).setOnClickListener(this);
		this.view.findViewById(R.id.btnAuthLogout).setOnClickListener(this);
		this.view.findViewById(R.id.btnAuthGetAccount).setOnClickListener(this);
		this.view.findViewById(R.id.btnAuthCheckMaintenance).setOnClickListener(this);
		this.view.findViewById(R.id.btnAuthShowTerms).setOnClickListener(this);
		this.view.findViewById(R.id.btnAuthShowAdultConfirm).setOnClickListener(this);
		this.view.findViewById(R.id.btnAuthReset).setOnClickListener(this);
		this.view.findViewById(R.id.btnAuthSetEmergencyMode).setOnClickListener(this);
        this.view.findViewById(R.id.btnUserEngagementSetEngagementListener).setOnClickListener(this);
		this.view.findViewById(R.id.btnUserEngagementSetEngagementReady).setOnClickListener(this);
		this.view.findViewById(R.id.btnPlatformHelperAuthRequestUserPermissions).setOnClickListener(this);
	}


	@Override
	public void onClick(View v) {

		int rid = v.getId();
		
		switch(rid) {

			case R.id.btnAuthRequestPermissionViewData: {
				addLog("\n\n=== HIVE SDK 권한 고지 팝업 데이터 요청 ===\n");
				addLog("Auth.requestPermissionViewData(PermissionViewDataListener) Called\n");

				Auth.requestPermissionViewData(new PermissionView.PermissionViewDataListener() {
					@Override
					public void onPermissionViewData(ResultAPI result, PermissionView.PermissionViewData data) {
						if (result.isSuccess()) {
							addLog(String.format("PermissionViewDataListener.onAuthPermissionViewData() Callback\nresult = %s\n", result.toString()));
							if (result.code == ResultAPI.Code.AuthSkipPermissionView) {
								addLog(String.format("Can skip permission noice"));
							}
							else {
								String popupTitle = data.contents;
								PermissionView.PermissionViewUnitData[] permissionViewUnitDatas = data.permissions;
								StringBuilder sb = new StringBuilder();
								for (PermissionView.PermissionViewUnitData permissionViewUnitData : permissionViewUnitDatas) {
									sb.append("Title : ").append(permissionViewUnitData.title).append("\n");
									sb.append("Description : ").append(permissionViewUnitData.contents).append("\n");
									sb.append("Permission Name : ").append(permissionViewUnitData.nativePermissionName).append("\n\n");
								}

								PermissionView.PermissionViewUnitData[] commonUnitDatas = data.commons;
								for (PermissionView.PermissionViewUnitData commonUnitData : commonUnitDatas) {
									sb.append("Title : ").append(commonUnitData.title).append("\n");
									sb.append("Description : ").append(commonUnitData.contents).append("\n");
									sb.append("Permission Name : ").append(commonUnitData.nativePermissionName).append("\n\n");
								}

								AlertDialog.Builder builder = new AlertDialog.Builder(activity);
								builder.setTitle(popupTitle);
								builder.setMessage(sb.toString());
								builder.setPositiveButton("OK", new DialogInterface.OnClickListener() {
									@Override
									public void onClick(DialogInterface dialog, int which) {
										//
									}
								});
								builder.create().show();
							}
						} else {
							addLog(String.format("PermissionViewDataListener.onAuthPermissionViewData() Callback\nresult = %s\n", result.toString()));
						}
					}
				});
			}
			break;

			case R.id.btnAuthInitialize: {
				
				addLog("\n\n=== HIVE SDK 초기화 요청 ===\n");
				addLog("Auth.initialize(authInitializeListener, engagementListener) Called\n");
				
				com.hive.Auth.initialize(new com.hive.Auth.AuthInitializeListener() {
	
					@Override
					public void onAuthInitialize(final ResultAPI result, final AuthInitResult authInitResult) {
						
						addLog(String.format("AuthInitializeListener.onAuthInitialize() Callback\nresult = %s\n", result.toString()));

						if (result.needExit()) {
//							System.exit(0);
							AlertDialog.Builder builder = new AlertDialog.Builder(activity);
							builder.setMessage("Exit Event received from HIVE SDK. App will be terminated when you press OK button");
							builder.setPositiveButton("OK", new DialogInterface.OnClickListener() {
								@Override
								public void onClick(DialogInterface dialog, int which) {
									Android.finish();
								}
							});
							builder.create().show();
							return;
						} else {
							if (authInitResult != null)
								addLog(String.format("authInitResult = %s\n", authInitResult.toString()));
						}
						
						
						// HIVE SDK 초기화 결과에 따라서 로그인 선택 스피너 UI 변경
						if(result.isSuccess()) {
							Spinner spinner = (Spinner) AuthTestView.this.view.findViewById(R.id.spLoginType);
							spinner.setSelection(authInitResult.loginType.ordinal());
						}
						
						final String msg = result.toString() + (authInitResult != null ? "\nLoginType : " + authInitResult.loginType : "");
						showAlertDialog("Auth Initialize Result", msg);
					}
				});
				
			}
			break;
			
			
			case R.id.btnAuthGetLoginType: {
				
				LoginType loginType = com.hive.Auth.getLoginType();
				
				if(loginType == null) {
					addLog("\n\n=== 로그인 가능 형태 조회 (GUEST, ACCOUNT, AUTO, SELECT) ===\nAuth.getLoginType() Called\nloginType is null, need initialize");
					
					showAlertDialog("Auth getLoginType Failed", "loginType is null, need initialize");
				}
				else {
					
					addLog(String.format("\n\n=== 로그인 가능 형태 조회 (GUEST, ACCOUNT, AUTO, SELECT) ===\nAuth.getLoginType() Called\nloginType = %s\n", loginType.name()));
					Spinner spinner = (Spinner) this.view.findViewById(R.id.spLoginType);
					spinner.setSelection(loginType.ordinal());
					
					Toast toast = Toast.makeText(Configuration.getContext(), "=== getLoginType : " + loginType + " ===", Toast.LENGTH_SHORT);
					toast.show();
				}
				
			}
			break;
			
			
			case R.id.btnAuthLogin: {
				
				Spinner spinner;
				
				spinner = (Spinner)this.view.findViewById(R.id.spLoginType);
				String selectedValue = (String) spinner.getAdapter().getItem(spinner.getSelectedItemPosition());
				LoginType loginType = LoginType.valueOf(selectedValue.toUpperCase(Locale.US));
				
				
				addLog("\n\n=== 선택된 로그인 타입으로 로그인을 요청 ===\n");
				addLog(String.format("Auth.login(loginType = %s, listener) Called\n", loginType.name()));
				
				com.hive.Auth.login(loginType, new com.hive.Auth.AuthLoginListener() {
					
					@Override
					public void onAuthLogin(final ResultAPI result, LoginType loginType, Account currentAccount, Account usedAccount) {
						
						final String msg = String.format("AuthLoginListener.onAuthLogin() Callback\nresult = %s\nloginType = %s\ncurrentAccount = %s\nusedAccount = %s\n"
								, result.toString(), loginType.name()
								, currentAccount!=null?currentAccount.toString():"null", (usedAccount!=null)?usedAccount.toString():"null");
								
						addLog(msg);
						
						/// http://its.withhive.com/issues/394 이슈처리. 정상적인 상황이 아닌경우.
						if (result.isSuccess() && loginType == LoginType.SELECT) {
							recentCurrentAccountVid = currentAccount.vid;
							recentUsedAccountVid = usedAccount.vid;
						}
						
						
						showAlertDialog("Auth Login Result", msg);
						
						
						if(result.isSuccess()) {
							GameUser.getInstance().login(null);
						}
						
						
					}
				});
			}
			break;
			
			
			case R.id.btnAuthSelectLogin: {
				
				// 사용자 선택 창, test
				Map<String, Object> currentVidData = new HashMap<>();
				currentVidData.put("vid", recentCurrentAccountVid);
				Map<String, Object> currentVidGameData = new HashMap<>();
				currentVidGameData.put("Name", "Nam");
				currentVidGameData.put("Level", 20);
				
				currentVidData.put("data", currentVidGameData);
				
				
				Map<String, Object> usedVidData = new HashMap<>();
				usedVidData.put("vid", recentUsedAccountVid);
				Map<String, Object> usedVidGameData = new HashMap<>();
				usedVidGameData.put("Name", "An");
				usedVidGameData.put("Level", 10);
				
				usedVidData.put("data", usedVidGameData);
				
				
				addLog("\n\n=== Auth.login() 메서드를 호출 후 결과 값이 LoginType.SELECT 이면 이 메서드를 호출 하여 사용자를 선택을 요청 ===\n");
				addLog("Auth.selectLogin(currentVidData, usedVidData, listener) Called\n");
				addLog(String.format("currentVidData = %s\n", currentVidData.toString()));
				addLog(String.format("usedVidData = %s\n", usedVidData.toString()));
				
				com.hive.Auth.showLoginSelection(currentVidData, usedVidData, new com.hive.Auth.AuthLoginListener() {
					
					@Override
					public void onAuthLogin(final ResultAPI result, LoginType loginType, Account currentAccount, Account usedAccount) {
						
						final String msg = String.format("AuthLoginListener.onAuthLogin() Callback\nresult = %s\nloginType = %s\ncurrentAccount = %s\nusedAccount = %s\n"
								, result.toString(), loginType.name()
								, currentAccount!=null?currentAccount.toString():"null", (usedAccount!=null)?usedAccount.toString():"null");

						addLog(msg);
						showAlertDialog("Auth SelectLogin Result", msg);
						
						
						if(result.isSuccess()) {
							GameUser.getInstance().login(null);
						}
					}
				});
			}
			break;
			
			
			case R.id.btnAuthLogout: {
				
				addLog("\n\n=== 로그 아웃 요청 ===\n");
				addLog("Auth.logout(listener) Called\n");
				
				com.hive.Auth.logout(new com.hive.Auth.AuthLogoutListener() {
					
					@Override
					public void onAuthLogout(final ResultAPI result) {
						
						addLog(String.format("AuthLogoutListener.onAuthLogout() Callback\nresult = %s\n", result.toString()));

						showAlertDialog("Auth Logout Result", result.toString());
						
						
						GameUser.getInstance().logout(null);
					}
				});
			}
			break;
			
			
			case R.id.btnAuthGetAccount: {
				
				Account account = com.hive.Auth.getAccount();
				
				addLog("\n\n=== HIVE 인증된 사용자의 정보 조회 ===\n");
				
				final String msg = String.format("Auth.getAccount() Called\naccount = %s\n", account!=null?account.toString():"null"); 
				addLog(msg);
				showAlertDialog("Auth Logout Result", msg);
			}
			break;
			
			
			case R.id.btnAuthCheckMaintenance: {
				
				addLog("\n\n=== 점검 상태 표시 및 데이터 반환 ===\n");
				addLog("Auth.checkMaintenance() Called\n");
				
				com.hive.Auth.checkMaintenance(true, new com.hive.Auth.AuthMaintenanceListener() {
					
					@Override
					public void onAuthMaintenance(final ResultAPI result, final AuthMaintenanceInfo authMaintenanceInfo) {
						
						addLog(String.format("AuthMaintenanceListener.onAuthMaintenance() Callback\nresult = %s\nauthMaintenanceInfo = %s\n"
							, result.toString(), authMaintenanceInfo != null ? authMaintenanceInfo.toString() : "null"));


						if (result.needExit()) {
//							System.exit(0);
							AlertDialog.Builder builder = new AlertDialog.Builder(activity);
							builder.setMessage("Exit Event received from HIVE SDK. App will be terminated when you press OK button");
							builder.setPositiveButton("OK", new DialogInterface.OnClickListener() {
								@Override
								public void onClick(DialogInterface dialog, int which) {
									Android.finish();
								}
							});
							builder.create().show();
							return;
						}

						String msg = null;
						
						if(result.isSuccess() && authMaintenanceInfo != null) {
						}
						
						else if (result.isSuccess() && authMaintenanceInfo == null){
							msg = result.toString() + "\n공지가 없습니다.";
						}
						
						else {
							msg = result.toString();
						}
						
						showAlertDialog("Auth CheckMaintenance Result", msg);
					}
				});
				
			}
			break;
			
			
			case R.id.btnAuthShowTerms: {
				
				addLog("\n\n=== 약관 정보 표시 ===\n");
				addLog(String.format("Auth.showTerms(listener) Called\n"));
				
				com.hive.Auth.showTerms(new com.hive.Auth.AuthShowTermsListener() {
					
					@Override
					public void onAuthShowTerms(ResultAPI result) {
						addLog(String.format("AuthShowTermsListener.onAuthShowTerms() Callback\nresult = %s\n", result.toString()));
					}
				});
			}
			break;
			
			
			case R.id.btnAuthShowAdultConfirm: {
				
				addLog("\n\n=== 성인 인증 화면 호출 ===\n");
				addLog("Auth.showAdultConfirm(listener) Called\n");
				
				com.hive.Auth.showAdultConfirm(new com.hive.Auth.AuthAdultConfirmListener() {
					
					@Override
					public void onAuthAdultConfirm(final ResultAPI result) {
						addLog(String.format("AuthAdultConfirmListener.onAuthAdultConfirm() Callback\nresult = %s\n", result.toString()));
						
						showAlertDialog("Auth showAdultConfirm Result", result.toString());
					}
				});
				
				
			}
			break;
			
			
			case R.id.btnAuthReset: {
				
				addLog("\n\n=== 인증 정보를 포함하여 SDK 에서 사용하는 모든 데이터 초기화 (테스트 용) ===\n");
				addLog("Auth.reset() Called\n");
				
				com.hive.Auth.reset();
				
				Toast toast = Toast.makeText(Configuration.getContext(), "=== 인증 정보를 포함하여 SDK 에서 사용하는 모든 데이터 초기화 (테스트 용) ===", Toast.LENGTH_SHORT);
				toast.show();
				
				
				GameUser.getInstance().logout(null);
			}
			break;


			case R.id.btnAuthSetEmergencyMode: {

				addLog("\n\n=== 긴급모드 사용 준비 ===\n");
				addLog("Auth.setEmergencyMode() Called\n");

				boolean ret = com.hive.Auth.setEmergencyMode();

				Toast toast = Toast.makeText(Configuration.getContext(), "=== 긴급모드 설정 결과 === : " + ret, Toast.LENGTH_SHORT);
				toast.show();

				addLog("\n\n=== 긴급모드 설정 결과 === : " + ret + "\n");

				if (ret)
					GameUser.getInstance().login(null);
			}
			break;


			case R.id.btnUserEngagementSetEngagementListener : {

                addLog("\n\n=== Engagement 리스너 등록 ===\n");
                addLog("UserEngagement.setEngagementListener(listener) Called.\n");

				Promotion.setEngagementListener(this.engagementListener);

                addLog("\n\n=== Engagement 리스너 등록 완료 ===\n");
                addLog("UserEngagement.setEngagementListener(listener) complete.");

                Toast toast = Toast.makeText(Configuration.getContext(),
                        "=== Engagement 리스너 등록 완료 ==="
                        , Toast.LENGTH_SHORT);
                toast.show();
			}
			break;

			case R.id.btnUserEngagementSetEngagementReady: {

				addLog("\n\n=== Engagement 작업 수행이 가능함으로 설정 ===\n");
				addLog("UserEngagement.setEngagementReady() Called.\n");

				ResultAPI retVal = Promotion.setEngagementReady(true);

				addLog("\n\n=== Engagement 작업 수행이 가능함으로 설정 결과 ===\n");
				addLog("UserEngagement.setEngagementReady() result : " + retVal);

				Toast toast = Toast.makeText(Configuration.getContext(),
						(retVal.isSuccess())
								? "=== Engagement 작업 수행이 가능함으로 설정 ==="
								: "=== Engagement 작업 수행이 가능함으로 설정되지 않음 ==="
						, Toast.LENGTH_SHORT);
				toast.show();

			}
			break;

			case R.id.btnPlatformHelperAuthRequestUserPermissions: {

				EditText editText = this.view.findViewById(R.id.editTextAuthPermissionList);
				String[] permissionArray = editText.getText().toString().split("\n");
				List<String> requests = Arrays.asList(permissionArray);

				PlatformHelper.requestUserPermissions(requests, new PlatformHelper.RequestUserPermissionsListener() {
					@Override
					public void onRequestUserPermissions(ResultAPI resultApi, List<String> granted, List<String> denied) {
						addLog("\n\n=== OS 권한동의 팝업 재노출 ===\n");
						addLog("resultApi :" + resultApi.toString() + "\n");

						addLog("동의한 권한 목록 : \n");
						for (String name : granted) {
							addLog(name + '\n');
						}
						addLog("\n거절한 권한 목록: \n");
						for (String name : denied) {
							addLog(name + '\n');
						}
					}
				});
			}
			break;

		}
	}
	
	
	
	
	// SDK 가 특정한 조건에서 클라이언트에 개입 (Engagement) 하기 위한 이벤트 리스너
	private EngagementListener engagementListener = new EngagementListener() {
		
		@Override
		public void onEngagement(ResultAPI result, EngagementEventType engagementEventType, EngagementEventState eventState, JSONObject param) {
			
			addLog(String.format("AuthInitializeListener.onEngagement() Callback\nresult = %s\n", result.toString()));
			addLog(String.format("engagementEventType = %s , %s\n", engagementEventType.name(), eventState.name()));
			addLog(String.format("param = %s\n", (param == null ? "null" : param.toString())));
			
			
			switch (engagementEventType)
			{
				case EVENT_TYPE: {
					// Engagement의 전체 시작과 끝.
					if(eventState == EngagementEventState.BEGIN){
						// Engagement의 모든 동작이 수행됨.
					} else if(eventState == EngagementEventState.FINISH) {
						// Engagement의 모든 동작이 수행 완료됨.
					}
				}
				break;

				case AUTH_LOGIN_VIEW: {

					if (eventState == EngagementEventState.START) {
						// HIVE 멤버십 로그인 페이지 노출 요청 시작
					} else if (eventState == EngagementEventState.END) {
						// HIVE 멤버십 로그인 수행 완료. (로그인 완료 / 창닫음 수행 이후 전달됨)

						String loginTypeName = null;
						try {
							loginTypeName = param.getString("loginType");
						} catch (JSONException e) {
							e.printStackTrace();
						}

						if (LoginType.SELECT.name().equals(loginTypeName)) {

							// 사용자 선택 창, test
							Map<String, Object> currentVidData = new HashMap<>();
							JSONObject jCurrentAccount = param.optJSONObject("currentAccount");
							currentVidData.put("vid", jCurrentAccount.optString("vid"));

							Map<String, Object> currentVidGameData = new HashMap<>();
							currentVidGameData.put("Name", "Nam");
							currentVidGameData.put("Level", 20);

							currentVidData.put("data", currentVidGameData);


							Map<String, Object> usedVidData = new HashMap<>();
							JSONObject jUsedAccount = param.optJSONObject("usedAccount");
							usedVidData.put("vid", jUsedAccount.optString("vid"));

							Map<String, Object> usedVidGameData = new HashMap<>();
							usedVidGameData.put("Name", "An");
							usedVidGameData.put("Level", 10);

							usedVidData.put("data", usedVidGameData);


							addLog("Auth.selectLogin(currentVidData, usedVidData, listener) Called\n");
							addLog(String.format("currentVidData = %s\n", currentVidData.toString()));
							addLog(String.format("usedVidData = %s\n", usedVidData.toString()));

							com.hive.Auth.showLoginSelection(currentVidData, usedVidData, new com.hive.Auth.AuthLoginListener() {

								@Override
								public void onAuthLogin(final ResultAPI result, LoginType loginType, Account currentAccount, Account usedAccount) {

									final String msg = String.format("AuthLoginListener.onAuthLogin() Callback\nresult = %s\nloginType = %s\ncurrentAccount = %s\nusedAccount = %s\n"
											, result.toString(), loginType.name()
											, currentAccount != null ? currentAccount.toString() : "null", (usedAccount != null) ? usedAccount.toString() : "null");

									addLog(msg);
									showAlertDialog("Auth SelectLogin Result", msg);


									if (result.isSuccess()) {
										GameUser.getInstance().login(null);
									}
								}
							});


						} else {

							addLog(String.format("로그인이 완료되지 않았습니다.\n"));

							Toast toast = Toast.makeText(Configuration.getContext(), "로그인이 완료되지 않았습니다.\n", Toast.LENGTH_SHORT);
							toast.show();
						}
					}
				}
				break;

				case SOCIAL_CAFE_VIEW: {
					if (eventState == EngagementEventState.START) {
						// HIVE 카페 페이지 노출 요청 시작
					} else if (eventState == EngagementEventState.END) {
						// HIVE 카페 페이지 노출 완료 (HIVE 카페 창 닫은 이후 전달)
					}
				}
				break;

				case SOCIAL_INQUIRY_VIEW: {
					if (eventState == EngagementEventState.START) {
						// 1:1 문의 페이지 노출 요청 시작
					} else if (eventState == EngagementEventState.END) {
						// 1:1 문의 페이지 노출 완료 (1:1문의 창 닫은 이후 전달)
					}
				}
				break;

				case SOCIAL_MESSAGE: {
					if (eventState == EngagementEventState.START) {
						// HIVE 메세지 전송 요청 시작
					} else if (eventState == EngagementEventState.END) {
						// HIVE 메세지 전송 완료
					}

				}
				break;

				case SOCIAL_INVITATION_VIEW: {
					if (eventState == EngagementEventState.START) {
						// HIVE 초대 메시지 전송 요청 시작
					} else if (eventState == EngagementEventState.END) {
						// HIVE 초대 메시지 전송 완료
					}

				}

				break;

				case PROMOTION_VIEW: {
					if (eventState == EngagementEventState.START) {
						// Promotion View 노출 요청 시작
					} else if (eventState == EngagementEventState.END) {
						// Promotion View 노출 완료 (Promotion 창 닫은 이후 전달)
					}


				}
				break;

				case COUPON: {
					if (eventState == EngagementEventState.START) {
						// 쿠폰 소모 요청 시작
					} else if (eventState == EngagementEventState.END) {
						// 쿠폰 소모 요청 완료 및 응답 전달
						UserEngagementCoupon.CouponConsumeResult couponConsumeResult = null;
						String message = null;

						try {
							couponConsumeResult = UserEngagementCoupon.CouponConsumeResult.getCouponConsumeResult(param.getInt("code"));
							message = param.getString("message");
						} catch (JSONException e) {
							e.printStackTrace();
						}
						addLog(String.format("Coupon Code = %s\nCoupon Message = %s\n"
								, couponConsumeResult.name(), message));
						Toast toast = Toast.makeText(Configuration.getContext(), "쿠폰 처리 결과입니다.\nCode : " + couponConsumeResult + "\nmMessage : " + message, Toast.LENGTH_SHORT);
						toast.show();
					}
				}
				break;

				case OFFERWALL_VIEW: {
					if (eventState == EngagementEventState.START) {
						// Offerwall 페이지 노출 요청 시작
					} else if (eventState == EngagementEventState.END) {
						// Offerwall 페이지 노출 완료 (Offerwall 창 닫은 이후 전달)
					}
				}
				break;

				case EVENT: {
					if (eventState == EngagementEventState.START) {
						// host가 game인 경우 해당 데이터 전달
					} else if (eventState == EngagementEventState.END) {
						// host가 game인 경우 해당 데이터 전달
					}

					String api = null;
					String parameter = null;
					try {
						api = param.getString("api");
						parameter = param.getString("param");
					}
					catch (JSONException e) {
						e.printStackTrace();
					}

					addLog(String.format("apiName = %s\napiParam = %s\n", api, parameter));

					Toast toast = Toast.makeText(Configuration.getContext(), "이벤트를 전달받았습니다.\nAPI : " + api + ", Param : "+ parameter, Toast.LENGTH_SHORT);
					toast.show();
				}
				break;


				case IAP_UPDATED: {
					if (eventState == EngagementEventState.START) {
						// 구매내역이 변경되었으니 받아가라는 ( Restore() 를 호출 ) 신호
					} else if (eventState == EngagementEventState.END) {
						// 구매내역이 변경되었으니 받아가라는 ( Restore() 를 호출 ) 신호
					}
					Toast toast = Toast.makeText(Configuration.getContext(), "IAP Purchase Updated.", Toast.LENGTH_SHORT);
					toast.show();
				}
				break;

				//4.5.0 new engagement Callback parameter.
				case IAP_PROMOTE: {
					if (eventState == EngagementEventState.START) {
						// IAP Promote 데이터 존재 여부 전달
					} else if (eventState == EngagementEventState.END) {
						// IAP Promote 데이터 존재 여부 전달
					}
				}
				break;

				case IAP_PURCHASE: {
					if (eventState == EngagementEventState.START) {
						// IAP Purhcase 구매 시작
					} else if (eventState == EngagementEventState.END) {
						// IAP Purhcase 구매 완료 (+영수증데이터)
						IAPV4.IAPV4Receipt iapV4Receipt = null;

						try {
							iapV4Receipt = (IAPV4.IAPV4Receipt) param.get("iapV4Receipt");
						} catch (JSONException e) {
							e.printStackTrace();
						}
						StringBuilder builder = new StringBuilder();
						builder.append(String.format("IAPV4PurchaseListener.onIAPV4Purchase() Callback - result = %s\n", result))
								.append("IAPV4Receipt : ").append(iapV4Receipt).append("\n");


						//                                                forLog.addLog(builder.toString());
						addLog(builder.toString());

						if (result.isSuccess() && iapV4Receipt != null) {

							GamePostboxV4Dialog.addPost(new Post(
									iapV4Receipt.product.marketPid,
									iapV4Receipt.product.currency,
									(int) iapV4Receipt.product.price,
									iapV4Receipt.type.toString(),
									iapV4Receipt.additionalInfo,
									new SimpleDateFormat("yyyy/MM/dd HH:mm:ss").format(Calendar.getInstance().getTime())
							));

						}
						showAlertDialog("Purchase Result", result.toString() + "\n" + builder.toString());
					}
				}
				break;
				default:
					break;

			}
		}
	};
}



