package com.hive.sample;

import android.app.Activity;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.Spinner;
import android.widget.Toast;

import com.com2us.hivesdk.normal.freefull.google.global.android.common.R;
import com.hive.AuthV4;
import com.hive.Configuration;
import com.hive.ResultAPI;
import com.hive.sample.game.GameUser;
import com.hive.sample.uihelper.ViewPagerBase;

import java.util.ArrayList;
import java.util.Locale;


/**
 * HIVE SDK 의 AuthV4 (인증) 기능 테스트 뷰
 *
 * @author hife
 *
 * Created by hife on 2017. 4. 11..
 */
public class AuthV4HelperTestView extends ViewPagerBase implements View.OnClickListener {


    private AuthV4.PlayerInfo recentCurrentPlayerInfo = new AuthV4.PlayerInfo();
    private AuthV4.PlayerInfo recentConflictPlayerInfo = new AuthV4.PlayerInfo();

    int achievementScore = 0;
    int leaderboardScore = 0;

    public AuthV4HelperTestView(Activity activity, View view, String title) {
        super(activity, view, title);


        initUi();
    }


    @SuppressWarnings({ "unchecked", "rawtypes" })
    private void initUi() {

        createLogView(R.id.logview, R.id.btnLogDialog, R.id.btnLogClear);


        ArrayAdapter spProviderAdapter;

        this.view.findViewById(R.id.btnAuthV4HelperSetup).setOnClickListener(this);
        this.view.findViewById(R.id.btnAuthV4HelperGetSignInType).setOnClickListener(this);

        this.view.findViewById(R.id.btnAuthV4HelperSyncAccount).setOnClickListener(this);
        this.view.findViewById(R.id.btnAuthV4HelperSignIn).setOnClickListener(this);

        spProviderAdapter = new ArrayAdapter(activity, android.R.layout.simple_spinner_item);
        spProviderAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        spProviderAdapter.add("GUEST");
        spProviderAdapter.add("HIVE");
        spProviderAdapter.add("FACEBOOK");
        spProviderAdapter.add("GOOGLE");
        spProviderAdapter.add("QQ");
        spProviderAdapter.add("WEIBO");
        spProviderAdapter.add("VK");
        spProviderAdapter.add("WECHAT");
        spProviderAdapter.add("APPLE");
        spProviderAdapter.add("SIGNIN_APPLE");
        spProviderAdapter.add("LINE");
        spProviderAdapter.add("TWITTER");
        spProviderAdapter.add("AUTO");

        ((Spinner)this.view.findViewById(R.id.spProviderTypeHelper)).setAdapter(spProviderAdapter);

        this.view.findViewById(R.id.btnAuthV4HelperSignOut).setOnClickListener(this);
        this.view.findViewById(R.id.btnAuthV4HelperConnect).setOnClickListener(this);
        this.view.findViewById(R.id.btnAuthV4HelperDisconnect).setOnClickListener(this);
        this.view.findViewById(R.id.btnAuthV4HelperAchievementsReveal).setOnClickListener(this);
        this.view.findViewById(R.id.btnAuthV4HelperAchievementsUnlock).setOnClickListener(this);
        this.view.findViewById(R.id.btnAuthV4HelperAchievementsIncrement).setOnClickListener(this);
        this.view.findViewById(R.id.btnAuthV4HelperShowAchievements).setOnClickListener(this);
        this.view.findViewById(R.id.btnAuthV4HelperLeaderboardsSubmitScore).setOnClickListener(this);
        this.view.findViewById(R.id.btnAuthV4HelperShowLeaderboard).setOnClickListener(this);
        this.view.findViewById(R.id.btnAuthV4HelperShowConflict1).setOnClickListener(this);
        this.view.findViewById(R.id.btnAuthV4HelperShowConflict2).setOnClickListener(this);
        this.view.findViewById(R.id.btnAuthV4HelperResolveConflict).setOnClickListener(this);
        this.view.findViewById(R.id.btnAuthV4HelperSwitchAccount).setOnClickListener(this);
        this.view.findViewById(R.id.btnAuthV4HelperGetIDPList).setOnClickListener(this);

    }


    @Override
    public void onClick(View v) {

        int rid = v.getId();

        switch(rid) {

            case R.id.btnAuthV4HelperSetup: {

                addLog("\n\n=== HIVE SDK 초기화 요청 ===\n");
                addLog("AuthV4.setup(authV4SetupListener, engagementListener) Called\n");

                AuthV4.setup(new AuthV4.AuthV4SetupListener() {
                    @Override
                    public void onAuthV4Setup(ResultAPI result, boolean isAutoSignIn, String did, ArrayList<AuthV4.ProviderType> providerTypeList) {

                        addLog(String.format("AuthV4SetupListener.onAuthV4Setup() Callback\nresult = %s\n", result.toString()));
                        addLog(String.format("signInType = %b\n", isAutoSignIn));

                        // HIVE SDK 초기화 결과에 따라서 로그인 선택 스피너 UI 변경
                        if(result.isSuccess()) {
                            Spinner spinner = (Spinner) AuthV4HelperTestView.this.view.findViewById(R.id.spProviderTypeHelper);
                            spinner.setSelection(isAutoSignIn ? AuthV4.ProviderType.AUTO.ordinal() : AuthV4.ProviderType.GUEST.ordinal());
                        }
                        String typeList = "";
                        if (providerTypeList != null) {
                            for (AuthV4.ProviderType type :
                                    providerTypeList) {
                                typeList = typeList + "\n  " + type;
                            }
                        }

                        final String msg = result.toString()
                             + "\nisAutoSignIn: " + isAutoSignIn
                             + "\ndid: " + did
                             + "\ntypeList: " + typeList;

                        showAlertDialog("AuthV4 Setup Result", msg);
                     }
                 });

            }
            break;


            case R.id.btnAuthV4HelperGetSignInType: {

                boolean isAutoSignIn = AuthV4.isAutoSignIn();

                addLog(String.format("\n\n=== 현재 사인-인 형태 조회 (GUEST, AUTO) ===\n" +
                        "AuthV4.getSignInType() Called\nsignInType = %s\n", isAutoSignIn ? "AUTO" : "GUEST" ));
                Spinner spinner = (Spinner) this.view.findViewById(R.id.spProviderTypeHelper);
                spinner.setSelection(isAutoSignIn ? AuthV4.ProviderType.AUTO.ordinal() : AuthV4.ProviderType.GUEST.ordinal());

                Toast toast = Toast.makeText(Configuration.getContext(), "=== isAutoSignIns: " + isAutoSignIn + " ===", Toast.LENGTH_SHORT);
                toast.show();
            }
            break;

            case R.id.btnAuthV4HelperSyncAccount: {

                Spinner spinner;

                spinner = (Spinner)this.view.findViewById(R.id.spProviderTypeHelper);
                String selectedValue = (String) spinner.getAdapter().getItem(spinner.getSelectedItemPosition());

                AuthV4.ProviderType providerType = AuthV4.ProviderType.getProviderType(selectedValue.toUpperCase(Locale.US));

                AuthV4.Helper.syncAccount(providerType, new AuthV4.Helper.AuthV4HelperListener() {
                    @Override
                    public void onAuthV4Helper(ResultAPI result, AuthV4.PlayerInfo playerInfo) {
                        String msg = "AuthV4HelperListener.onAuthV4Helper() Callback"
                                + "\n" + result
                                + "\n" + playerInfo;

                        addLog(msg);

                        showAlertDialog("AuthV4 Helper syncAccount Result", msg);

                        if(result.isSuccess()) {
                            GameUser.getInstance().login(null);
                        }
                    }
                });
            }
            break;


            case R.id.btnAuthV4HelperSignIn: {

                Spinner spinner;

                spinner = (Spinner)this.view.findViewById(R.id.spProviderTypeHelper);
                String selectedValue = (String) spinner.getAdapter().getItem(spinner.getSelectedItemPosition());

                AuthV4.ProviderType providerType = AuthV4.ProviderType.getProviderType(selectedValue.toUpperCase(Locale.US));

                addLog("\n\n=== 자동으로 사인-인을 요청 ===\n");
                addLog(String.format("AuthV4.Helper.signIn(listener) Called\n"));

                AuthV4.Helper.signIn(new AuthV4.Helper.AuthV4HelperListener() {
                    @Override
                    public void onAuthV4Helper(ResultAPI result, AuthV4.PlayerInfo playerInfo) {
                        String msg = "AuthV4HelperListener.onAuthV4Helper() Callback"
                                + "\n" + result
                                + "\n" + playerInfo;

                        addLog(msg);

                        showAlertDialog("AuthV4 Helper SignIn Result", msg);

                        if(result.isSuccess()) {
                            GameUser.getInstance().login(null);
                        }
                    }
                });
            }
            break;


            case R.id.btnAuthV4HelperSignOut: {

                addLog("\n\n=== 사인 아웃 요청 ===\n");
                addLog("AuthV4.Helper.signOut(listener) Called\n");

                AuthV4.Helper.signOut(new AuthV4.Helper.AuthV4HelperListener() {
                    @Override
                    public void onAuthV4Helper(ResultAPI result, AuthV4.PlayerInfo playerInfo) {
                        addLog(String.format("AuthV4HelperListener.onAuthV4Helper() Callback\nresult = %s\n", result.toString()));

                        showAlertDialog("AuthV4 Helper SignOut Result", result.toString());

                        GameUser.getInstance().logout(null);
                    }
                });
            }
            break;


            case R.id.btnAuthV4HelperConnect: {

                Spinner spinner;

                spinner = (Spinner)this.view.findViewById(R.id.spProviderTypeHelper);
                String selectedValue = (String) spinner.getAdapter().getItem(spinner.getSelectedItemPosition());

                AuthV4.ProviderType providerType = AuthV4.ProviderType.getProviderType(selectedValue.toUpperCase(Locale.US));

                addLog("\n\n=== Provider Connect 요청 ===\n");
                addLog(String.format("AuthV4.Helper.connect(providerType = %s, listener) Called\n", providerType.name()));

//                AuthV4.connect(providerType, new AuthV4.AuthV4ConnectListener() {
//                    @Override
//                    public void onAuthV4Connect(ResultAPI result, AuthV4.PlayerInfo conflictPlayer) {
//
//                        String msg = "AuthV4ConnectListener.onAuthV4Connect() Callback"
//                                + "\n" + result.toString()
//                                + "\nCONFLICT: \n" + conflictPlayer;
//
//                        addLog(msg);
//
//                        showAlertDialog("AuthV4 connect Result", msg);
//
//                        if(result.code == ResultAPI.Code.AuthV4ConflictPlayer && conflictPlayer != null) {
//                            recentCurrentPlayerInfo = AuthV4.getPlayerInfo();
//                            recentConflictPlayerInfo = conflictPlayer;
//                        }
//
//                    }
//                });

                AuthV4.Helper.connect(providerType, new AuthV4.Helper.AuthV4HelperListener() {
                    @Override
                    public void onAuthV4Helper(ResultAPI result, AuthV4.PlayerInfo playerInfo) {

                        String msg = "AuthV4HelperListener.onAuthV4Helper() Callback"
                                + "\n" + result.toString()
                                + "\nplayerInfo: \n" + playerInfo;

                        addLog(msg);

                        showAlertDialog("AuthV4 Helper connect Result", msg);

                        if(result.code == ResultAPI.Code.AuthV4ConflictPlayer && playerInfo != null) {
                            recentCurrentPlayerInfo = AuthV4.getPlayerInfo();
                            recentConflictPlayerInfo = playerInfo;
                        }
                    }
                });
            }
            break;


            case R.id.btnAuthV4HelperDisconnect: {

                Spinner spinner;

                spinner = (Spinner)this.view.findViewById(R.id.spProviderTypeHelper);
                String selectedValue = (String) spinner.getAdapter().getItem(spinner.getSelectedItemPosition());

                AuthV4.ProviderType providerType = AuthV4.ProviderType.getProviderType(selectedValue.toUpperCase(Locale.US));

                addLog("\n\n=== Provider Disconnect 요청 ===\n");
                addLog(String.format("AuthV4.Helper.disconnect(providerType = %s, listener) Called\n", providerType.name()));


//                AuthV4.Helper.disconnect(providerType, new AuthV4.AuthV4DisconnectListener() {
//                    @Override
//                    public void onAuthV4Disconnect(ResultAPI result) {
//
//                        String msg = "AuthV4ConnectListener.onAuthV4Connect() Callback"
//                                + "\n" + result.toString();
//
//                        addLog(msg);
//
//                        showAlertDialog("AuthV4 connect Result", msg);
//                    }
//                });

                AuthV4.Helper.disconnect(providerType, new AuthV4.Helper.AuthV4HelperListener() {
                    @Override
                    public void onAuthV4Helper(ResultAPI result, AuthV4.PlayerInfo playerInfo) {

                        String msg = "AuthV4HelperListener.onAuthV4Helper() Callback"
                                + "\n" + result.toString();

                        addLog(msg);

                        showAlertDialog("AuthV4 Helper disconnect Result", msg);
                    }
                });
            }
            break;

            case R.id.btnAuthV4HelperAchievementsReveal: {

                addLog("\n\n=== 업적 숨김 해제 요청 ===\n");
                addLog("AuthV4.Helper.achievementsReveal() Called\n");

                AuthV4.Helper.achievementsReveal("CgkIzev2g9MJEAIQAQ", new AuthV4.Helper.AuthV4HelperListener() {
                    @Override
                    public void onAuthV4Helper(ResultAPI result, AuthV4.PlayerInfo playerInfo) {
                        if(result.isSuccess()){

                        }else{

                        }
                        String title = "AuthV4HelperAchievementsReveal";
                        String msg = "AuthV4HelperListener.onAuthV4Helper() Callback"
                                + "\n" + result.toString();
                        showAlertDialog(title, msg);
                    }
                });

            }
            break;


            case R.id.btnAuthV4HelperAchievementsUnlock: {

                addLog("\n\n=== 업적 획득 요청 ===\n");
                addLog("AuthV4.Helper.achievementsUnlock() Called\n");

                AuthV4.Helper.achievementsUnlock("CgkIzev2g9MJEAIQAw", new AuthV4.Helper.AuthV4HelperListener() {
                    @Override
                    public void onAuthV4Helper(ResultAPI result, AuthV4.PlayerInfo playerInfo) {
                        if(result.isSuccess()){

                        }else{

                        }
                        String title = "AuthV4HelperAchievementsUnlock";
                        String msg = "AuthV4HelperListener.onAuthV4Helper() Callback"
                                + "\n" + result.toString();
                        showAlertDialog(title, msg);
                    }
                });

            }
            break;


            case R.id.btnAuthV4HelperAchievementsIncrement: {

                addLog("\n\n=== 업적 단계 증가 (+1) / 5 ===\n");
                addLog("AuthV4.Helper.achievementsIncrement() Called\n");

                achievementScore++;

                AuthV4.Helper.achievementsIncrement("CgkIzev2g9MJEAIQAg", 1);
                AuthV4.Helper.achievementsIncrement("CgkIzev2g9MJEAIQBQ", 1);
                AuthV4.Helper.achievementsIncrement("CgkIzev2g9MJEAIQBg", 1, new AuthV4.Helper.AuthV4HelperListener() {
                    @Override
                    public void onAuthV4Helper(ResultAPI result, AuthV4.PlayerInfo playerInfo) {
                        if (result.isSuccess()) {

                        } else {

                        }
                        String title = "AuthV4HelperAchievementsIncrement";
                        String msg = "AuthV4HelperListener.onAuthV4Helper() Callback"
                                + "\n" + result.toString();
                        showAlertDialog(title, msg);
                    }
                });

            }
            break;

            case R.id.btnAuthV4HelperShowAchievements: {

                addLog("\n\n=== Show Achievements 요청 ===\n");
                addLog(String.format("AuthV4.Helper.showAchievements(listener) Called\n"));

                AuthV4.Helper.showAchievements(new AuthV4.Helper.AuthV4HelperListener() {
                    @Override
                    public void onAuthV4Helper(ResultAPI result, AuthV4.PlayerInfo playerInfo) {

                        String msg = "AuthV4HelperListener.onAuthV4Helper() Callback"
                                + "\n" + result.toString()
                                + "\nplayerInfo: \n" + playerInfo;

                        addLog(msg);

                        showAlertDialog("AuthV4 Helper showAchievements Result", msg);

                        if(result.code == ResultAPI.Code.AuthV4ConflictPlayer && playerInfo != null) {
                            recentCurrentPlayerInfo = AuthV4.getPlayerInfo();
                            recentConflictPlayerInfo = playerInfo;
                        }
                    }
                });
            }
            break;

            case R.id.btnAuthV4HelperLeaderboardsSubmitScore: {

                addLog("\n\n=== 리더보드 점수 증가 (+10) ===\n");
                addLog("AuthV4.Helper.achievementsIncrement() Called\n");

                AuthV4.Helper.leaderboardsSubmitScore("CgkIzev2g9MJEAIQBw", leaderboardScore += 10, new AuthV4.Helper.AuthV4HelperListener() {
                    @Override
                    public void onAuthV4Helper(ResultAPI result, AuthV4.PlayerInfo playerInfo) {
                        if (result.isSuccess()) {

                        } else {

                        }

                        String title = "AuthV4HelperLeaderboardsSubmitScore";
                        String msg = "AuthV4HelperListener.onAuthV4Helper() Callback"
                                + "\n" + result.toString();
                        showAlertDialog(title, msg);
                    }
                });
            }
            break;

            case R.id.btnAuthV4HelperShowLeaderboard: {

                addLog("\n\n=== Show Leaderboard 요청 ===\n");
                addLog(String.format("AuthV4.Helper.showAchievements(listener) Called\n"));

                AuthV4.Helper.showLeaderboard(new AuthV4.Helper.AuthV4HelperListener() {
                    @Override
                    public void onAuthV4Helper(ResultAPI result, AuthV4.PlayerInfo playerInfo) {

                        String msg = "AuthV4HelperListener.onAuthV4Helper() Callback"
                                + "\n" + result.toString()
                                + "\nplayerInfo: \n" + playerInfo;

                        addLog(msg);

                        showAlertDialog("AuthV4 Helper showLeaderboards Result", msg);

                        if(result.code == ResultAPI.Code.AuthV4ConflictPlayer && playerInfo != null) {
                            recentCurrentPlayerInfo = AuthV4.getPlayerInfo();
                            recentConflictPlayerInfo = playerInfo;
                        }
                    }
                });
            }
            break;

            case R.id.btnAuthV4HelperShowConflict1: {

                addLog("\n\n=== Show Conflict 요청 ===\n");
                addLog(String.format("AuthV4.Helper.showConflict(listener) Called\n"));

                AuthV4.Helper.showConflict(new AuthV4.Helper.AuthV4HelperListener() {
                    @Override
                    public void onAuthV4Helper(ResultAPI result, AuthV4.PlayerInfo playerInfo) {

                        String msg = "AuthV4HelperListener.onAuthV4Helper() Callback"
                                + "\n" + result.toString()
                                + "\nplayerInfo: \n" + playerInfo;

                        addLog(msg);

                        showAlertDialog("AuthV4 Helper showConflict Result", msg);

                        if(result.code == ResultAPI.Code.AuthV4ConflictPlayer && playerInfo != null) {
                            recentCurrentPlayerInfo = AuthV4.getPlayerInfo();
                            recentConflictPlayerInfo = playerInfo;
                        }
                    }
                });
            }
            break;

            case R.id.btnAuthV4HelperShowConflict2: {

                AuthV4.Helper.ConflictSingleViewInfo viewInfo = new AuthV4.Helper.ConflictSingleViewInfo(recentConflictPlayerInfo.playerId);
                viewInfo.setValue("Name", "ConflictPlayer-" + recentConflictPlayerInfo.playerId);
                viewInfo.setValue("Level", 10);

                addLog("\n\n=== Show Conflict 요청 ===\n");
                addLog(String.format("AuthV4.Helper.showConflict(currentPlayerData, conflictPlayerData, listener) Called\n"));

                AuthV4.Helper.showConflict(viewInfo, new AuthV4.Helper.AuthV4HelperListener() {
                    @Override
                    public void onAuthV4Helper(ResultAPI result, AuthV4.PlayerInfo playerInfo) {

                        String msg = "AuthV4HelperListener.onAuthV4Helper() Callback"
                                + "\n" + result.toString()
                                + "\nplayerInfo: \n" + playerInfo;

                        addLog(msg);

                        showAlertDialog("AuthV4 Helper showConflict Result", msg);

                        if(result.code == ResultAPI.Code.AuthV4ConflictPlayer && playerInfo != null) {
                            recentCurrentPlayerInfo = AuthV4.getPlayerInfo();
                            recentConflictPlayerInfo = playerInfo;
                        }
                    }
                });
            }
            break;


            case R.id.btnAuthV4HelperResolveConflict: {

                addLog("AuthV4.Helper.switchAccount(listener) Called\n");

                AuthV4.Helper.resolveConflict(new AuthV4.Helper.AuthV4HelperListener() {
                    @Override
                    public void onAuthV4Helper(ResultAPI result, AuthV4.PlayerInfo playerInfo) {

                        String msg = "AuthV4HelperListener.onAuthV4Helper() Callback"
                                + "\n" + result.toString();

                        addLog(msg);

                        showAlertDialog("AuthV4 Helper Resolve Conflict Result", msg);
                    }
                });

            }
            break;


            case R.id.btnAuthV4HelperSwitchAccount: {

                addLog("AuthV4.Helper.switchAccount(listener) Called\n");

                AuthV4.Helper.switchAccount(new AuthV4.Helper.AuthV4HelperListener() {
                    @Override
                    public void onAuthV4Helper(ResultAPI result, AuthV4.PlayerInfo playerInfo) {

                        String msg = "AuthV4HelperListener.onAuthV4Helper() Callback"
                                + "\n" + result.toString();

                        addLog(msg);

                        showAlertDialog("AuthV4 Helper Switch Account Result", msg);
                    }
                });

            }
            break;


            case R.id.btnAuthV4HelperGetIDPList: {

                addLog("\n\n=== IDP List 조회 ===\n");

                ArrayList<AuthV4.ProviderType> idpList = AuthV4.Helper.getIDPList();

                addLog( String.format("AuthV4.Helper.getIDPList() Called\n") );

                String msg = "";

                for (int i=0; i<idpList.size(); ++i) {
                    msg = msg + idpList.get(i) + "\n";
                }

                addLog(msg);

                showAlertDialog("AuthV4 Get Info Result", msg);
            }
            break;

        }
    }
}



