package com.hive.sample;


import java.io.InputStream;
import java.util.LinkedHashMap;
import java.util.Locale;
import java.util.Map;

import org.json.JSONObject;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.content.DialogInterface;
import android.os.Bundle;
import android.os.Handler;
import android.view.View;
import android.util.Log;
import android.widget.ArrayAdapter;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.Switch;
import android.widget.TextView;
import android.widget.ToggleButton;
import androidx.annotation.StringRes;


import com.hive.Configuration;
import com.hive.PermissionView;
import com.hive.base.Android;
import com.hive.base.Property;
import com.hive.base.Resource;
import com.hive.Const;
import com.hive.Configuration.ZoneType;

import com.hive.sample.uihelper.ViewPagerBase;

import com.com2us.hivesdk.normal.freefull.google.global.android.common.R;




/**
 * HIVE SDK 의 Configuration 설정 대화 상자
 * 
 * @author ryuvsken
 */
public class ConfigurationDialog extends Dialog {

	private Activity activity;
	
	
	// 앱의 고유 ID
	public String appId;
	
	// HIVE SDK 가 접속하는 플랫폼 서비스의 형태 (sandbox, real)
	public ZoneType zoneType = ZoneType.SANDBOX;
	
	// 사용자가 선택한 게임 언어
	public String gameLanguage = "en";
	
	// 사용자가 선택한 게임 서버
	public String gameServerId = "kr";
	
	
	// 이하 HIVE SDK 동작 로그 관련
	public Boolean isLogging = false;
	
	public String loggingUserId = null;
	
	public Boolean isRemoteLogging = false;
	
	public Boolean isRemoteDetailLogging = false;

	public Boolean isAgeGateU13 = false;
	
	
	public Property property;

	
	public ConfigurationDialog(Activity activity) {
		super(activity);

		this.activity = activity;
		
		this.property = new Property(activity, "sample.prop");
		this.property.setIsPrefixFileName(false);
		
		
		languageMap.put("en", new LanguageDisplayContent("en", Locale.ENGLISH, R.string.hive_language_en));
		languageMap.put("ko", new LanguageDisplayContent("ko", Locale.KOREAN, R.string.hive_language_ko));
		languageMap.put("zh-Hans", new LanguageDisplayContent("zh-Hans", Locale.SIMPLIFIED_CHINESE, R.string.hive_language_zh_Hans));
		languageMap.put("zh-Hant", new LanguageDisplayContent("zh-Hant", Locale.TRADITIONAL_CHINESE, R.string.hive_language_zh_Hant));
		languageMap.put("ja", new LanguageDisplayContent("ja", Locale.JAPANESE, R.string.hive_language_ja));
		languageMap.put("ru", new LanguageDisplayContent("ru", new Locale("ru"), R.string.hive_language_ru));
		languageMap.put("fr", new LanguageDisplayContent("fr", Locale.FRENCH, R.string.hive_language_fr));
		languageMap.put("de", new LanguageDisplayContent("de", Locale.GERMAN, R.string.hive_language_de));
		languageMap.put("es", new LanguageDisplayContent("es", new Locale("es"), R.string.hive_language_es));
		languageMap.put("pt", new LanguageDisplayContent("pt", new Locale("pt"), R.string.hive_language_pt));
		languageMap.put("id", new LanguageDisplayContent("id", new Locale("id"), R.string.hive_language_id));
//		languageMap.put("ms", new LanguageDisplayContent("ms", new Locale("ms"), R.string.hive_language_ms));// ms_MY
		languageMap.put("th", new LanguageDisplayContent("th", new Locale("th"), R.string.hive_language_th));
		languageMap.put("vi", new LanguageDisplayContent("vi", new Locale("vi"), R.string.hive_language_vi));// vi_VN
		languageMap.put("it", new LanguageDisplayContent("it", Locale.ITALIAN, R.string.hive_language_it));
		languageMap.put("tr", new LanguageDisplayContent("tr", new Locale("tr"), R.string.hive_language_tr));
	}
	
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		int dialogId = Resource.getLayoutId(getContext(), "test_view_config");
		setContentView(dialogId);
		
		setTitle("Select to HIVE SDK Configuration");
		
		this.buildUI();
	}

	// 출력 view에 값을 저장하기 위한 폴링 함수.
	// 광고 아이디는 구글 서버로부터 asynchronous 하게 수신되므로,
	// UI thread 에 handler를 이용하여 post 한다.
	private void updateAdvertisingIdTextView() {

		final Handler handler = new Handler();
		
		new Thread( new Runnable() {
 
			@Override
			public void run() {
				// TODO Auto-generated method stub
				do {					
					if (com.hive.base.Android.getAdvertisingId() == null) {
						try {
							Thread.sleep(500);
						} 
						catch (Exception e) {							
						}
					}
					else {
						handler.post(new Runnable() {
							public void run() {
								TextView txtView = (TextView)ConfigurationDialog.this.findViewById(R.id.txtADID);
								txtView.setText(com.hive.base.Android.getAdvertisingId());								
							}
						});
						break;
					}
				} while (true);
			}
		}).start(); 
	}
	
	@SuppressWarnings({ "unchecked", "rawtypes" })
	private void buildUI() {

		// 기존에 설정했던 내용이 있으면 화면을 구성할 때 기본 값으로 구성한다.
		property.loadProperties();
		
		
		// hive_config.xml 에 설정되어 있는 값을 읽어온다.
		InputStream is = com.hive.base.Resource.openRawResource(activity, "hive_config");
		if(is == null) {
			Log.e(Const.TAG, "hive_config.xml file not found!");
		}
		
		JSONObject root = com.hive.base.Resource.readXmlToJSON(is);
		if(root == null) {
			Log.e(Const.TAG, "hive_config.xml file is invalid format!");
		}
		
		
		String defAppId = Android.getPackageName(activity); // default로 AndroidManifest.xml 의 packageName 을 설정
		ZoneType defZoneType = ZoneType.SANDBOX;
		Boolean defUseLog = false;
		Boolean defUseAgeGateU13 = false;
		try {

			JSONObject properties = root.getJSONObject("properties");
			defAppId = properties.optString("appId", defAppId);
			defZoneType = ZoneType.valueOf(properties.optString("zone", ZoneType.SANDBOX.name()).toUpperCase(Locale.US));
			defUseLog = properties.optBoolean("useLog", true);
			defUseAgeGateU13 = properties.optBoolean("ageGateU13", false);
		}
		catch(Exception e) {
		}
		
		
		TextView txtView;
		EditText edtText;
		ToggleButton btnToggle;

		ArrayAdapter<CharSequence> spAdapter;
		final ArrayAdapter<LanguageDisplayContent> spLanguageAdapter;
		Spinner spinner;
		

		// SDK Version 출력
		StringBuffer sb = new StringBuffer();
		sb.append("HIVE SDK=");
		sb.append(com.hive.Configuration.getHiveSDKVersion());
		
		String refSDKVersion = com.hive.Configuration.getReferenceSDKVersion();
		if(refSDKVersion.length() > 0) {
			sb.append("\n");
			sb.append(refSDKVersion);
		}
		txtView = (TextView)this.findViewById(R.id.txtSDKVer);
		txtView.setText(sb.toString());
		
		// 광고 아이디 출력용 view 저장.
		txtView = (TextView)this.findViewById(R.id.txtADID);
		txtView.setText("wait..."); // 초기값.
		updateAdvertisingIdTextView(); // 값의 업데이트를 위한 폴링 함수 호
				
		// AppId 설정
		appId = property.getValue("sample.appId", defAppId);
		edtText = (EditText)this.findViewById(R.id.edtPackageInfo);
		edtText.setText(appId);
		
		
		// Hive 플랫폼의 서버 선택. sandbox 는 개발용, real 은 실계
		spAdapter = new ArrayAdapter(activity, android.R.layout.simple_spinner_item);
		spAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
		spAdapter.add("SANDBOX");
		spAdapter.add("REAL");
		spAdapter.add("TEST");
		spAdapter.add("DEV");
		
		spinner = (Spinner)this.findViewById(R.id.spZoneType);
		spinner.setAdapter(spAdapter);
		
		String value = property.getValue("sample.zoneType", defZoneType.name());
		if("REAL".equals(value)) {
			spinner.setSelection(1);
		} else if("TEST".equals(value)) {
			spinner.setSelection(2);
		}
		
		
		// HiveSDK 의 사용 가능 언어 선택
		spLanguageAdapter = new ArrayAdapter<LanguageDisplayContent>(activity, android.R.layout.simple_spinner_item);
		spLanguageAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);

		for (LanguageDisplayContent content : languageMap.values()) {
			spLanguageAdapter.add(content);
		}
		
		spinner = (Spinner) this.findViewById(R.id.spHiveLanguage);
		spinner.setAdapter(spLanguageAdapter);
		
		value = property.getValue("sample.gameLanguage", "en");
		spinner.setSelection(spLanguageAdapter.getPosition(languageMap.get(value)));
		
		
		// HiveSDK 의 사용 게임 서버 선택
		value = property.getValue("sample.gameServerId", "kr");
		edtText = (EditText)this.findViewById(R.id.edtHiveGameServerId);
		edtText.setText(value);
		
		
		// HIVE SDK 동작 로그 여부
		value = property.getValue("sample.isLogging", defUseLog?"true":"false");
		btnToggle = (ToggleButton)this.findViewById(R.id.tbUseLog);
		btnToggle.setChecked("true".equals(value));
		
		
		// 리모트 로그를 위한 사용자 ID
		value = property.getValue("sample.loggingUserId", "");
		edtText = (EditText)this.findViewById(R.id.edtLogUserId);
		edtText.setText(value);
		
		// 리모트 로그 사용 유무
		value = property.getValue("sample.isRemoteLogging", "false");
		btnToggle = (ToggleButton)this.findViewById(R.id.tbRemoteLog);
		btnToggle.setChecked("true".equals(value));
		
		// 리모트 상세 로그 사용 유무
		value = property.getValue("sample.isRemoteDetailLogging", "false");
		btnToggle = (ToggleButton)this.findViewById(R.id.tbRemoteDetailLog);
		btnToggle.setChecked("true".equals(value));

		// AgeGateU13 사용 유무
		value = property.getValue("sample.isAgeGateU13", "false");
		btnToggle = (ToggleButton)this.findViewById(R.id.tbAgeGateU13);
		btnToggle.setChecked("true".equals(value));

		// getPermissionViewData에 사용할 Support Language 등록
		spinner = (Spinner) this.findViewById(R.id.spinnerResourceLanguages);
		String[] resourceLanguages = {"de", "en", "es", "fr", "in", "it", "ja", "ko", "pt", "ru","th", "tr", "vi", "zh-hans", "zh-hant" };

		ArrayAdapter<String> adapter = new ArrayAdapter<String>(activity, android.R.layout.simple_spinner_item, resourceLanguages);
		adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
		spinner.setAdapter(adapter);



		// HiveSDK 설정 파일 (hive_config.xml) 보기
		this.findViewById(R.id.btnViewConfigurationFile).setOnClickListener(new View.OnClickListener() {

			@Override
			public void onClick(View v) {
				
				String xmlContent = Resource.readFile(Resource.openRawResource(activity, "hive_config"));
				ViewPagerBase.showLogDialog(activity, xmlContent);				
			}
		});

		//HIVE SDK 권한고지 팝업 노출 여부 설정
		this.findViewById(R.id.btnSetHivePermissionViewOn).setOnClickListener(new View.OnClickListener() {

			@Override
			public void onClick(View v) {
				final boolean isOn = ((Switch)v.getRootView().findViewById(R.id.switchSetHivePermissionViewOn)).isChecked();
				Configuration.setHivePermissionViewOn(isOn);

				AlertDialog.Builder builder = new AlertDialog.Builder(activity);
				builder.setTitle("setHivePermissionViewOn");
				builder.setMessage(isOn ? "true" : "false");
				builder.setPositiveButton("OK", new DialogInterface.OnClickListener() {
					@Override
					public void onClick(DialogInterface dialog, int which) {

					}
				});
				builder.create().show();
			}
		});

		//HIVE SDK 권한고지 데이터
		this.findViewById(R.id.btnGetPermissionViewData).setOnClickListener(new View.OnClickListener() {

			@Override
			public void onClick(View v) {
//				addLog("\n\n=== (Configuration)HIVE SDK 권한 고지 팝업 데이터 요청 ===\n");
//				addLog("Configuration.getPermissionViewData() Called\n");

				Spinner spinner = v.getRootView().findViewById(R.id.spinnerResourceLanguages);
				String languageStr = spinner.getSelectedItem().toString();
				Configuration.HIVELanguage language = Configuration.HIVELanguage.HIVELanguageEN;

				for (Configuration.HIVELanguage item : Configuration.HIVELanguage.values()) {
					if (item.getValue().equals(languageStr)) {
						language = item;
						break;
					}
					//UI에서는 "id"값으로 가져오므로 Android에서 따로 "in"으로 로컬라이징 해야함.
					if (languageStr.equals("id")) {
						language = Configuration.HIVELanguage.HIVELanguageIN;
						break;
					}
				}
				PermissionView.PermissionViewData data = Configuration.getPermissionViewData(language);

				if (data == null) {
					//addLog("Not supported language.\n");
					return;
				}

				String popupTitle = data.contents;
				PermissionView.PermissionViewUnitData[] permissionViewUnitDatas = data.permissions;
				StringBuilder sb = new StringBuilder();

				for (PermissionView.PermissionViewUnitData permissionViewUnitData : permissionViewUnitDatas) {
					sb.append("Title : ").append(permissionViewUnitData.title).append("\n");
					sb.append("Description : ").append(permissionViewUnitData.contents).append("\n");
					sb.append("Permission Name : ").append(permissionViewUnitData.nativePermissionName).append("\n\n");
				}

				PermissionView.PermissionViewUnitData[] commonDatas = data.commons;
				for (PermissionView.PermissionViewUnitData commonUnitData : commonDatas) {
					sb.append("Title : ").append(commonUnitData.title).append("\n");
					sb.append("Description : ").append(commonUnitData.contents).append("\n");
					sb.append("Permission Name : ").append(commonUnitData.nativePermissionName).append("\n\n");
				}

				AlertDialog.Builder builder = new AlertDialog.Builder(activity);
				builder.setTitle(popupTitle);
				builder.setMessage(sb.toString());
				builder.setPositiveButton("OK", new DialogInterface.OnClickListener() {
					@Override
					public void onClick(DialogInterface dialog, int which) {

					}
				});
				builder.create().show();
			}
		});


		// 설정 초기화
		this.findViewById(R.id.btnResetConfiguration).setOnClickListener(new View.OnClickListener() {
			
			@Override
			public void onClick(View v) {
				
				Property property = ConfigurationDialog.this.property;
				property.deletePropertyFile();
				
				ConfigurationDialog.this.buildUI();
			}
		});
		
		
		// 설정 적용
		this.findViewById(R.id.btnApplyConfiguration).setOnClickListener(new View.OnClickListener() {
			
			@Override
			public void onClick(View v) {
				
				ConfigurationDialog.this.dismiss();
			}
		});		
	}
	
	
	public void showConfigurationDialog(final ConfigurationDialogListener listener) {
		
		this.setOnDismissListener(new OnDismissListener() {

			@Override
			public void onDismiss(DialogInterface dialog) {

				EditText edtText;
				ToggleButton btnToggle;
				Spinner spinner;
				
				
				// 설정 대화 상자가 닫힐 때 설정한 내용을 설정 파일에 기록한다.
				edtText = (EditText)ConfigurationDialog.this.findViewById(R.id.edtPackageInfo);
				ConfigurationDialog.this.appId = edtText.getText().toString();
				property.setValue("sample.appId", ConfigurationDialog.this.appId);
				
				spinner = (Spinner)ConfigurationDialog.this.findViewById(R.id.spZoneType);
				String selectedValue = (String) spinner.getAdapter().getItem(spinner.getSelectedItemPosition());
				ConfigurationDialog.this.zoneType = ZoneType.valueOf(selectedValue);
				property.setValue("sample.zoneType", ConfigurationDialog.this.zoneType.name().toUpperCase(Locale.US));
				
				spinner = (Spinner)ConfigurationDialog.this.findViewById(R.id.spHiveLanguage);
				LanguageDisplayContent content = (LanguageDisplayContent)spinner.getAdapter().getItem(spinner.getSelectedItemPosition());
				ConfigurationDialog.this.gameLanguage = content.languageCode;
				property.setValue("sample.gameLanguage", ConfigurationDialog.this.gameLanguage);
				
				edtText = (EditText)ConfigurationDialog.this.findViewById(R.id.edtHiveGameServerId);
				ConfigurationDialog.this.gameServerId = edtText.getText().toString();
				property.setValue("sample.gameServerId", ConfigurationDialog.this.gameServerId);
				
				btnToggle = (ToggleButton)ConfigurationDialog.this.findViewById(R.id.tbUseLog);
				ConfigurationDialog.this.isLogging = btnToggle.isChecked();
				property.setValue("sample.isLogging", ConfigurationDialog.this.isLogging?"true":"false");
				
				edtText = (EditText)ConfigurationDialog.this.findViewById(R.id.edtLogUserId);
				ConfigurationDialog.this.loggingUserId = edtText.getText().toString();
				property.setValue("sample.loggingUserId", ConfigurationDialog.this.loggingUserId);
				
				btnToggle = (ToggleButton)ConfigurationDialog.this.findViewById(R.id.tbRemoteLog);
				ConfigurationDialog.this.isRemoteLogging = btnToggle.isChecked();
				property.setValue("sample.isRemoteLogging", ConfigurationDialog.this.isRemoteLogging?"true":"false");
				
				btnToggle = (ToggleButton)ConfigurationDialog.this.findViewById(R.id.tbRemoteDetailLog);
				ConfigurationDialog.this.isRemoteDetailLogging = btnToggle.isChecked();
				property.setValue("sample.isRemoteDetailLogging", ConfigurationDialog.this.isRemoteDetailLogging?"true":"false");
				property.writeProperties();

				btnToggle = (ToggleButton)ConfigurationDialog.this.findViewById(R.id.tbAgeGateU13);
				ConfigurationDialog.this.isAgeGateU13 = btnToggle.isChecked();
				property.setValue("sample.isAgeGateU13", ConfigurationDialog.this.isAgeGateU13?"true":"false");
				property.writeProperties();
				
				
				if(listener != null) {
					
					// 설정 창을 닫을 때 설정된 결과를 콜백
					listener.onConfigurationSelect(ConfigurationDialog.this.appId
							, ConfigurationDialog.this.zoneType
							, ConfigurationDialog.this.gameLanguage
							, ConfigurationDialog.this.gameServerId
							, ConfigurationDialog.this.isLogging
							, ConfigurationDialog.this.loggingUserId
							, ConfigurationDialog.this.isRemoteLogging
							, ConfigurationDialog.this.isRemoteDetailLogging
							, ConfigurationDialog.this.isAgeGateU13);
				}
			}
		});
		
		
		this.setCancelable(false);
		this.show();
	}

	
	private class LanguageDisplayContent {
		
		String	languageCode	= "";
		@SuppressWarnings("unused")
		Locale	locale			= null;
		@StringRes
		int		displayStringId	= 0;

		LanguageDisplayContent(String languageCode, Locale locale, @StringRes int displayStringId) {
			this.languageCode = languageCode;
			this.locale = locale;
			this.displayStringId = displayStringId;
		}

		@Override
		public String toString() {

			if (displayStringId != 0)
				return activity.getResources().getString(displayStringId) + "(" + languageCode + ")";
			else
				return languageCode;
		}
	}

	private Map<String, LanguageDisplayContent> languageMap = new LinkedHashMap<String, LanguageDisplayContent>();
	
	
	
	
	/**
	 * HIVE SDK 설정이 완료될 경우 호출되는 리스너
	 * 
	 * @author ryuvsken
	 */
	public interface ConfigurationDialogListener {
		
		/**
		 * HIVE SDK 설정이 완료될 경우 호출
		 * 
		 * @param appId						// 입력된 AppId
		 * @param zoneType					// 선택된 HIVE 플랫폼 서비스 형태
		 * @param gameLanguage				// 선택된 게임 언어
		 * @param gameServerId				// 선택된 게임 서버
		 * @param isLogging					// HIVE SDK 동작 로그 여부
		 * @param loggingUserId				// 리모트 로그를 위한 사용자 ID
		 * @param isRemoteLogging			// 리모트 로그 사용 유무
		 * @param isRemoteDetailLogging		// 리모트 상세 로그 사용 유무
		 * @param isAgeGateU13				// AgeGateU13 사용 유무
		 */
		public void onConfigurationSelect(String appId, ZoneType zoneType, String gameLanguage, String gameServerId
				, Boolean isLogging, String loggingUserId, Boolean isRemoteLogging, Boolean isRemoteDetailLogging, Boolean isAgeGateU13);
	}
}



