package com.hive.sample;

import android.app.AlertDialog;
import android.app.Dialog;
import android.content.Context;
import android.content.DialogInterface;
import android.text.TextUtils;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.widget.BaseAdapter;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;

import com.hive.IAP;
import com.hive.IAP.IAPProduct;
import com.hive.IAP.IAPShop;
import com.hive.ResultAPI;
import com.hive.base.Android;
import com.hive.base.Logger;
import com.hive.base.Resource;
import com.hive.sample.uihelper.ViewPagerBase;

import org.json.JSONObject;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.TimeZone;


/**
 * HIVE SDK 의 IAP (In App Purchase) 상점 정보를 보여주는 대화 상자
 *
 * @author hife
 */
public class IAPShopView extends Dialog {

	TextView textview_locationcode_msg;
	TextView textview_shopid_msg;
	TextView textview_shoptype_msg;
	TextView textview_balance_msg;
	ImageView imageview_shop_badge;
	TextView textview_additional_info_msg;

	ListView listview_products;

	ListViewAdapter listviewadapter_product;

	ViewPagerBase forLog;

	String additionalInfo;


	public IAPShopView(Context context, IAPShop iapShop, int balance, String additionalInfo, ViewPagerBase forLog) {
		super(context);

		if (TextUtils.isEmpty(additionalInfo)) {
			this.additionalInfo = additionalInfo;

		} else {
			try {
				JSONObject jsonObject = new JSONObject();
				jsonObject.put("character_key", additionalInfo);
				this.additionalInfo = jsonObject.toString();

			} catch (Exception e) {
				e.printStackTrace();
				this.additionalInfo = null;
			}
		}

		this.forLog = forLog;

		this.requestWindowFeature(Window.FEATURE_NO_TITLE);
		this.setCancelable(true);

		setContentView(Resource.getLayoutId(context, "test_view_iap_shop"));

		textview_locationcode_msg = (TextView) findViewById(Resource.getViewId(context, "textview_locationcode_msg"));
		textview_shopid_msg = (TextView) findViewById(Resource.getViewId(context, "textview_shopid_msg"));
		textview_shoptype_msg = (TextView) findViewById(Resource.getViewId(context, "textview_shoptype_msg"));
		textview_balance_msg = (TextView) findViewById(Resource.getViewId(context, "textview_balance_msg"));
		imageview_shop_badge = (ImageView) findViewById(Resource.getViewId(context, "imageview_shop_badge"));
		textview_additional_info_msg = (TextView) findViewById(Resource.getViewId(context, "textview_additional_info_msg"));

		listview_products = (ListView) findViewById(Resource.getViewId(context, "listview_products"));


		textview_locationcode_msg.setText(iapShop.locationCode);
		textview_shopid_msg.setText(String.valueOf(iapShop.shopId));
		textview_shoptype_msg.setText(iapShop.shopType);
		textview_balance_msg.setText(String.valueOf(balance));
		textview_additional_info_msg.setText(this.additionalInfo);

		listviewadapter_product = new ListViewAdapter();

		listview_products.setAdapter(listviewadapter_product);

		for (IAPProduct iapProduct : iapShop.productList) {
			listviewadapter_product.add(iapProduct);
		}

		listview_products.setLayoutParams(new LinearLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT));
	}


	@Override
	public void dismiss() {
		//TODO list bitmap recycle

		super.dismiss();
	}


	class ListViewAdapter extends BaseAdapter {

		private ArrayList<IAPProduct> iapProductList;

		public ListViewAdapter() {
			iapProductList = new ArrayList<IAPProduct>();
		}

		@Override
		public int getCount() {
			return iapProductList.size();
		}

		@Override
		public Object getItem(int position) {
			return iapProductList.get(position);
		}

		@Override
		public long getItemId(int position) {
			return position;
		}

		@Override
		public View getView(int position, View convertView, ViewGroup parent) {

			final Context context = parent.getContext();

			final IAPProduct iapProduct = iapProductList.get(position);
			final ViewHolder holder;


			if (convertView == null) {

				LayoutInflater inflater = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
		        convertView = inflater.inflate(Resource.getLayoutId(context, "test_view_iap_product"), parent, false);

		        holder = new ViewHolder();

		        holder.imageview_product = (ImageView) convertView.findViewById(Resource.getViewId(context, "imageview_product"));
		        holder.textview_product_displayprice = (TextView) convertView.findViewById(Resource.getViewId(context, "textview_product_displayprice"));
		        holder.imageview_product_title = (ImageView) convertView.findViewById(Resource.getViewId(context, "imageview_product_title"));
		        holder.textview_product_title = (TextView) convertView.findViewById(Resource.getViewId(context, "textview_product_title"));
		        holder.imageview_product_badge = (ImageView) convertView.findViewById(Resource.getViewId(context, "imageview_product_badge"));
		        holder.imageview_product_desc = (ImageView) convertView.findViewById(Resource.getViewId(context, "imageview_product_desc"));
		        holder.textview_product_desc = (TextView) convertView.findViewById(Resource.getViewId(context, "textview_product_desc"));
		        holder.iapProduct = iapProduct;

		        convertView.setTag(holder);
			}

			else {
				holder = (ViewHolder) convertView.getTag();
			}

			//display price
			holder.textview_product_displayprice.setText(iapProduct.displayPrice);

			convertView.setOnClickListener(new View.OnClickListener() {

				@Override
				public void onClick(View v) {

					String pid = iapProduct.gamePid;

					forLog.addLog("IAP.purchase(pid: " + iapProduct.gamePid + "), listener) Called\n");

			    	Log.v("sample","\n\n=== HIVE IAP 상품에 대한 구매를 요청. ===\n");
			        Log.v("sample", String.format("IAP.purchase(pid = %s, onIAPPurchase) Called\n", pid));

					com.hive.IAP.purchase(pid, additionalInfo, new IAP.IAPPurchaseReceiptListener() {
						@Override
						public void onIAPPurchaseReceipt(ResultAPI result, IAP.IAPReceipt iapReceipt) {

							StringBuilder builder = new StringBuilder();
							builder.append(String.format("IAPPurchaseListener.onIAPPurchase() Callback - result = %s\n", result));
							if (iapReceipt != null)
								builder.append("IAPReceipt : ").append(iapReceipt).append("\n");

							forLog.addLog(builder.toString());
							Log.v("sample", builder.toString());

							if (result.needExit()) {
//								System.exit(0);
								AlertDialog.Builder builder2 = new AlertDialog.Builder(context);
								builder2.setMessage("Exit Event Occur. App will be terminated when you press OK button");
								builder2.setPositiveButton("OK", new DialogInterface.OnClickListener() {
									@Override
									public void onClick(DialogInterface dialog, int which) {
										Android.finish();
									}
								});
								builder2.create().show();
								return;
							}

							new AlertDialog.Builder(context)
									.setTitle("Purchase Result")
									.setMessage(result.toString() + "\n" + builder.toString())
									.create().show();
						}
					});

				}

			});


			return convertView;
		}

		public void add(IAPProduct iapProduct) {
			iapProductList.add(iapProduct);
			this.notifyDataSetChanged();
		}

		public void remove(int position) {
			iapProductList.remove(position);
			this.notifyDataSetChanged();
		}
	}


	class ViewHolder {

		ImageView imageview_product;
	    TextView textview_product_displayprice;
	    ImageView imageview_product_title;
	    TextView textview_product_title;
	    ImageView imageview_product_badge;
	    ImageView imageview_product_desc;
	    TextView textview_product_desc;

	    IAPProduct iapProduct;
	}


}
