package com.hive.sample;

import android.app.AlertDialog;
import android.app.Dialog;
import android.content.Context;
import android.content.DialogInterface;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Handler;
import android.os.Looper;
import android.text.Html;
import android.text.TextUtils;
import android.text.style.StrikethroughSpan;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.widget.BaseAdapter;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import com.hive.IAPV4;
import com.hive.IAPV4.IAPV4Product;
import com.hive.ResultAPI;
import com.hive.base.Logger;
import com.hive.base.Resource;
import com.hive.sample.game.GameNetwork;
import com.hive.sample.game.GameUser;
import com.hive.sample.game.IGameListener;
import com.hive.sample.game.Response;
import com.hive.sample.uihelper.ViewPagerBase;

import org.json.JSONObject;

import java.util.ArrayList;
import java.util.List;


/**
 * HIVE SDK 의 IAP (In App Purchase) 상점 정보를 보여주는 대화 상자
 *
 * @author hife
 */
public class IAPV4ProductInfoView extends Dialog {

	TextView textview_balance_msg;
	TextView textview_additional_info_msg;

	ListView listview_products;

	ListViewAdapter listviewadapter_product;

	ViewPagerBase forLog;

	String additionalInfo;


	public IAPV4ProductInfoView(Context context, List<IAPV4.IAPV4Product> iapV4ProductList, int balance, String additionalInfo, ViewPagerBase forLog) {
		super(context);

		if (TextUtils.isEmpty(additionalInfo)) {
			this.additionalInfo = additionalInfo;

		} else {
			try {
				JSONObject jsonObject = new JSONObject();
				jsonObject.put("character_key", additionalInfo);
				this.additionalInfo = jsonObject.toString();

			} catch (Exception e) {
				e.printStackTrace();
				this.additionalInfo = null;
			}
		}

		this.forLog = forLog;

		this.requestWindowFeature(Window.FEATURE_NO_TITLE);
		this.setCancelable(true);

		setContentView(Resource.getLayoutId(context, "test_view_iapv4_product_info"));

		textview_balance_msg = (TextView) findViewById(Resource.getViewId(context, "iapv4_textview_balance_msg"));
		textview_additional_info_msg = (TextView) findViewById(Resource.getViewId(context, "iapv4_textview_additional_info_msg"));

		listview_products = (ListView) findViewById(Resource.getViewId(context, "iapv4_listview_products"));


		textview_balance_msg.setText(String.valueOf(balance));
		textview_additional_info_msg.setText(this.additionalInfo);

		listviewadapter_product = new ListViewAdapter();

		listview_products.setAdapter(listviewadapter_product);

		for (IAPV4Product iapProduct : iapV4ProductList) {
			listviewadapter_product.add(iapProduct);
		}

		listview_products.setLayoutParams(new LinearLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT));
	}


	@Override
	public void dismiss() {
		//TODO list bitmap recycle

		super.dismiss();
	}


	class ListViewAdapter extends BaseAdapter {

		private ArrayList<IAPV4Product> iapProductList;

		public ListViewAdapter() {
			iapProductList = new ArrayList<IAPV4Product>();
		}

		@Override
		public int getCount() {
			return iapProductList.size();
		}

		@Override
		public Object getItem(int position) {
			return iapProductList.get(position);
		}

		@Override
		public long getItemId(int position) {
			return position;
		}

		@Override
		public View getView(int position, View convertView, ViewGroup parent) {

			final Context context = parent.getContext();

			final IAPV4Product iapProduct = iapProductList.get(position);
			final ViewHolder holder;


			if (convertView == null) {

				LayoutInflater inflater = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
		        convertView = inflater.inflate(Resource.getLayoutId(context, "test_view_iapv4_product"), parent, false);

		        holder = new ViewHolder();

		        holder.textview_product_displayprice = (TextView) convertView.findViewById(Resource.getViewId(context, "iapv4_textview_product_displayprice"));
		        holder.textview_product_title = (TextView) convertView.findViewById(Resource.getViewId(context, "iapv4_textview_product_title"));
		        holder.textview_product_desc = (TextView) convertView.findViewById(Resource.getViewId(context, "iapv4_textview_product_desc"));
		        holder.iapProduct = iapProduct;

		        convertView.setTag(holder);
			}

			else {
				holder = (ViewHolder) convertView.getTag();
			}

			//display price
			String text = "<strike><font color=\'#757575\'>"+iapProduct.displayOriginalPrice+"</font></strike><br>"+iapProduct.displayPrice;
			holder.textview_product_displayprice.setText(Html.fromHtml(text));


			//title
			StringBuilder titleBuilder = new StringBuilder();
			titleBuilder
					.append("PID: ").append(iapProduct.marketPid)
					.append("\n")
					.append("타이틀: ")
					.append(iapProduct.title);

			holder.textview_product_title.setText(titleBuilder.toString());

			//description
			StringBuilder descriptionBuilder = new StringBuilder();
			descriptionBuilder.append("아이템 설명: ").append(iapProduct.description);
			holder.textview_product_desc.setText(descriptionBuilder.toString());


			convertView.setOnClickListener(new View.OnClickListener() {

				@Override
				public void onClick(View v) {

					String pid = iapProduct.marketPid;

					forLog.addLog("IAPV4.purchase(pid: " + pid + "), listener) Called\n");

					Log.v("sample", "\n\n=== HIVE IAP 상품에 대한 구매를 요청. ===\n");
					Log.v("sample", String.format("IAP.purchase(pid = %s, additionalInfo = %s, onIAPV4Purchase) Called\n", pid, additionalInfo));


					if( pid.startsWith("sub") ) {
						String oldMarketPid = GameUser.getInstance().subscrptionMarketPid;
						 IAPV4.purchaseSubscriptionUpdate(pid, pid.equals(oldMarketPid) ? null : oldMarketPid, additionalInfo, new IAPV4.IAPV4PurchaseListener() {
							 @Override
							 public void onIAPV4Purchase(ResultAPI result, final IAPV4.IAPV4Receipt iapV4Receipt) {
								 StringBuilder builder = new StringBuilder();
								 builder.append(String.format("IAPV4PurchaseListener.onIAPV4Purchase() Callback - result = %s\n", result))
										 .append("IAPV4Receipt : ").append(iapV4Receipt).append("\n");

								 forLog.addLog(builder.toString());
							Log.v("sample", builder.toString());

								 if (result.isSuccess() && iapV4Receipt != null) {

									 GameNetwork.receiptVerify(iapV4Receipt, new IGameListener() {

										 @Override
										 public void onGameListener(boolean result, Response response) {

											 if (result == true) {

												 if (response != null && response.isSuccess()) {

													 IAPV4.transactionFinish(iapV4Receipt.product.marketPid, new IAPV4.IAPV4TransactionFinishListener() {
														 @Override
														 public void onIAPV4TransacionFinish(ResultAPI result, String marketPid) {

															 Toast.makeText(context, "Purchase Complete", Toast.LENGTH_SHORT).show();
														 }
													 });


												 } else {

													 new AlertDialog.Builder(context)
															 .setTitle("Purchase Result")
															 .setMessage("Game Server Response Error\n" + response.resultMsg)
															 .setPositiveButton("OK", new OnClickListener() {
																 @Override
																 public void onClick(DialogInterface dialog, int which) {

																 }
															 })
															 .setNegativeButton("Transaction Finish", new OnClickListener() {
																 @Override
																 public void onClick(DialogInterface dialog, int which) {
																	 IAPV4.transactionFinish(iapV4Receipt.product.marketPid, new IAPV4.IAPV4TransactionFinishListener() {
																		 @Override
																		 public void onIAPV4TransacionFinish(ResultAPI result, String marketPid) {

																			 Toast.makeText(context, "Purchase Complete", Toast.LENGTH_SHORT).show();
																		 }
																	 });
																 }
															 })
															 .create().show();
												 }

											 }
										 }
									 }, "subscription");
								 }
								 new AlertDialog.Builder(context)
										 .setTitle("Purchase Result")
										 .setMessage(result.toString() + "\n" + builder.toString())
										 .create().show();
							 }
						 });
					}
					else {
						IAPV4.purchase(pid, additionalInfo, new IAPV4.IAPV4PurchaseListener() {
							@Override
							public void onIAPV4Purchase(ResultAPI result, final IAPV4.IAPV4Receipt iapV4Receipt) {

								StringBuilder builder = new StringBuilder();
								builder.append(String.format("IAPV4PurchaseListener.onIAPV4Purchase() Callback - result = %s\n", result))
										.append("IAPV4Receipt : ").append(iapV4Receipt).append("\n");

								forLog.addLog(builder.toString());
								Logger.log(Logger.HIVELogType.HIVELogTypeV,"sample", builder.toString());

								if (result.isSuccess() && iapV4Receipt != null) {

									GameNetwork.receiptVerify(iapV4Receipt, new IGameListener() {

										@Override
										public void onGameListener(boolean result, Response response) {

											if (result == true) {

												if (response != null && response.isSuccess()) {

													IAPV4.transactionFinish(iapV4Receipt.product.marketPid, new IAPV4.IAPV4TransactionFinishListener() {
														@Override
														public void onIAPV4TransacionFinish(ResultAPI result, String marketPid) {

															Toast.makeText(context, "Purchase Complete", Toast.LENGTH_SHORT).show();
														}
													});


												} else {

													new AlertDialog.Builder(context)
															.setTitle("Purchase Result")
															.setMessage("Game Server Response Error\n" + response.resultMsg)
															.setPositiveButton("OK", new OnClickListener() {
																@Override
																public void onClick(DialogInterface dialog, int which) {

																}
															})
															.setNegativeButton("Transaction Finish", new OnClickListener() {
																@Override
																public void onClick(DialogInterface dialog, int which) {
																	IAPV4.transactionFinish(iapV4Receipt.product.marketPid, new IAPV4.IAPV4TransactionFinishListener() {
																		@Override
																		public void onIAPV4TransacionFinish(ResultAPI result, String marketPid) {

																			Toast.makeText(context, "Purchase Complete", Toast.LENGTH_SHORT).show();
																		}
																	});
																}
															})
															.create().show();
												}

											}
										}
									}, "consumable");
								}
								new AlertDialog.Builder(context)
										.setTitle("Purchase Result")
										.setMessage(result.toString() + "\n" + builder.toString())
										.create().show();
							}
						});
					}
				}

			});


			return convertView;
		}

		public void add(IAPV4Product iapProduct) {
			iapProductList.add(iapProduct);
			this.notifyDataSetChanged();
		}

		public void remove(int position) {
			iapProductList.remove(position);
			this.notifyDataSetChanged();
		}
	}


	class ViewHolder {

	    TextView textview_product_displayprice;
	    TextView textview_product_title;
	    TextView textview_product_desc;

		IAPV4Product iapProduct;
	}

}
