package com.hive.sample;


import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.os.Handler;
import android.os.Looper;
import android.text.TextUtils;
import android.view.View;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;

import com.com2us.hivesdk.normal.freefull.google.global.android.common.R;
import com.hive.Configuration;
import com.hive.IAPV4;
import com.hive.IAPV4.IAPV4Type;
import com.hive.ResultAPI;
import com.hive.base.Property;
import com.hive.impl.IAPV4Impl;
import com.hive.sample.game.GameNetwork;
import com.hive.sample.game.GamePostboxDialog;
import com.hive.sample.game.GameUser;
import com.hive.sample.game.IGameListener;
import com.hive.sample.game.Post;
import com.hive.sample.game.Response;
import com.hive.sample.uihelper.ViewPagerBase;

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;


/**
 * HIVE SDK 의 IAP (In App Purchase) 기능 테스트 뷰
 * 
 * @author hife
 */
public class IAPV4TestView extends ViewPagerBase implements View.OnClickListener {


	public IAPV4TestView(Activity activity, View view, String title) {
		super(activity, view, title);

		initUi();
	}


	private void initUi() {

		createLogView(R.id.logview, R.id.btnLogDialog, R.id.btnLogClear);

		this.view.findViewById(R.id.btnIAPV4Postbox).setOnClickListener(this);
		this.view.findViewById(R.id.btnIAPV4MarketConnect).setOnClickListener(this);
		this.view.findViewById(R.id.btnIAPV4GetProductInfo).setOnClickListener(this);
        this.view.findViewById(R.id.btnIAPV4Restore).setOnClickListener(this);
        this.view.findViewById(R.id.btnIAPV4GetSubscriptionProductInfo).setOnClickListener(this);
        this.view.findViewById(R.id.btnIAPV4RestoreSubscription).setOnClickListener(this);
		this.view.findViewById(R.id.btnIAPV4GetBalance).setOnClickListener(this);
		this.view.findViewById(R.id.btnIAPV4ShowPayment).setOnClickListener(this);
		this.view.findViewById(R.id.btnIAPV4ShowCharge).setOnClickListener(this);
		this.view.findViewById(R.id.btnIAPV4CheckPromote).setOnClickListener(this);
		Property iapProperty = new Property("iapv4_property");
		String additionalInfo = iapProperty.getValue("iapv4.additionalInfo");

		if ( !TextUtils.isEmpty(additionalInfo) ) {
			EditText edtAdditionalInfo = (EditText) this.view.findViewById(R.id.edtIAPV4AdditionalInfo);
			edtAdditionalInfo.setText(additionalInfo);
		}
	}


	public void onGameUserUiUpdate() {

		new Handler(Looper.getMainLooper()).post(new Runnable() {

			@Override
			public void run() {

				TextView tvServerId = (TextView) IAPV4TestView.this.view.findViewById(R.id.tvIapV4ServerId);
				tvServerId.setText("Server ID: " + Configuration.getServerId());

				TextView tvIapNickname = (TextView) IAPV4TestView.this.view.findViewById(R.id.tvIapV4Nickname);
				tvIapNickname.setText("Name: " + GameUser.getInstance().nickname);

				TextView tvIapGold = (TextView) IAPV4TestView.this.view.findViewById(R.id.tvIapV4Gold);
				tvIapGold.setText(GameUser.getInstance().gold + " GOLD");

				TextView tvIapGem = (TextView) IAPV4TestView.this.view.findViewById(R.id.tvIapV4Gem);
				tvIapGem.setText(GameUser.getInstance().gem + " GEM");

                TextView tvIapSuvscription = (TextView) IAPV4TestView.this.view.findViewById(R.id.tvIapV4Subscription);
                tvIapSuvscription.setText("구독 여부 : " + !TextUtils.isEmpty(GameUser.getInstance().subscrptionMarketPid) );

			}
		});
	}


	@Override
	public void onUpdateView() {

		onGameUserUiUpdate();

		super.onUpdateView();

//		GameUser.getInstance().update(null);
	}

	private ArrayList<Post> gamePostList = new ArrayList<Post>();

	@Override
	public void onClick(View v) {

		int rid = v.getId();

		switch(rid) {


			case R.id.btnIAPV4Postbox:
			{
				new Handler(Looper.getMainLooper()).post(new Runnable() {

					@Override
					public void run() {
//						GamePostboxV4Dialog gamePostboxDialog = new GamePostboxV4Dialog(activity, IAPV4TestView.this);
//						gamePostboxDialog.show();

						GamePostboxDialog gamePostboxDialog = new GamePostboxDialog(activity, IAPV4TestView.this);
						gamePostboxDialog.show();
					}
				});
			}
				break;


			case R.id.btnIAPV4MarketConnect:
			{
				addLog("\n\n=== IAP 결재 API 초기화 요청 ===\n");
				addLog("IAPV4.marketConnect(listener) Called\n");



				IAPV4.marketConnect(new IAPV4.IAPV4MarketInfoListener() {
					@Override
					public void onIAPV4MarketInfo(ResultAPI result, List<IAPV4.IAPV4Type> iapV4TypeList) {

						addLog(String.format("IAPV4MarketInfoListener.onIAPV4MarketInfo() Callback\nresult = %s\niapTypeList = %s\n"
							, result.toString(), iapV4TypeList != null ? iapV4TypeList.toString() : "null"));

						TextView txtSelectedMarket = (TextView) IAPV4TestView.this.view.findViewById(R.id.txtIAPV4SelectedMarket);

						if(iapV4TypeList != null) {
							StringBuffer buffer = new StringBuffer();
							for (IAPV4.IAPV4Type iapType : iapV4TypeList) {
								buffer.append(" " + iapType.name());
							}
							txtSelectedMarket.setText(buffer.toString());
						}

						if(result.isFailure()) {
							showAlertDialog("IAPV4 marketConnect Failed", result.toString());
						}
					}

				});
			}
				break;


			case R.id.btnIAPV4GetProductInfo:
			{
				// 인자 설정
				EditText edtAdditionalInfo = (EditText) this.view.findViewById(R.id.edtIAPV4AdditionalInfo);
				final String additionalInfo = edtAdditionalInfo.getText().toString();

				Property iapProperty = new Property("iapv4_property");
				iapProperty.setValue("iapv4.additionalInfo", additionalInfo);
				iapProperty.writeProperties();


				addLog("\n\n=== 상점 소모성 상품 정보 조회 ===\n");
				addLog(String.format("IAPV4.getProductInfo(listener) Called\n"));

				IAPV4.getProductInfo(new IAPV4.IAPV4ProductInfoListener() {
					@Override
					public void onIAPV4ProductInfo(ResultAPI result, final List<IAPV4.IAPV4Product> iapV4ProductList, final int balance) {

						addLog(String.format("IAPV4ProductInfoListener.onIAPV4ProductInfo() Callback\nresult = %s\nbalance = %d\n"
							, result.toString(), balance));

						if(result.isSuccess()) {

							TextView txtSelectedMarket = (TextView) IAPV4TestView.this.view.findViewById(R.id.txtIAPV4SelectedMarket);

							IAPV4Type selectedIAPType = null;
							for (IAPV4Type iapType : IAPV4Type.values()) {
								if(iapType.getValue() == IAPV4Impl.getSelectedMarket()) {
									selectedIAPType = iapType;
								}
							}
							txtSelectedMarket.setText(selectedIAPType.name());

							new Handler(Looper.getMainLooper()).post(new Runnable() {

								@Override
								public void run() {
									IAPV4ProductInfoView iapProductView = new IAPV4ProductInfoView(activity, iapV4ProductList, balance, additionalInfo, IAPV4TestView.this);
									iapProductView.show();
								}
							});
						}
						else {

							showAlertDialog("IAPV4 getProductInfo Failed", result.toString());
						}

					}
				});
			}
				break;


			case R.id.btnIAPV4Restore:
			{
				addLog("\n\n=== HIVE IAPV4 상품에 대한 미지급된 아이템 지급을 요청. ===\n");
				addLog("IAPV4.restore(listener) Called\n");

				IAPV4.restore(new IAPV4.IAPV4RestoreListener() {
					@Override
					public void onIAPV4Restore(ResultAPI result, List<IAPV4.IAPV4Receipt> iapv4ReceiptList) {

						StringBuilder builder = new StringBuilder();
						builder.append(String.format("IAPV4RestoreListener.onIAPV4Restore() Callback\nresult = %s\n", result.toString()));

						if (result.isSuccess() && iapv4ReceiptList != null) {

							builder.append("\n" + iapv4ReceiptList.size() + " item restored.\n\n");

							List<String> marketPidList = new ArrayList<String>();

							for (final IAPV4.IAPV4Receipt receipt : iapv4ReceiptList) {
								if (receipt != null) {
									builder.append("\n").append(receipt.toString()).append("\n\n------------\n\n");

									marketPidList.add(receipt.product.marketPid);

//									GamePostboxV4Dialog.addPost(new Post(
//											receipt.product.marketPid,
//											receipt.product.currency,
//											(int)receipt.product.price,
//											receipt.type.toString(),
//											receipt.additionalInfo,
//											new SimpleDateFormat("yyyy/MM/dd HH:mm:ss").format(Calendar.getInstance().getTime())
//									));

									GameNetwork.receiptVerify(receipt, new IGameListener() {

										@Override
										public void onGameListener(boolean result, Response response) {

											if (result == true) {

												if (response != null && response.isSuccess()) {

													IAPV4.transactionFinish(receipt.product.marketPid, new IAPV4.IAPV4TransactionFinishListener() {
														@Override
														public void onIAPV4TransacionFinish(ResultAPI result, String marketPid) {

															Toast.makeText(activity, "Restore Complete", Toast.LENGTH_SHORT).show();
														}
													});


												} else {

													new AlertDialog.Builder(activity)
															.setTitle("Purchase Result")
															.setMessage("Game Server Response Error\n" + response.resultMsg)
															.setPositiveButton("OK", new DialogInterface.OnClickListener() {
																@Override
																public void onClick(DialogInterface dialog, int which) {

																}
															})
															.setNegativeButton("Transaction Finish", new DialogInterface.OnClickListener() {
																@Override
																public void onClick(DialogInterface dialog, int which) {

																	IAPV4.transactionFinish(receipt.product.marketPid, new IAPV4.IAPV4TransactionFinishListener() {
																		@Override
																		public void onIAPV4TransacionFinish(ResultAPI result, String marketPid) {

																			Toast.makeText(activity, "Restore Complete", Toast.LENGTH_SHORT).show();
																		}
																	});
																}
															})
															.create().show();
												}

											}
										}
									},"consumable");
								}
							}

//							IAPV4.transactionMultiFinish(marketPidList, new IAPV4.IAPV4TransactionMultiFinishListener() {
//								@Override
//								public void onIAPV4TransacionMultiFinish(List<ResultAPI> resultList, List<String> marketPidList) {
//
//									for (int i = 0; i < resultList.size(); i++) {
//										LoggerImpl.log("transactionFinish : " + resultList.get(i));
//									}
//								}
//							});
						}

						addLog(builder.toString());

						showAlertDialog("IAPV4 Restore result", result.toString() + builder.toString());
					}
				});
			}
				break;

			case R.id.btnIAPV4GetSubscriptionProductInfo:
			{
				// 인자 설정
				EditText edtAdditionalInfo = (EditText) this.view.findViewById(R.id.edtIAPV4AdditionalInfo);
				final String additionalInfo = edtAdditionalInfo.getText().toString();

				Property iapProperty = new Property("iapv4_property");
				iapProperty.setValue("iapv4.additionalInfo", additionalInfo);
				iapProperty.writeProperties();


				addLog("\n\n=== 상점 구독 상품 정보 조회 ===\n");
				addLog(String.format("IAPV4.getProductInfo(listener) Called\n"));

				IAPV4.getSubscriptionProductInfo(new IAPV4.IAPV4ProductInfoListener() {
					@Override
					public void onIAPV4ProductInfo(ResultAPI result, final List<IAPV4.IAPV4Product> iapV4ProductList, final int balance) {

						addLog(String.format("IAPV4ProductInfoListener.onIAPV4ProductInfo() Callback\nresult = %s\nbalance = %d\n"
								, result.toString(), balance));

						if(result.isSuccess()) {

							TextView txtSelectedMarket = (TextView) IAPV4TestView.this.view.findViewById(R.id.txtIAPV4SelectedMarket);

							IAPV4Type selectedIAPType = null;
							for (IAPV4Type iapType : IAPV4Type.values()) {
								if(iapType.getValue() == IAPV4Impl.getSelectedMarket()) {
									selectedIAPType = iapType;
								}
							}
							txtSelectedMarket.setText(selectedIAPType.name());

							new Handler(Looper.getMainLooper()).post(new Runnable() {

								@Override
								public void run() {
									IAPV4ProductInfoView iapProductView = new IAPV4ProductInfoView(activity, iapV4ProductList, balance, additionalInfo, IAPV4TestView.this);
									iapProductView.show();
								}
							});
						}
						else {

							showAlertDialog("IAPV4 getProductInfo Failed", result.toString());
						}

					}
				});
			}
				break;

			case R.id.btnIAPV4RestoreSubscription:
				addLog("\n\n=== HIVE IAPV4 구독형 상품에 대한 복구 요청. ===\n");
				addLog("IAPV4.restore(listener) Called\n");

				IAPV4.restoreSubscription(new IAPV4.IAPV4RestoreListener() {
					@Override
					public void onIAPV4Restore(ResultAPI result, List<IAPV4.IAPV4Receipt> iapv4ReceiptList) {

						StringBuilder builder = new StringBuilder();
						builder.append(String.format("IAPV4RestoreListener.onIAPV4Restore() Callback\nresult = %s\n", result.toString()));

						if (result.isSuccess() && iapv4ReceiptList != null) {

							builder.append("\n" + iapv4ReceiptList.size() + " item restored.\n\n");

							List<String> marketPidList = new ArrayList<String>();

							for (final IAPV4.IAPV4Receipt receipt : iapv4ReceiptList) {
								if (receipt != null) {
									builder.append("\n").append(receipt.toString()).append("\n\n------------\n\n");

									marketPidList.add(receipt.product.marketPid);

									GameNetwork.receiptVerify(receipt, new IGameListener() {

										@Override
										public void onGameListener(boolean result, Response response) {

											if (result == true) {

												if (response != null && response.isSuccess()) {

													IAPV4.transactionFinish(receipt.product.marketPid, new IAPV4.IAPV4TransactionFinishListener() {
														@Override
														public void onIAPV4TransacionFinish(ResultAPI result, String marketPid) {

															Toast.makeText(activity, "Restore Complete", Toast.LENGTH_SHORT).show();
														}
													});


												} else {

													new AlertDialog.Builder(activity)
															.setTitle("Purchase Result")
															.setMessage("Game Server Response Error\n" + response.resultMsg)
															.setPositiveButton("OK", new DialogInterface.OnClickListener() {
																@Override
																public void onClick(DialogInterface dialog, int which) {

																}
															})
															.setNegativeButton("Transaction Finish", new DialogInterface.OnClickListener() {
																@Override
																public void onClick(DialogInterface dialog, int which) {

																	IAPV4.transactionFinish(receipt.product.marketPid, new IAPV4.IAPV4TransactionFinishListener() {
																		@Override
																		public void onIAPV4TransacionFinish(ResultAPI result, String marketPid) {

																			Toast.makeText(activity, "Restore Complete", Toast.LENGTH_SHORT).show();
																		}
																	});
																}
															})
															.create().show();
												}

											}
										}
									},"subscription");
								}
							}
						}

						addLog(builder.toString());

						showAlertDialog("IAPV4 Restore result", result.toString() + builder.toString());
					}
				});
				break;


			case R.id.btnIAPV4GetBalance:
			{
				addLog("\n\n=== HIVE 러비 상점일 경우 잔액 정보 조회. ===\n");
				addLog("IAPV4.getBalance(listener) Called\n");

				IAPV4.getBalanceInfo(new IAPV4.IAPV4BalanceInfoListener() {
					@Override
					public void onIAPV4Balance(ResultAPI result, int balance) {

						addLog(String.format("IAPV4BalanceInfoListener.onIAPV4Balance() Callback\nresult = %s\nbalance = %d\n"
							, result.toString(), balance));

						showAlertDialog("IAPV4 getBalance result", result.toString() + (result.isSuccess() ? "\n잔액 : " + balance : ""));
					}
				});
			}
				break;


			case R.id.btnIAPV4ShowPayment:
			{
				addLog("\n\n=== 러비 상점이나 구글 상점을 선택하기 위한 창을 띄운다. ===\n");
				addLog("IAPV4.showPayment(listener) Called\n");

				IAPV4.showMarketSelection(new IAPV4.IAPV4MarketInfoListener() {
					@Override
					public void onIAPV4MarketInfo(ResultAPI result, List<IAPV4Type> iapV4TypeList) {

						final String log = String.format("IAPV4MarketInfoListener.onIAPV4MarketInfo() Callback\nresult = %s\niapTypeList = %s\n"
								, result.toString(), iapV4TypeList != null ? iapV4TypeList.toString() : "null");

						addLog(log);

						TextView txtSelectedMarket = (TextView) IAPV4TestView.this.view.findViewById(R.id.txtIAPV4SelectedMarket);

						if(iapV4TypeList != null) {
							StringBuffer buffer = new StringBuffer();
							for (IAPV4Type iapType : iapV4TypeList) {
								buffer.append(" " + iapType.name());
							}

							txtSelectedMarket.setText(buffer.toString());
						}

						showAlertDialog("IAPV4 showPayment result", log);
					}
				});
			}
				break;


			case R.id.btnIAPV4ShowCharge:
			{
		    	addLog("\n\n=== 러비 충전 페이지 호출. ===\n");
		    	addLog("IAPV4.showCharge(listener) Called\n");

				IAPV4.showCharge(new IAPV4.IAPV4BalanceInfoListener() {
					@Override
					public void onIAPV4Balance(ResultAPI result, int balance) {

						final String log = String.format(Locale.US, "IAPV4BalanceInfoListener.onIAPV4Balance() Callback\nresult = %s\nbalance = %d\n"
								, result.toString(), balance);
						
						addLog(log);
						
						showAlertDialog("IAPV4 ShowCharge Result", log);
					}
				});
			}
				break;
			case R.id.btnIAPV4CheckPromote:
			{
				IAPV4.checkPromotePurchase(new IAPV4.IAPV4CheckPromotePurchaseListener() {
					@Override
					public void onIAPV4CheckPromotePurchase(ResultAPI result, String marketPid) {
						final String log = String.format(Locale.US, "IAPV4CheckPromotePurchaseListener.onIAPV4CheckPromotePurchase() Callback\nresult = %s\nmarketPid = %s\n"
								, result.toString(), marketPid);

						addLog(log);

						showAlertDialog("IAPV4 checkPromotePurchase Result", log);
					}
				});
			}
				break;
		}

	}
}



