package com.hive.sample;


import com.com2us.hivesdk.normal.freefull.google.global.android.common.R;
import com.hive.Configuration;
import com.hive.Configuration.ZoneType;
import com.hive.HiveActivity;
import com.hive.sample.uihelper.ViewPagerManager;

import android.annotation.TargetApi;
import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.util.Log;
import android.view.View;

import androidx.viewpager.widget.ViewPager;

import static android.view.View.SYSTEM_UI_FLAG_FULLSCREEN;
import static android.view.View.SYSTEM_UI_FLAG_HIDE_NAVIGATION;
import static android.view.View.SYSTEM_UI_FLAG_IMMERSIVE_STICKY;
import static android.view.View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN;
import static android.view.View.SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION;
import static android.view.View.SYSTEM_UI_FLAG_LAYOUT_STABLE;


/**
 * HIVE SDK 샘플 예제의 메인 Activity
 * 
 * @author ryuvsken
 */
public class MainActivity extends Activity {

	private ViewPager.OnPageChangeListener pageChangeListener;

	@Override
	public void onMultiWindowModeChanged(boolean isInMultiWindowMode, android.content.res.Configuration newConfig) {
		super.onMultiWindowModeChanged(isInMultiWindowMode, newConfig);

		HiveActivity.onMultiWindowModeChanged(this, isInMultiWindowMode, newConfig);
	}

	@Override
	public void onConfigurationChanged(android.content.res.Configuration newConfig) {
		super.onConfigurationChanged(newConfig);

		HiveActivity.onConfigurationChanged(this, newConfig);

	}

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		Log.i("MainActivity", "onCreate");
		
		this.requestWindowFeature(android.view.Window.FEATURE_NO_TITLE);

		HiveActivity.onCreate(this, savedInstanceState);

		this.initUi();
	}
	
	@Override
	protected void onStart() {
		super.onStart();
		
		HiveActivity.onStart(this);
	}
	
	@Override
	protected void onRestart() {
		super.onRestart();
		
		HiveActivity.onRestart(this);
	}

	@Override
    protected void onResume() {
    	super.onResume();

		Log.i("MainActivity", "onResume");

        final View decorView = getWindow().getDecorView();
        final int uiFlags = 0
                |SYSTEM_UI_FLAG_LAYOUT_STABLE
                |SYSTEM_UI_FLAG_HIDE_NAVIGATION
                |SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION
//                |SYSTEM_UI_FLAG_IMMERSIVE
                |SYSTEM_UI_FLAG_IMMERSIVE_STICKY
                |SYSTEM_UI_FLAG_FULLSCREEN
                |SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN
        ;
        decorView.setSystemUiVisibility(uiFlags);

        final Activity activity = this;

        decorView.setOnSystemUiVisibilityChangeListener(new View.OnSystemUiVisibilityChangeListener() {
            @Override
            public void onSystemUiVisibilityChange(int visibility) {
                if ((visibility & View.SYSTEM_UI_FLAG_FULLSCREEN) == 0) {
					Log.i("MainActivity", "Ma visibility a : " + visibility);
//                    new Handler().postDelayed(new Runnable() {
//                        @Override
//                        public void run() {
//                            decorView.setSystemUiVisibility(uiFlags);
//                        }
//                    }, 5000);
                    decorView.setSystemUiVisibility(uiFlags);
                } else {
					Log.i("MainActivity", "Ma visibility b : " + visibility);
                }
            }
        });
    	
    	HiveActivity.onResume(this);


    }
    
    
    @Override
    protected void onPause() {

    	HiveActivity.onPause(this);

    	super.onPause();
    }
    
    @Override
    protected void onStop() {
    	HiveActivity.onStop(this);
    	
    	super.onStop();
    }
    
    @Override
    protected void onDestroy() {
    	HiveActivity.onDestroy(this);

		releaseViewPager();
    	
    	super.onDestroy();
    }

	@Override
	protected void attachBaseContext(Context newBase) {
		
		super.attachBaseContext(HiveActivity.attachBaseContext(newBase));
	}

	@Override
	public void onWindowFocusChanged(boolean hasFocus) {
    	super.onWindowFocusChanged(hasFocus);
    	
    	HiveActivity.onWindowFocusChanged(this, hasFocus);
    }
    
    
    @Override
    protected void onNewIntent(Intent intent) {
    	super.onNewIntent(intent);

		Log.i("MainActivity", "onNewIntent");
    	HiveActivity.onNewIntent(this, intent);
    }
    
    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
    	super.onActivityResult(requestCode, resultCode, data);
    	
    	HiveActivity.onActivityResult(this, requestCode, resultCode, data);
    }
    

	// Android SDK 6.0 (android-23) API over
    @TargetApi(23)
	@Override
    public void onRequestPermissionsResult(int requestCode, String[] permissions, int[] grantResults) {
    	super.onRequestPermissionsResult(requestCode, permissions, grantResults);
    	
    	HiveActivity.onRequestPermissionsResult(this, requestCode, permissions, grantResults);
    }


	@Override
	protected void onSaveInstanceState(Bundle outState) {
		super.onSaveInstanceState(outState);

		HiveActivity.onSaveInstanceState(this, outState);
	}

	@Override
    public void onBackPressed() {
//    	super.onBackPressed();
    }
    
    
	@SuppressWarnings("deprecation")
	private void initUi() {
		
		setContentView(R.layout.main_activity);
		

		// HiveSDK 설정 대화 상자 노출
		ConfigurationDialog configurationDialog = new ConfigurationDialog(this);
		configurationDialog.showConfigurationDialog(new ConfigurationDialog.ConfigurationDialogListener() {

			@Override
			public void onConfigurationSelect(String appId, ZoneType zoneType, String gameLanguage, String gameServerId
					, Boolean useLog, String loggingUserId, Boolean isRemoteLogging, Boolean isRemoteDetailLogging, Boolean isAgeGateU13) {

				// 설정 대화 상자에서 설정이 완료되면 HiveSDK 설정 을 수행
				Configuration.setAppId(appId);
				Configuration.setZone(zoneType);
				Configuration.setGameLanguage(gameLanguage);
				Configuration.setServerId(gameServerId);
				Configuration.setUseLog(useLog);
				Configuration.setAgeGateU13(isAgeGateU13);

				buildViewPager();

//				AuthV4.setup(new AuthV4.AuthV4SetupListener() {
//					@Override
//					public void onAuthV4Setup(ResultAPI result, boolean isAutoSignIn, String did, ArrayList<AuthV4.ProviderType> providerTypeList) {
//						if( result.isSuccess() ) {
//							if( isAutoSignIn ) {
//								AuthV4.signIn(AuthV4.ProviderType.AUTO, new AuthV4.AuthV4SignInListener() {
//									@Override
//									public void onAuthV4SignIn(ResultAPI result, AuthV4.PlayerInfo playerInfo) {
//										if( result.isSuccess() ) {
//											GameUser.getInstance().login(new IGameListener() {
//												@Override
//												public void onGameListener(boolean result, Response response) {
//													Android.runOnMainThread(new Runnable() {
//														@Override
//														public void run() {
//															buildViewPager();
//														}
//													});
//												}
//											});
//
//										}
//									}
//								});
//							} else {
//								AuthV4.signIn(AuthV4.ProviderType.GUEST, new AuthV4.AuthV4SignInListener() {
//									@Override
//									public void onAuthV4SignIn(ResultAPI result, AuthV4.PlayerInfo playerInfo) {
//										if( result.isSuccess() ) {
//											GameUser.getInstance().login(new IGameListener() {
//												@Override
//												public void onGameListener(boolean result, Response response) {
//													Android.runOnMainThread(new Runnable() {
//														@Override
//														public void run() {
//															buildViewPager();
//														}
//													});
//												}
//											});
//
//										}
//									}
//								});
//							}
//
//						}
//					}
//				});

			}
		});
	}
	
	private void releaseViewPager() {
		
		final ViewPagerManager pagerAdapterManager = ViewPagerManager.getInstance();
		if( pagerAdapterManager != null ) {
			final ViewPager viewPager = pagerAdapterManager.getViewPager();
			if( viewPager != null ) {
				viewPager.removeOnPageChangeListener(pageChangeListener);

				pagerAdapterManager.releaseViewPager();
			}
		}
	}
	
	private void buildViewPager() {
		
		final ViewPagerManager pagerAdapterManager = ViewPagerManager.getInstance();
		pagerAdapterManager.setActivity(this);
		
		
		View view;

		// AuthV4 테스트 뷰 추가
		view = getLayoutInflater().inflate(R.layout.test_view_authv4, null);
		pagerAdapterManager.addView(new AuthV4TestView(this, view, "AuthV4 >>"));

		// AuthV4Helper 테스트 뷰 추가
		view = getLayoutInflater().inflate(R.layout.test_view_authv4_helper, null);
		pagerAdapterManager.addView(new AuthV4HelperTestView(this, view, "AuthV4 >>"));

		// ProviderGoogle 테스트 뷰 추가
		view = getLayoutInflater().inflate(R.layout.test_view_provider, null);
		pagerAdapterManager.addView(new ProviderTestView(this, view, "Provider >>"));

		// 인증 테스트 뷰 추가
		view = getLayoutInflater().inflate(R.layout.test_view_auth, null);
		pagerAdapterManager.addView(new AuthTestView(this, view, "AuthV1 >>"));
		
		
		// 소셜 테스트 뷰 추가
		view = getLayoutInflater().inflate(R.layout.test_view_social, null);
		pagerAdapterManager.addView(new SocialTestView(this, view, "<< Social >>"));
		
		
		// 프로모션 뷰의 테스트 뷰 추가
		view = getLayoutInflater().inflate(R.layout.test_view_promotion, null);
		pagerAdapterManager.addView(new PromotionTestView(this, view, "<< Promotion >>"));
		
		
		// 푸시 테스트 뷰 추가
		view = getLayoutInflater().inflate(R.layout.test_view_push, null);
		pagerAdapterManager.addView(new PushTestView(this, view, "<< Push Notification >>"));
		
		
		// IAP (빌링) 테스트 뷰 추가
		view = getLayoutInflater().inflate(R.layout.test_view_iap, null);
		pagerAdapterManager.addView(new IAPTestView(this, view, "<< IAP >>"));


		// IAPV4 (빌링) 테스트 뷰 추가
		view = getLayoutInflater().inflate(R.layout.test_view_iapv4, null);
		pagerAdapterManager.addView(new IAPV4TestView(this, view, "<< IAPV4 >>"));
		
		
		// Analytics 테스트 뷰 추가
		view = getLayoutInflater().inflate(R.layout.test_view_analytics, null);
		pagerAdapterManager.addView(new AnalyticsTestView(this, view, "Analytics >>"));


		final ViewPager viewPager = (ViewPager)findViewById(R.id.main_pager);
		viewPager.setAdapter(pagerAdapterManager);
		pagerAdapterManager.setViewPager(viewPager);


//		viewPager.setOnPageChangeListener(new OnPageChangeListener() {

		pageChangeListener = new ViewPager.OnPageChangeListener() {

			@Override
			public void onPageSelected(int position) {
				
				pagerAdapterManager.onPageChange(position);
			}

			@Override
			public void onPageScrolled(int arg0, float arg1, int arg2) {}

			@Override
			public void onPageScrollStateChanged(int arg0) {}
		};

		viewPager.addOnPageChangeListener(pageChangeListener);
	}
}



