package com.hive.sample;

import android.app.Dialog;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Handler;
import android.os.Looper;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.widget.BaseAdapter;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;

import com.hive.Promotion;
import com.hive.Promotion.PromotionBanner;
import com.hive.ResultAPI;
import com.hive.base.Resource;
import com.hive.sample.uihelper.HiveImageLoader;
import com.hive.sample.uihelper.HiveImageLoadingListener;
import com.hive.sample.uihelper.ViewPagerBase;

import java.util.ArrayList;


/**
 * HIVE SDK 의 IAP (In App Purchase) 상점 정보를 보여주는 대화 상자
 *
 * @author hife
 */
public class PromotionBannerDialog extends Dialog {

	ListView listview_banners;

	ListViewAdapter listviewadapter_banner;

	ViewPagerBase forLog;


	public PromotionBannerDialog(Context context, ArrayList<PromotionBanner> bannerInfoList, ViewPagerBase forLog) {
		super(context);

		this.forLog = forLog;

		this.requestWindowFeature(Window.FEATURE_NO_TITLE);
		this.setCancelable(true);

		setContentView(Resource.getLayoutId(context, "test_view_promotion_banner_info"));

		listview_banners = (ListView) findViewById(Resource.getViewId(context, "promotion_listview_banners"));

		listviewadapter_banner = new ListViewAdapter();

		listview_banners.setAdapter(listviewadapter_banner);

		for (PromotionBanner promotionBanner : bannerInfoList) {
			listviewadapter_banner.add(promotionBanner);
		}

		listview_banners.setLayoutParams(new LinearLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT));
	}


	@Override
	public void dismiss() {
		//TODO list bitmap recycle

		super.dismiss();
	}


	class ListViewAdapter extends BaseAdapter {

		private ArrayList<PromotionBanner> bannerInfoList;

		public ListViewAdapter() {
			bannerInfoList = new ArrayList<PromotionBanner>();
		}

		@Override
		public int getCount() {
			return bannerInfoList.size();
		}

		@Override
		public Object getItem(int position) {
			return bannerInfoList.get(position);
		}

		@Override
		public long getItemId(int position) {
			return position;
		}

		@Override
		public View getView(int position, View convertView, ViewGroup parent) {

			final Context context = parent.getContext();

			final PromotionBanner promotionBanner = bannerInfoList.get(position);
//			final ViewHolder holder;


//			if (convertView == null) {

				LayoutInflater inflater = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
		        convertView = inflater.inflate(Resource.getLayoutId(context, "test_view_promotion_banner"), parent, false);

//		        holder = new ViewHolder();

			final TextView promotion_textview_banner_desc = (TextView) convertView.findViewById(Resource.getViewId(context, "promotion_textview_banner_desc"));
			final ImageView promotion_imageview_banner = (ImageView) convertView.findViewById(Resource.getViewId(context, "promotion_imageview_banner"));
//
//		        convertView.setTag(holder);
//			}
//
//			else {
//				holder = (ViewHolder) convertView.getTag();
//			}

			//banner desc
			StringBuilder builder = new StringBuilder();
			builder
					.append("PID: ").append(promotionBanner.pid)
					.append("\n")
					.append("Start: ").append(promotionBanner.displayStartDate)
					.append("\n")
					.append("End: ").append(promotionBanner.displayEndDate);

			promotion_textview_banner_desc.setText(builder.toString());

			//banner image
			HiveImageLoader.getInstance(context).loadImage(promotionBanner.imageUrl, new HiveImageLoadingListener() {

				@Override
				public void onLoadingFailed(String url, String failReason) {
					// TODO set fail default image
				}

				@Override
				public void onLoadingComplete(String url, final byte[] data) {

					Bitmap img = BitmapFactory.decodeByteArray(data, 0, data.length);
					promotion_imageview_banner.setImageBitmap(img);
				}
			});
//			setImageView(context, promotionBanner.imageUrl, promotion_imageview_banner);


			convertView.setOnClickListener(new View.OnClickListener() {

				@Override
				public void onClick(View v) {

					String pid = String.valueOf(promotionBanner.pid);

					forLog.addLog("Promotion.banner(pid: " + pid + ") opened.\n");

					Log.v("sample", "\n\n=== HIVE Promotion DIRECT Banner. (" + pid + ") ===\n");

					Promotion.showCustomContents(Promotion.PromotionCustomType.DIRECT, pid, new Promotion.PromotionViewListener() {
						@Override
						public void onPromotionView(ResultAPI result, Promotion.PromotionViewResultType promotionEventType) {

							forLog.addLog(String.format("PromotionViewListener.onPromotionView() Callback\nresult = %s\ntype = %s\n", result.toString(), promotionEventType.toString()));
						}
					});

				}
			});


			return convertView;
		}

		public void add(PromotionBanner promotionBanner) {
			bannerInfoList.add(promotionBanner);
			this.notifyDataSetChanged();
		}

		public void remove(int position) {
			bannerInfoList.remove(position);
			this.notifyDataSetChanged();
		}
	}


//	class ViewHolder {
//
//	    TextView promotion_textview_banner_desc;
//	    ImageView promotion_imageview_banner;
//	}
//
//
//	void setImageView(Context context, String imageUrl, final ImageView view) {
//
//		HiveImageLoader.getInstance(context).loadImage(imageUrl, new HiveImageLoadingListener() {
//
//			@Override
//			public void onLoadingFailed(String url, String failReason) {
//				// TODO set fail default image
//			}
//
//			@Override
//			public void onLoadingComplete(String url, final byte[] data) {
//
//				new Handler(Looper.getMainLooper()).post(new Runnable() {
//
//					@Override
//					public void run() {
//						Bitmap img = BitmapFactory.decodeByteArray(data, 0, data.length);
//						view.setImageBitmap(img);
//					}
//				});
//			}
//		});
//	}


}
