package com.hive.sample;



import android.app.Activity;
import android.view.View;
import android.widget.EditText;
import android.widget.RadioGroup;
import android.widget.ToggleButton;

import com.hive.Push;
import com.hive.Push.LocalPush;
import com.hive.Push.LocalPushListener;
import com.hive.Push.RemotePush;
import com.hive.Push.RemotePushListener;
import com.hive.ResultAPI;


import com.hive.sample.uihelper.ViewPagerBase;

import com.com2us.hivesdk.normal.freefull.google.global.android.common.R;




/**
 * HIVE SDK 의 Push Notification 기능 테스트 뷰
 * 
 * @author ryuvsken
 */
public class PushTestView extends ViewPagerBase implements View.OnClickListener {

	private int bucketType = 0;

	public PushTestView(Activity activity, View view, String title) {
		super(activity, view, title);

		initUi();
	}
	
	private void initUi() {

		createLogView(R.id.logview, R.id.btnLogDialog, R.id.btnLogClear);
		
		this.view.findViewById(R.id.btnPushGetPushSetting).setOnClickListener(this);
		this.view.findViewById(R.id.btnPushSetPushSetting).setOnClickListener(this);
		this.view.findViewById(R.id.btnPushGetRemote).setOnClickListener(this);
		this.view.findViewById(R.id.btnPushSetRemote).setOnClickListener(this);
		this.view.findViewById(R.id.btnPushRegisterLocal).setOnClickListener(this);
		this.view.findViewById(R.id.btnPushUnregisterLocal).setOnClickListener(this);

		final ToggleButton tbRemoteEnabledWhileRunning = (ToggleButton)this.view.findViewById(R.id.tbRemoteEnabledWhileRunning);
		final ToggleButton tbLocalEnabledWhileRunning = (ToggleButton)this.view.findViewById(R.id.tbLocalEnabledWhileRunning);

		Push.getForegroundPush(new Push.PushSettingListener() {
			@Override
			public void onPushSetting(ResultAPI result, Push.PushSetting pushSetting) {
				if (result.isSuccess()) {
					tbRemoteEnabledWhileRunning.setChecked(pushSetting.isRemotePushEnabledWhileRunning());
					tbLocalEnabledWhileRunning.setChecked(pushSetting.isLocalPushEnabledWhileRunning());
				}
			}
		});
		
	}


	@Override
	public void onClick(View v) {

		int rid = v.getId();
		
		switch(rid) {

		case R.id.btnPushGetPushSetting: {

			final ToggleButton tbRemoteEnabledWhileRunning = (ToggleButton)this.view.findViewById(R.id.tbRemoteEnabledWhileRunning);
			final ToggleButton tbLocalEnabledWhileRunning = (ToggleButton)this.view.findViewById(R.id.tbLocalEnabledWhileRunning);

			addLog("\n\n=== 푸시 설정값 조회 ===\n");
			addLog("Push.getForegroundPush(listener) called\n");

			Push.getForegroundPush(new Push.PushSettingListener() {
				@Override
				public void onPushSetting(ResultAPI result, Push.PushSetting pushSetting) {

					addLog(String.format("PushSettingListener.onPushSetting() Callback\nresult = %s\npushSetting = %s\n"
							, result.toString(), pushSetting != null ? pushSetting.toString() : "null"));

					if (result.isSuccess()) {
						tbRemoteEnabledWhileRunning.setChecked(pushSetting.isRemotePushEnabledWhileRunning());
						tbLocalEnabledWhileRunning.setChecked(pushSetting.isLocalPushEnabledWhileRunning());
					}
				}
			});
		}
			break;

		case R.id.btnPushSetPushSetting: {

			final ToggleButton tbRemoteEnabledWhileRunning = (ToggleButton)this.view.findViewById(R.id.tbRemoteEnabledWhileRunning);
			final ToggleButton tbLocalEnabledWhileRunning = (ToggleButton)this.view.findViewById(R.id.tbLocalEnabledWhileRunning);

			addLog("\n\n=== 푸시 설정값 저장 ===\n");
			addLog("Push.setForegroundPush(pushSetting, listener) called\n");

			Push.PushSetting pushSetting = new Push.PushSetting(tbRemoteEnabledWhileRunning.isChecked(), tbLocalEnabledWhileRunning.isChecked());

			Push.setForegroundPush(pushSetting, new Push.PushSettingListener() {
				@Override
				public void onPushSetting(ResultAPI result, Push.PushSetting pushSetting) {
					addLog(String.format("PushSettingListener.onPushSetting() Callback\nresult = %s\npushSetting = %s\n"
							, result.toString(), pushSetting != null ? pushSetting.toString() : "null"));


				}
			});
		}
			break;

		case R.id.btnPushGetRemote: {
			
			final ToggleButton tbUseNotice = (ToggleButton)this.view.findViewById(R.id.tbUseNotice);
			final ToggleButton tbUseNoticeNight = (ToggleButton)this.view.findViewById(R.id.tbUseNoticeNight);
			
			addLog("\n\n=== 리모트 푸시 정보 조회 ===\n");
			addLog("Push.getRemotePush(listener) called\n");
			
			Push.getRemotePush(new RemotePushListener() {
				
				@Override
				public void onPushToken(ResultAPI result, RemotePush remotePush) {

					addLog(String.format("RemotePushListener.onPushToken() Callback\nresult = %s\nremotePush = %s\n"
						, result.toString(), remotePush!=null?remotePush.toString():"null"));

					
					if (!result.isSuccess()) {
						
//						AlertDialog.Builder alertDialog = new AlertDialog.Builder(HiveActivity.getRecentActivity());
//						alertDialog.setTitle("setPushToken 결과 받음");
//						alertDialog.setMessage("에러가 발생했으며 내용은 다음과 같습니다.\n"
//								+ result.message);
//						alertDialog.setPositiveButton("확인", null);
//						alertDialog.show();
					}
					else {
						
						tbUseNotice.setChecked(remotePush.isAgreeNotice);
						tbUseNoticeNight.setChecked(remotePush.isAgreeNight);
					}
				}
			});
		}
			break;
			
			
		case R.id.btnPushSetRemote: {
			
			ToggleButton tbUseNotice = (ToggleButton)this.view.findViewById(R.id.tbUseNotice);
			ToggleButton tbUseNoticeNight = (ToggleButton)this.view.findViewById(R.id.tbUseNoticeNight);
			
			RemotePush remotePush = new RemotePush(tbUseNotice.isChecked(), tbUseNoticeNight.isChecked());

			
			addLog("\n\n=== 리모트 푸시 정보 설정 ===\n");
			addLog(String.format("Push.setRemotePush(remotePush, listener) called\nremotePush = %s\n", remotePush.toString()));
			
			Push.setRemotePush(remotePush, new RemotePushListener() {
				
				@Override
				public void onPushToken(ResultAPI result, RemotePush remotePush) {

					addLog(String.format("RemotePushListener.onPushToken() Callback\nresult = %s\nremotePush = %s\n"
							, result.toString(), remotePush!=null?remotePush.toString():"null"));
					
					if (!result.isSuccess()) {
						
//						AlertDialog.Builder alertDialog = new AlertDialog.Builder(HiveActivity.getRecentActivity());
//						alertDialog.setTitle("getPushToken 결과 받음");
//						alertDialog.setMessage("에러가 발생했으며 내용은 다음과 같습니다.\n"
//								+ result.message);
//						alertDialog.setPositiveButton("확인", null);
//						alertDialog.show();
					}
					else {
						
//						AlertDialog.Builder alertDialog = new AlertDialog.Builder(HiveActivity.getRecentActivity());
//						alertDialog.setTitle("setPushToken 결과 받음");
//						alertDialog.setMessage("다음의 PushToken이 정상적으로 서버에 전송되었습니다.\n"
//								+ remotePush.toString());
//						alertDialog.setPositiveButton("확인", null);
//						alertDialog.show();
					}
				}
			});
		}
			break;
			
			
		case R.id.btnPushRegisterLocal: {
			
			EditText etTitle = (EditText)this.view.findViewById(R.id.etPushTitle);
			EditText etMessage = (EditText)this.view.findViewById(R.id.etPushMessage);
			EditText etBigMesage = this.view.findViewById(R.id.etPushBigMessage);
			EditText etNo = (EditText)this.view.findViewById(R.id.etPushNo);
			EditText etAfter = (EditText)this.view.findViewById(R.id.etPushAfter);
			EditText etBucketSize = this.view.findViewById(R.id.etBucketSize);

			RadioGroup rgBucketType = this.view.findViewById(R.id.rgBucketType);

			long after = 0L;
			int noticeId = 1;

			int bucketSize = 0;

			String title = null;
			String msg = null;
			String bigMsg = null;

			try {
				after = Long.valueOf(etAfter.getText().toString());
				noticeId = Integer.valueOf(etNo.getText().toString());
				title = etTitle.getText().toString();
				msg = etMessage.getText().toString();
				bigMsg = etBigMesage.getText().toString();
				bucketSize = Integer.parseInt(etBucketSize.getText().toString());
				
			} catch (Exception e) {
				e.printStackTrace();
			}

			rgBucketType.setOnCheckedChangeListener(new RadioGroup.OnCheckedChangeListener() {
				@Override
				public void onCheckedChanged(RadioGroup group, int checkedId) {
					switch (checkedId) {
						case R.id.rbOverWriteType:
							bucketType = 0;
							break;
						case R.id.rbInboxingType:
							bucketType = 1;
							break;
						case R.id.rbBigTextType:
							bucketType = 2;
							break;
						default:
							bucketType = 0;
							break;
					}
				}
			});
			
			LocalPush localPush = new LocalPush(noticeId, title, msg, after,
					bigMsg, "푸시 티커",
					null, null, null, null, 
					null, bucketType, bucketSize, null, null);
			
			
			addLog("\n\n=== 로컬 푸시 등록 ===\n");
			addLog(String.format("Push.registerLocalPush(localPush, listener) called\nlocalPush = %s\n"
				, localPush!=null?localPush.toString():"null"));
			
			Push.registerLocalPush(localPush, new LocalPushListener() {
				
				@Override
				public void onRegisterLocalPush(ResultAPI result, LocalPush localPush) {
					
					addLog(String.format("RemotePushListener.onPushToken() Callback\nresult = %s\nlocalPush = %s\n"
							, result.toString(), localPush!=null?localPush.toString():"null"));
				}
			});
		}
			break;
			
			
		case R.id.btnPushUnregisterLocal: {
			
			EditText etNo = (EditText)this.view.findViewById(R.id.etPushNo);
			int noticeID = 1;
			try {
				noticeID = Integer.valueOf(etNo.getText().toString());
			} catch (Exception e) {
				e.printStackTrace();
			}

			
			addLog("\n\n=== 로컬 푸시 해제 ===\n");
			addLog(String.format("Push.unregisterLocalPush(noticeID = %d) called\n", noticeID));
			
			Push.unregisterLocalPush(noticeID);
		}
			break;
			
		}

	}
}



