package com.hive.sample;


import android.app.Activity;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.EditText;
import android.widget.Spinner;

import com.com2us.hivesdk.normal.freefull.google.global.android.common.R;
import com.hive.ResultAPI;
import com.hive.SocialFacebook.ProfileFacebook;
import com.hive.SocialFacebook.SocialFacebookMessage;
import com.hive.SocialGoogle;
import com.hive.SocialHive;
import com.hive.SocialHive.FriendType;
import com.hive.SocialHive.HiveDialogType;
import com.hive.SocialHive.MessageContent;
import com.hive.SocialHive.ProfileHive;
import com.hive.SocialHive.SocialBadge;
import com.hive.base.Android;
import com.hive.sample.uihelper.SelectDialog;
import com.hive.sample.uihelper.ViewPagerBase;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;




/**
 * HIVE SDK 의 Social 기능 테스트 뷰
 * 
 * @author ryuvsken
 */
public class SocialTestView extends ViewPagerBase implements View.OnClickListener {
	
	
	private List<ProfileHive> hiveFriends;
	private List<ProfileFacebook>	facebookFriends;
	final private String			defaultSampleImageURL	= "http://gw.com2us.com/gw_resource/css/gw_skin01/images_gw/logo.png";
	
	public SocialTestView(Activity activity, View view, String title) {
		super(activity, view, title);

		initUi();
		hiveFriends = new ArrayList<ProfileHive>();
		facebookFriends = new ArrayList<ProfileFacebook>();
	}

	
	@SuppressWarnings({ "unchecked", "rawtypes" })
	private void initUi() {

		createLogView(R.id.logview, R.id.btnLogDialog, R.id.btnLogClear);
		
		
		ArrayAdapter spAdapter;
		
		// Social Hive
		this.view.findViewById(R.id.btnSocialHiveGetMyProfile).setOnClickListener(this);
		this.view.findViewById(R.id.btnSocialHiveSetMyProfile).setOnClickListener(this);
		
		spAdapter = new ArrayAdapter(activity, android.R.layout.simple_spinner_item);
		spAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
		spAdapter.add("IN_GAME");
		spAdapter.add("OUT_GAME");
		spAdapter.add("INVITED");
		spAdapter.add("ALL_GAME");
		((Spinner)this.view.findViewById(R.id.spFriendType)).setAdapter(spAdapter);
		
		this.view.findViewById(R.id.btnSocialHiveGetFriends).setOnClickListener(this);
		this.view.findViewById(R.id.btnSocialHiveSendMessage).setOnClickListener(this);
		this.view.findViewById(R.id.btnSocialHiveSendInvitationMessage).setOnClickListener(this);
		
		spAdapter = new ArrayAdapter(activity, android.R.layout.simple_spinner_item);
		spAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
		for (SocialHive.HiveDialogType type : SocialHive.HiveDialogType.values())
			spAdapter.add(type.name());

		((Spinner)this.view.findViewById(R.id.spHiveDialogType)).setAdapter(spAdapter);
		
		EditText editText = (EditText) this.view.findViewById(R.id.etHiveDialogVid);
		editText.setFocusable(false);
		editText.setOnClickListener(this);

		this.view.findViewById(R.id.btnSocialHiveShowHiveDialog).setOnClickListener(this);
		this.view.findViewById(R.id.btnSocialHiveCheckNotification).setOnClickListener(this);
		

		// Social Facebook
		this.view.findViewById(R.id.btnSocialFacebookGetMyProfile).setOnClickListener(this);
		this.view.findViewById(R.id.btnSocialFacebookGetFriends).setOnClickListener(this);
		this.view.findViewById(R.id.btnSocialFacebookSendMessage).setOnClickListener(this);
		this.view.findViewById(R.id.btnSocialFacebookShowInvitationDialog).setOnClickListener(this);
		this.view.findViewById(R.id.btnSocialFacebookPostFacebook).setOnClickListener(this);
		
		
		// Social Google
		this.view.findViewById(R.id.btnSocialGoogleGetMyProfile).setOnClickListener(this);
	}


	@Override
	public void onClick(View v) {

		int rid = v.getId();
		
		switch(rid) {
		
// SocialHive
		
			case R.id.btnSocialHiveGetMyProfile: {
				
				addLog("\n\n=== HIVE 인증 사용자의 자기 프로필 조회 ===\n");
				addLog("SocialHive.getMyProfile(listener) Called\n");
				
				com.hive.SocialHive.getMyProfile(new com.hive.SocialHive.ProfileListener() {

					@Override
					public void onProfile(ResultAPI result, List<ProfileHive> profileList) {

						addLog(String.format("SocialHive.ProfileListener.onProfile() Callback\nresult = %s\nProfileHive List = %s\n"
							, result.toString(), profileList!=null?profileList.toString():"null"));

						ProfileHive profile = null;
						if (result.isSuccess()) {
							profile = profileList.get(0);
						}
					}
				});
			}
			break;
			
			
			case R.id.btnSocialHiveSetMyProfile: {
				
				new SelectDialog.Builder(this.getActivity()).setTitle("SetMyProfile").addInputText("displayName").setDialogResultListener(new SelectDialog.OnDialogResultListener() {

					@Override
					public void onDialogResult(boolean completed, Map<String, Object> result) {

						if (completed) {
							String displayName = (String) result.get("displayName");

							
							addLog("\n\n=== HIVE 인증 사용자의 자기 대화명 수정 ===\n");
							addLog(String.format("SocialHive.setMyProfile(\"%s\", listener) Called\n", displayName));

							com.hive.SocialHive.setMyProfile(displayName, new com.hive.SocialHive.ProfileListener() {

								@Override
								public void onProfile(ResultAPI result, List<ProfileHive> profileList) {

									addLog(String.format("SocialHive.ProfileListener.onProfile() Callback\nresult = %s\nProfileHive List = %s\n"
										, result.toString(), profileList!=null?profileList.toString():"null"));
								}
							});
						}
					}
				}).build().show();
			}
			break;
			
			
			case R.id.btnSocialHiveGetFriends: {
				
				Spinner spinner;
				
				spinner = (Spinner)this.view.findViewById(R.id.spFriendType);
				String selectedValue = (String) spinner.getAdapter().getItem(spinner.getSelectedItemPosition());
				FriendType friendType = FriendType.valueOf(selectedValue.toUpperCase());
				
				
				addLog("\n\n=== HIVE 인증 사용자의 친구 목록 조회 ===\n");
				addLog(String.format("SocialHive.getFriends(friendType = %s, listener) Called\n", friendType.name()));
				
				com.hive.SocialHive.getFriends(friendType, new com.hive.SocialHive.ProfileListener() {

					@Override
					public void onProfile(ResultAPI result, List<ProfileHive> profileList) {
						
						addLog(String.format("SocialHive.ProfileListener.onProfile() Callback\nresult = %s\nProfileHive List = %s\n"
								, result.toString(), profileList!=null?profileList.toString():"null"));

						if (result.isSuccess()) {
							hiveFriends.clear();
							hiveFriends.addAll(profileList);


							ArrayList<String> vidList = new ArrayList<>();

							for (ProfileHive profile : profileList) {

								vidList.add(profile.vid);
							}

							String[] vids = new String[vidList.size()];
							vids = vidList.toArray(vids);

							com.hive.SocialHive.getProfiles(vids, new com.hive.SocialHive.ProfileListener() {

								@Override
								public void onProfile(ResultAPI result, List<ProfileHive> profileList) {

									addLog(String.format("SocialHive.ProfileListener.onProfile() Callback\nresult = %s\nProfileHive List = %s\n"
											, result.toString(), profileList!=null?profileList.toString():"null"));

								}
							});


						} else {

							String [] vids = {"61217583"};
							com.hive.SocialHive.getProfiles(vids, new com.hive.SocialHive.ProfileListener() {

								@Override
								public void onProfile(ResultAPI result, List<ProfileHive> profileList) {

									addLog(String.format("SocialHive.ProfileListener.onProfile() Callback\nresult = %s\nProfileHive List = %s\n"
											, result.toString(), profileList!=null?profileList.toString():"null"));

								}
							});

						}
					}
				});
			}
			break;
			
			
			case R.id.btnSocialHiveSendMessage: {

				// 입력 창 띄우기
				SelectDialog.Builder builder = new SelectDialog.Builder(this.getActivity()).setTitle("SendMessage").addInputText("vid");
				if (hiveFriends.isEmpty()) {
					builder.addInputText("uid","");
				}
				else {
					List<String> uidLists = new ArrayList<String>(hiveFriends.size());
					for (ProfileHive profile : hiveFriends)
					{
						uidLists.add(profile.uid);
					}
					builder.addSelectList("uid", uidLists);
				}
				builder.addInputText("message", "메세지").addInputText("imageUrl", "이미지 URL").addInputText("thumbnailUrl", "썸네일 URL").addSwitch("usePush", true)
					.setDialogResultListener(new SelectDialog.OnDialogResultListener() {

						@Override
						public void onDialogResult(boolean completed, Map<String, Object> result) {

							if (completed) {
								MessageContent messageContent = new MessageContent();
								messageContent.vid = (String) result.get("vid");
								messageContent.uid = (String) result.get("uid");
								messageContent.message = (String) result.get("message");
								messageContent.imageUrl = (String) result.get("imageUrl");
								messageContent.thumbnailUrl = (String) result.get("thumbnailUrl");
								messageContent.usePush = (Boolean) result.get("usePush");

								
								addLog("\n\n=== HIVE 친구에게 메시지 전송 요청 ===\n");
								addLog(String.format("SocialHive.sendMessage(messageContent, listener) Called\nmessageContent = %s\n"
									, messageContent.toString()));

								com.hive.SocialHive.sendMessage(messageContent, new com.hive.SocialHive.SendMessageListener() {

									@Override
									public void onSendMessage(ResultAPI result) {

										addLog(String.format("SocialHive.SendMessageListener.onSendMessage() Callback\nresult = %s\n", result.toString()));
									}
								});
							}

						}
				}).build().show();

			}
			break;
			
			
			case R.id.btnSocialHiveSendInvitationMessage: {
				
				// 입력 창 띄우기
				SelectDialog.Builder builder = new SelectDialog.Builder(this.getActivity()).setTitle("SendInvitationMessage").addInputText("vid");
				if (hiveFriends.isEmpty()) {
					builder.addInputText("uid");
				}
				else {
					List<String> uidLists = new ArrayList<String>(hiveFriends.size());
					for (ProfileHive profile : hiveFriends) {
						uidLists.add(profile.uid);
					}
					builder.addSelectList("uid", uidLists);
				}
				builder.addInputText("message", "메세지").addInputText("imageUrl", "이미지 URL").addInputText("thumbnailUrl", "썸네일 URL").addSwitch("usePush", true)
						.setDialogResultListener(new SelectDialog.OnDialogResultListener() {

					@Override
					public void onDialogResult(boolean completed, Map<String, Object> result) {

						if (completed) {
							MessageContent messageContent = new MessageContent();
							messageContent.vid = (String) result.get("vid");
							messageContent.uid = (String) result.get("uid");
							messageContent.message = (String) result.get("message");
							messageContent.imageUrl = (String) result.get("imageUrl");
							messageContent.thumbnailUrl = (String) result.get("thumbnailUrl");
							messageContent.usePush = (Boolean) result.get("usePush");

							
							addLog("\n\n=== HIVE 친구에게 초대 메시지 전송 요청 ===\n");
							addLog(String.format("SocialHive.sendInvitationMessage(messageContent, listener) Called\nmessageContent = %s\n"
								, messageContent.toString()));

							com.hive.SocialHive.sendInvitationMessage(messageContent, new com.hive.SocialHive.SendMessageListener() {

								@Override
								public void onSendMessage(ResultAPI result) {

									addLog(String.format("SocialHive.sendInvitationMessage.onSendMessage() Callback\nresult = %s\n", result.toString()));
								}
							});
						}

					}
				}).build().show();
			}
			break;
			
			case R.id.etHiveDialogVid: {
				SelectDialog.Builder builder = new SelectDialog.Builder(this.getActivity()).setTitle("InputVid");
				if (hiveFriends.isEmpty()) {
					builder.addInputText("vid");
				}
				else {
					List<String> vidLists = new ArrayList<String>(hiveFriends.size());
					for (ProfileHive profile : hiveFriends) {
						vidLists.add(profile.vid);
					}
					builder.addSelectList("vid", vidLists);
				}
				builder.setDialogResultListener(new SelectDialog.OnDialogResultListener() {

					@Override
					public void onDialogResult(boolean completed, Map<String, Object> result) {

						// TODO Auto-generated method stub
						if (completed) {
							EditText editText = (EditText) view.findViewById(R.id.etHiveDialogVid);
							String vid = (String) result.get("vid");
							editText.setText(vid);
						}

					}
				}).build().show();
				;

			}
				break;
			
			case R.id.btnSocialHiveShowHiveDialog: {
				
				Spinner spinner;
				
				spinner = (Spinner)this.view.findViewById(R.id.spHiveDialogType);
				String selectedValue = (String) spinner.getAdapter().getItem(spinner.getSelectedItemPosition());
				HiveDialogType hiveDialogType = HiveDialogType.valueOf(selectedValue.toUpperCase());
				

				EditText editText;

				editText = (EditText) this.view.findViewById(R.id.etHiveDialogVid);
				String vid = editText.getText().toString();
				// String vid = "";

				addLog("\n\n=== HIVE 웹뷰 대화상자 호출 ===\n");
				addLog(String.format("SocialHive.showHiveDialog(hiveDialogType = %s, vid = %s, listener) Called\n"
						, hiveDialogType.name(), vid));
				
				com.hive.SocialHive.showHiveDialog(hiveDialogType, vid, new com.hive.SocialHive.ShowHiveDialogListener() {
					
					@Override
					public void onShowHiveDialog(ResultAPI result) {
						addLog(String.format("SocialHive.ShowHiveDialogListener.onShowHiveDialog() Callback\nresult = %s\n", result.toString()));
					}
				});
			}
			break;
			
			
			case R.id.btnSocialHiveCheckNotification: {
				addLog("\n\n=== HIVE 인증 사용자의 쪽지 조회 ===\n");
				addLog("SocialHive.getBadgeInfo(listener) Called\n");

				com.hive.SocialHive.getBadgeInfo(new com.hive.SocialHive.SocialBadgeListener() {

					@Override
					public void onSocialBadge(ResultAPI result, SocialBadge badge) {

						addLog(String.format("SocialHive.ProfileListener.onProfile() Callback\nresult = %s\nProfileHive List = %s\n", result.toString(), badge != null ? badge.messageCount : "null"));

					}
				});
			}
				break;
			
// SocialFacebook
			
			case R.id.btnSocialFacebookGetMyProfile: {
				
				addLog("\n\n=== Facebook 프로필 정보 조회 ===\n");
				addLog("SocialFacebook.getMyProfile(listener) Called\n");
				
				com.hive.SocialFacebook.getMyProfile(new com.hive.SocialFacebook.ProfileListener() {
					
					@Override
					public void onProfile(ResultAPI result, List<ProfileFacebook> profileList) {

						addLog(String.format("SocialFacebook.ProfileListener.onProfile() Callback\nresult = %s\nprofileList = %s\n"
							, result.toString(), profileList!=null?profileList.toString():"null"));
					}
				});
			}
			break;
			
			
			case R.id.btnSocialFacebookGetFriends: {
				
				addLog("\n\n=== Facebook 친구 목록 조회 ===\n");
				addLog("SocialFacebook.getFriends(listener) Called\n");
				
				com.hive.SocialFacebook.getFriends(new com.hive.SocialFacebook.ProfileListener() {
					
					@Override
					public void onProfile(ResultAPI result, List<ProfileFacebook> profileList) {

						addLog(String.format("SocialFacebook.ProfileListener.onProfile() Callback\nresult = %s\nprofileList = %s\n"
							, result.toString(), profileList!=null?profileList.toString():"null"));
					}
				});
			}
			break;
			
			
			case R.id.btnSocialFacebookSendMessage: {

				SelectDialog.Builder builder = new SelectDialog.Builder(this.getActivity())
						.setTitle("SocialFacebookSendMessage")
						.addInputText("message", "전달할 메세지.")
						.addInputText("dialogTitle","메세지 전달할 다이얼로그 제목.");

				if( facebookFriends.isEmpty() )
					builder.addInputText("recipients", "메세지를 보낼 친구가 없거나, 친구목록을 업데이트 하지 않았습니다.");
				else
				{
					List<String> idLists = new ArrayList<String>();
					for( ProfileFacebook pf : facebookFriends )
					{
						idLists.add(pf.uid);
					}
					if (!idLists.isEmpty())
						builder.addSelectList("recipients", idLists);
				}
				
				builder.setDialogResultListener(new SelectDialog.OnDialogResultListener() {
					
					@Override
					public void onDialogResult(boolean completed, Map<String, Object> result) {

						if (completed) {
							
							SocialFacebookMessage content = Android.fromMap(result, SocialFacebookMessage.class);

							content.data = "";
							
							addLog("\n\n=== Facebook 인증 사용자의 친구에게 메시지를 전송 ===\n");
							addLog(String.format("SocialFacebook.sendMessageFacebook(content, listener) Called\ncontent = %s\n"
								, content!=null?content.toString():"null"));

							com.hive.SocialFacebook.sendMessageFacebook(content, new com.hive.SocialFacebook.SendMessageFacebookListener() {

								@Override
								public void onSendMessageFacebook(ResultAPI result) {

									addLog(String.format("SocialFacebook.SendMessageFacebookListener.onSendMessageFacebook() Callback\nresult = %s\n", result.toString()));
								}
							});
						}

					}
				}).build().show();

			}
			break;
			
			
			case R.id.btnSocialFacebookShowInvitationDialog: {
				
				SelectDialog.Builder builder = new SelectDialog.Builder(this.getActivity())
						.setTitle("SocialFacebookAppInvite")
						.addInputText("message", "초대 메세지.")
						.addInputText("dialogTitle","초대 다이얼로그 제목.");
				builder.setDialogResultListener(new SelectDialog.OnDialogResultListener() {

					@Override
					public void onDialogResult(boolean completed, Map<String, Object> result) {

						if (completed) {
							SocialFacebookMessage content = Android.fromMap(result, SocialFacebookMessage.class);

							content.data = "";
							content.recipients = null;
							
							addLog("\n\n=== Facebook 인증 사용자의 친구에게 초대 요청을 위한 대화상자 호출. ===\n");
							addLog(String.format("SocialFacebook.showInvitationDialog(content, listener) Called\ncontent = %s\n"
								, content!=null?content.toString():"null"));

							com.hive.SocialFacebook.showInvitationDialog(content, new com.hive.SocialFacebook.ShowInvitationDialogListener() {

								@Override
								public void onShowInvitationDialog(ResultAPI result, List<String> invitedUserList) {

									addLog(String.format("SocialFacebook.ShowInvitationDialogListener.onShowInvitationDialog() Callback\nresult = %s\ninvitedUserList - %s\n"
										, result.toString(), invitedUserList != null ? invitedUserList.toString() : "null"));

								}
							});
						}

					}
				}).build().show();

			}
			break;
			
			
			case R.id.btnSocialFacebookPostFacebook: {

				SelectDialog.Builder builder = new SelectDialog.Builder(this.getActivity())
						.setTitle("SocialFacebookPost")
						.addInputText("contentURL", "내용 클릭/터치시 이동하는 URL");

				builder.setDialogResultListener(new SelectDialog.OnDialogResultListener() {

					@Override
					public void onDialogResult(boolean completed, Map<String, Object> result) {

						if (completed) {
							String contentURL = (String)(result.get("contentURL"));
							// if (TextUtils.isEmpty(content.imageURL))
							// content.imageURL = defaultSampleImageURL;

							
							addLog("\n\n=== Facebook 게시글 등록. ===\n");
							addLog(String.format("SocialFacebook.postFacebook(content, listener) Called\ncontent = %s"
								, contentURL));
							
							com.hive.SocialFacebook.postFacebookWithContentURL(contentURL, new com.hive.SocialFacebook.PostFacebookListener() {

								@Override
								public void onPostFacebook(ResultAPI result) {

									addLog(String.format("SocialFacebook.PostFacebookListener.onPostFacebook() Callback\nresult = %s\n", result.toString()));
								}
							});
						}

					}
				}).build().show();

			}
			break;
			

// SocialGoogle
			
			case R.id.btnSocialGoogleGetMyProfile: {
				
				addLog("\n\n=== Google 프로필 정보 조회. ===\n");
				addLog("SocialGoogle.getMyProfile(listener) Called\n");
				
				com.hive.SocialGoogle.getMyProfile(new com.hive.SocialGoogle.SocialProfileGoogleListener() {

					@Override
					public void onProfile(ResultAPI result, ArrayList<SocialGoogle.ProfileGoogle> profileList) {

						SocialGoogle.ProfileGoogle profile = null;

						if (profileList != null && !profileList.isEmpty()
								&& profileList.get(0) != null) {

							profile = profileList.get(0);
						}

						addLog(String.format("SocialGoogle.ProfileListener.onProfile() Callback\nresult = %s\nprofile = %s\n"
							, result.toString(), profile != null ? profile.toString() : "null"));
					}
				});
			}
			break;
			
		}
	}
}



