package com.hive.sample.game;

import org.json.JSONArray;
import org.json.JSONObject;

import com.hive.Configuration;
import com.hive.Const;
import com.hive.IAPV4;
import com.hive.ResultAPI;
import com.hive.base.Android;
import com.hive.base.HttpClient;
import com.hive.base.HttpClient.HttpClientResponse;
import com.hive.base.HttpClient.HttpRequestListener;
import com.hive.base.Property;
import com.hive.impl.AuthV4Impl;
import com.hive.impl.ConfigurationImpl;
import com.hive.impl.IAPV4Impl;
import com.hive.impl.authv4.AuthV4Keys;

import android.graphics.Bitmap;
import android.os.Handler;
import android.os.Looper;

import java.lang.reflect.Array;
import java.util.ArrayList;

public class GameNetwork {


	public static void login(final IGameListener listener) {

		request(getUrl("account/login"), listener);
	}

	public static void userInfo(final IGameListener listener) {
		
		request(getUrl("account/user"), listener);
	}

	public static void posts(final IGameListener listener) {
		
		request(getUrl("postbox/posts"), listener);
	}


	public static void postReceive(IGameListener listener, PostBoxObject... posts) {
		
		request(getUrl("postbox/receive"), listener, posts);
	}

	public static void postReceive(ArrayList<Post> posts, IGameListener listener) {

		request(getUrl("postbox/receive"), listener, posts);
	}

	public static void receiptVerify(IAPV4.IAPV4Receipt iapv4Receipt, IGameListener listener, String itemType) {

		request(getUrl("hive/iap/verify"), listener, iapv4Receipt, itemType);
	}
	
	
	private static void request(String url, final IGameListener listener) {
		request(url, listener, new Post[0]);
	}
	private static void request(String url, final IGameListener listener, PostBoxObject... posts) {
		
		AuthV4Impl.AccountV4 accountV4 = AuthV4Impl.getAccountV4();

		String vid;
		String uid;
		String did;
		String lc_sessionkey;
		String vidType;

		if(accountV4 != null) {
			vid = accountV4.vid;
			uid = accountV4.uid;
			did = accountV4.did;
			lc_sessionkey = accountV4.accessToken;
			vidType = accountV4.vidType;
		}
		else {
			new Handler(Looper.getMainLooper()).post(new Runnable() {

				@Override
				public void run() {

					listener.onGameListener(false, null);
				}
			});
			return;
		}

		
		JSONObject params = new JSONObject();
        try {
        	params.put("session_token", GameUser.getInstance().sessionToken);
			params.put("world", Configuration.getServerId());
			params.put("vid", vid);
			params.put("did", did);
			params.put("uid", uid);
			params.put("appid", Configuration.getAppId());
			params.put("country", ConfigurationImpl.getInstance().getHiveCountry());
			params.put("language", ConfigurationImpl.getInstance().getHiveLanguage()); //샘플앱이라서 Config 에서 세팅한 값을 읽음
			params.put("appver", Android.getAppVersion(Configuration.getContext()));
			params.put("lc_sessionkey", lc_sessionkey);
			params.put("vid_type", vidType);
			
			if(posts != null && posts.length > 0) {
				JSONArray postIdList = new JSONArray();
				
				for (PostBoxObject post : posts) {
					if( post.getPostId() != null )
						postIdList.put(post.getPostId());
				}
				
				params.put("post_id_list", postIdList);
			}
			
        } catch (Exception e) {
        	e.printStackTrace();
        }
        
		HttpClient httpClient = new HttpClient(url);
        httpClient.requestHttp(params, new HttpRequestListener() {
			
			@Override
			public void onHttpRequest(ResultAPI resultApi, HttpClientResponse httpClientResponse) {
				
				boolean result = false;
				Response response = null;
				
				if(httpClientResponse != null) {
					
					try {
						result = true;
						response = new Response(httpClientResponse.content);
						
						GameUser.getInstance().reflesh(response);
						
					} catch (Exception ignore) {
						result = false;
						response = null;
					}
				}
				System.out.println("====Game Network Response : " + response);
				listener.onGameListener(result, response);
			}
		});
	}

	private static void request(String url, final IGameListener listener, ArrayList<Post> posts) {

		AuthV4Impl.AccountV4 accountV4 = AuthV4Impl.getAccountV4();

		String vid;
		String uid;
		String did;
		String lc_sessionkey;
		String vidType;

		if(accountV4 != null) {
			vid = accountV4.vid;
			uid = accountV4.uid;
			did = accountV4.did;
			lc_sessionkey = accountV4.accessToken;
			vidType = accountV4.vidType;
		}
		else {
			new Handler(Looper.getMainLooper()).post(new Runnable() {

				@Override
				public void run() {

					listener.onGameListener(false, null);
				}
			});
			return;
		}


		JSONObject params = new JSONObject();
		try {
			params.put("session_token", GameUser.getInstance().sessionToken);
			params.put("world", Configuration.getServerId());
			params.put("vid", vid);
			params.put("did", did);
			params.put("uid", uid);
			params.put("appid", Configuration.getAppId());
			params.put("country", ConfigurationImpl.getInstance().getHiveCountry());
			params.put("language", ConfigurationImpl.getInstance().getHiveLanguage()); //샘플앱이라서 Config 에서 세팅한 값을 읽음
			params.put("appver", Android.getAppVersion(Configuration.getContext()));
			params.put("lc_sessionkey", lc_sessionkey);
			params.put("vid_type", vidType);

			if(posts != null && posts.size() > 0) {
				JSONArray postIdList = new JSONArray();

				for (Post post : posts) {
					postIdList.put(post.postId);
				}

				params.put("post_id_list", postIdList);
			}

		} catch (Exception e) {
			e.printStackTrace();
		}

		HttpClient httpClient = new HttpClient(url);
		httpClient.requestHttp(params, new HttpRequestListener() {

			@Override
			public void onHttpRequest(ResultAPI resultApi, HttpClientResponse httpClientResponse) {

				boolean result = false;
				Response response = null;

				if(httpClientResponse != null) {

					try {
						result = true;
						response = new Response(httpClientResponse.content);

						GameUser.getInstance().reflesh(response);

					} catch (Exception ignore) {
						result = false;
						response = null;
					}
				}
				System.out.println("====Game Network Response : " + response);
				listener.onGameListener(result, response);
			}
		});
	}

	private static void request(String url, final IGameListener listener, IAPV4.IAPV4Receipt iapv4Receipt, String itemType) {

		AuthV4Impl.AccountV4 accountV4 = AuthV4Impl.getAccountV4();

		String vid;
		String uid;
		String did;
		String lc_sessionkey;
		String vidType;

		if(accountV4 != null) {
			vid = accountV4.vid;
			uid = accountV4.uid;
			did = accountV4.did;
			lc_sessionkey = accountV4.accessToken;
			vidType = accountV4.vidType;
		}
		else {
			new Handler(Looper.getMainLooper()).post(new Runnable() {

				@Override
				public void run() {

					listener.onGameListener(false, null);
				}
			});
			return;
		}


		JSONObject params = new JSONObject();
		try {

			params.put("session_token", GameUser.getInstance().sessionToken);
			params.put("world", Configuration.getServerId());
			params.put("vid", vid);
			params.put("purchase_bypass_info", iapv4Receipt.bypassInfo);
			params.put("game_info", iapv4Receipt.additionalInfo);
			params.put("item_type", itemType);


		} catch (Exception e) {
			e.printStackTrace();
		}

		HttpClient httpClient = new HttpClient(url);
		httpClient.requestHttp(params, new HttpRequestListener() {

			@Override
			public void onHttpRequest(ResultAPI resultApi, HttpClientResponse httpClientResponse) {

				boolean result = false;
				Response response = null;

				if(httpClientResponse != null) {

					try {
						result = true;
						response = new Response(httpClientResponse.content);

						GameUser.getInstance().reflesh(response);

					} catch (Exception ignore) {
						result = false;
						response = null;
					}
				}
				System.out.println("====Game Network Response : " + response);
				listener.onGameListener(result, response);
			}
		});
	}

	
	private static String getUrl(String value) {

		StringBuffer buffer = new StringBuffer();
		buffer.append("http://");
		
		switch(Configuration.getZone()) {
		
		case REAL:
			break;
			
		case SANDBOX:
			buffer.append("sandbox-");
			break;
			
		case TEST:
			buffer.append("test-");
			break;
		case DEV:
			buffer.append("dev-");
			break;
		}

		buffer.append("misample.com2us.net/").append(value);
		
		return buffer.toString();
	}
}
