package com.hive.sample.uihelper;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLSession;

import android.content.Context;
import android.util.Log;


/**
 * @author hife
 */
class HiveImageWorker {
	
	private Context context;
	private boolean isIdle = true;
	private int hashCode;
	
	HiveImageWorker(Context context) {
		this.context = context;
		hashCode = this.hashCode();
	}

	synchronized void work() {
		Log.v("HiveIAPImageLoader", "[HiveIAPImageWorker](" + hashCode +") work!");
		
		if(!isIdle) {
			Log.d("HiveIAPImageLoader", "[HiveIAPImageWorker](" + hashCode +") is working");
			return;
		}

		final HiveImageLoader loader = HiveImageLoader.getInstance(context);
		
		isIdle = false;
		Log.d("HiveIAPImageLoader", "[HiveIAPImageWorker](" + hashCode +") worker state: working");
		
		new Thread(new Runnable() {
			
			@Override
			public void run() {
				
				while(loader.peekTaskQueue() != null) {
					final HiveImageRequestData requestData = loader.pollTaskQueue();
					Log.d("HiveIAPImageLoader", "[HiveIAPImageLoader](" + hashCode +") pollTaskQueue");
					
					if(requestData == null) continue;
					
					//find data in cache (read cache)
					final byte[] data = loader.readCache(requestData.getUrl());
					if(data != null) {
						Log.d("HiveIAPImageLoader", "[HiveIAPImageWorker](" + hashCode +") cache data exist: " + requestData.getUrl());
						
						requestData.getHandler().post(new Runnable() {
							@Override
							public void run() {
								Log.v("HiveIAPImageLoader", "[HiveIAPImageWorker](" + hashCode +") onLoadingComplete: " + requestData.getUrl());
								requestData.getListener().onLoadingComplete(requestData.getUrl(), data);
							}
						});
						
					} else {
						//network download
						byte[] newData = null;
						String failReason = null;
						
						try {
							newData = getByteFromUrl(requestData.getUrl());
						} catch (IllegalArgumentException e) {
							failReason = e.toString();
						} catch (IOException e) {
							failReason = e.toString();
						} catch (Exception e) {
							failReason = e.toString();
						}
						
//						HttpClient httpClient = new HttpClient(requestData.getUrl());
//						HttpClientResponse response = httpClient.requestHttp("", null);
//						
//						if(response != null && response.resultApi.isSuccess())
//							newData = response.content.getBytes();
//						
//						else
//							failReason = response.resultApi.message;
						
						
						final byte[] sendData = newData;
						final String sendFailReason = failReason;
						if(sendData != null) {
							//write cache
							loader.writeCache(requestData.getUrl(), sendData);
							
							//send to callback
							requestData.getHandler().post(new Runnable() {
								@Override
								public void run() {
									Log.v("HiveIAPImageLoader", "[HiveIAPImageWorker](" + hashCode +") onLoadingComplete: " + requestData.getUrl());
									requestData.getListener().onLoadingComplete(requestData.getUrl(), sendData);
								}
							});
							
						} else {
							requestData.getHandler().post(new Runnable() {
								@Override
								public void run() {
									Log.w("HiveIAPImageLoader", "[HiveIAPImageWorker](" + hashCode +") onLoadingFailed: " + sendFailReason + ", url: " + requestData.getUrl());
									requestData.getListener().onLoadingFailed(requestData.getUrl(), sendFailReason);
								}
							});
						}
					}
					//next queue poll
				}
				
				isIdle = true;
				Log.d("HiveIAPImageLoader", "[HiveIAPImageWorker](" + hashCode +") worker state: idle");
			}
			
		}).start();
	}
	
	/**
	 * Issue a POST request to the server.
	 *
	 * @param endpoint
	 *            POST address.
	 *
	 * @throws IOException, IllegalArgumentException
	 *             propagated from POST.
	 */
	private byte[] getByteFromUrl(String endpoint) throws IOException, IllegalArgumentException, Exception {
		
		URL url;
		try {
			url = new URL(endpoint);
		} catch (MalformedURLException e) {
			Log.w("HiveIAPImageLoader", "[HiveIAPImageWorker](" + hashCode +") invalid url: " + endpoint);
			throw new IllegalArgumentException("[HiveIAPImageWorker](" + hashCode +") invalid url: " + endpoint);
		}
		
		Log.v("HiveIAPImageLoader", "[HiveIAPImageLoader] request to " + url);

		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		InputStream is = null;
		HttpURLConnection conn = null;
		try {
			conn = (HttpURLConnection) url.openConnection();
			if(conn instanceof HttpsURLConnection) {
				HttpsURLConnection conns = (HttpsURLConnection) conn;
				conns.setHostnameVerifier(new HostnameVerifier() {
					
					@Override
					public boolean verify(String hostname, SSLSession session) {
						Log.d("HiveIAPImageLoader", "[HiveIAPImageWorker](" + hashCode +") hostname: " + hostname);
						Log.d("HiveIAPImageLoader", "[HiveIAPImageWorker](" + hashCode +") session: " + session.isValid());
						return session.isValid();
					}
				});
			}
//			conn.setDoOutput(true);
//			conn.setUseCaches(false);
//			conn.setFixedLengthStreamingMode(bytes.length);
//			conn.setRequestMethod("POST");
//			conn.setRequestProperty("Content-Type", "application/x-www-form-urlencoded;charset=UTF-8");
//			conn.setRequestProperty("Content-Type", "application/json");
//			conn.setRequestProperty("Content-Type", "text/html");
//			conn.setRequestProperty("Accept", "application/json");
//			conn.setRequestProperty("Accept", "text/html");
			conn.setConnectTimeout(com.hive.Configuration.getHttpConnectTimeout() * 1000);
			conn.setReadTimeout(com.hive.Configuration.getHttpReadTimeout() * 1000);
			
			// handle the response
//			int status = conn.getResponseCode();
			String message = conn.getResponseMessage();
			Log.i("HiveIAPImageLoader", "[HiveIAPImageWorker](" + hashCode +") Response Message : " + message);
			
//			if (status != 200) {
//				throw new IOException("[HiveIAPImageWorker] getByteFromUrl failed with error code " + status);
//			}
			
			is = conn.getInputStream();
			byte buffer[] = new byte[8096];
			int readLen;
			while( (readLen = is.read(buffer))>0) {
	            baos.write(buffer, 0, readLen);
	        }
			byte[] response = baos.toByteArray();
			Log.i("HiveIAPImageLoader", "[HiveIAPImageWorker](" + hashCode +") Response data success");
			
			return response;
			
		} catch (Exception e) {
			Log.w("HiveIAPImageLoader", "[HiveIAPImageWorker](" + hashCode +") getByteFromUrl failed with exception: " + e.toString());
			throw new Exception("[HiveIAPImageWorker](" + hashCode +") getByteFromUrl failed with exception: " + e.toString());
			
		} finally {
			if(is != null) try{is.close();}catch(IOException ignore){};
			if(baos!=null) try{baos.close();}catch(IOException ignore){};
			if(conn != null) try{conn.disconnect();}catch(Exception ignore){};
		}
	}
}
